(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    506086,      13679]
NotebookOptionsPosition[    489144,      13184]
NotebookOutlinePosition[    489481,      13199]
CellTagsIndexPosition[    489438,      13196]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["General Functions", "Section",
 CellChangeTimes->{{3.681473305392375*^9, 3.681473317638763*^9}}],

Cell[TextData[{
 StyleBox["This file was created with ",
  FontWeight->"Bold"],
 StyleBox["Mathematica",
  FontWeight->"Bold",
  FontSlant->"Italic"],
 StyleBox["/10.2\n\nPrior to analysis you must :",
  FontWeight->"Bold"],
 "\n(1) Run the simulations (see versatile_framework_paper/README.md)\n(2) \
Postprocess: \nIn most cases this means converting the human-readable-output \
of the simulation to something easier for mathematica to read. My solution \
was a \[OpenCurlyDoubleQuote]dat\[CloseCurlyDoubleQuote] file. The python \
script ",
 StyleBox["versatile_framework_paper/python/txt2dat.py",
  FontWeight->"Bold"],
 " converts the txt files from the simulation output to dat files that \
mathematica can import easily using the ",
 StyleBox["pts ",
  FontWeight->"Bold"],
 "function, below.\nExample Usage:\n",
 StyleBox["python versatile_framework_paper/python/txt2dat.py test actins",
  FontWeight->"Bold"],
 "\ngenerates the file test/txt_stack/actins.dat from the file \
test/txt_stack/actins.txt. \nFor Cytosim trajectories, \
there\[CloseCurlyQuote]s a similar command\n",
 StyleBox["python versatile_framework_paper/python/cs_txt2dat.py test \n",
  FontWeight->"Bold"],
 "to produce the \[OpenCurlyDoubleQuote]objects.dat\[CloseCurlyDoubleQuote] \
file\nFor most of the analyses below, this must be done for the links file \
and sometimes the actins file. \n\nFor figures 2, 3D, 7C-D and S3 there are \
separate pre/post processing requirements; see there for details. \n"
}], "Text",
 CellChangeTimes->{{3.681659222291408*^9, 3.681659584956533*^9}, {
   3.681660355002295*^9, 3.6816603610030527`*^9}, {3.68166084199124*^9, 
   3.6816609247031393`*^9}, {3.694174831532711*^9, 3.694174832372414*^9}, 
   3.694811307141963*^9, {3.694953992692346*^9, 3.6949540360311193`*^9}, 
   3.702033177939393*^9, {3.702662836033036*^9, 3.702662837728963*^9}, {
   3.706958769407761*^9, 3.7069587703437357`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "Import", "[", 
   "\"\<~/Code/cytomod/versatile_framework_paper/mathematica/cytomod_\
functions.m\>\"", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.692670090652869*^9, 3.692670109602195*^9}, {
  3.704113167570312*^9, 3.704113224576852*^9}, {3.7045592623012953`*^9, 
  3.704559271555539*^9}, {3.704559307779113*^9, 3.7045593108983803`*^9}}],

Cell[CellGroupData[{

Cell["Directory Setup", "Subsection",
 CellChangeTimes->{{3.6817612584675627`*^9, 3.681761268380281*^9}, {
  3.6949688244592867`*^9, 3.694968828907257*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
  "outdir", "=", "\"\</project/weare-dinner/simonfreedman/cytomod/out\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vfdir", "=", 
   RowBox[{"outdir", "<>", "\"\</vf_r2\>\""}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"figdir", "=", 
   RowBox[{"vfdir", "<>", "\"\</figs\>\""}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1dir", "=", 
   RowBox[{"outdir", "<>", "\"\</minimal_actomyosin/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig2dir", "=", 
   RowBox[{"vfdir", "<>", "\"\</profiling/koff1_kend10/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig3dir", "=", 
   RowBox[{"vfdir", "<>", "\"\</pl/\>\""}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "fig3Cdir", "=", 
   "\"\</scratch/midway2/simonfreedman/cytomod/out/vf_r2/3C/\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig5dir", "=", 
   RowBox[{"vfdir", "<>", "\"\</shear/shear_xy75_koff0.1/kf1000/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fig6dir", "=", 
    RowBox[{"vfdir", "<>", "\"\</testing/afines_low_rc\>\""}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig7dir", "=", 
   RowBox[{
   "vfdir", "<>", "\"\</contracting/mdens0.2_xldens1/mkoff0.1_mkend1\>\""}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"figS3dir", "=", 
    RowBox[{"vfdir", "<>", "\"\</contracting/mdens1_xldens1\>\""}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"figS2dir", "=", 
    RowBox[{"vfdir", "<>", "\"\</shear/nbw_arr\>\""}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "csoutdir", "=", 
   "\"\</project/weare-dinner/simonfreedman/cytosim/out\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"figS4dir", "=", 
   RowBox[{"csoutdir", "<>", "\"\</pl\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"figS5dir", "=", 
   RowBox[{"csoutdir", "<>", "\"\</vf_r2/motility\>\""}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.681473349544886*^9, 3.681473556112371*^9}, 
   3.681476219578808*^9, 3.681498532807477*^9, {3.681641179932774*^9, 
   3.6816411858261757`*^9}, {3.6899593270904818`*^9, 3.689959329304768*^9}, {
   3.689959601241787*^9, 3.6899596614999437`*^9}, {3.689959700279179*^9, 
   3.6899597018071623`*^9}, {3.689961199057282*^9, 3.689961213585288*^9}, {
   3.689961262727428*^9, 3.689961264599613*^9}, {3.6899615659458923`*^9, 
   3.689961566553389*^9}, {3.689971180473173*^9, 3.689971181669218*^9}, 
   3.689971274219899*^9, {3.690160325782806*^9, 3.690160341350134*^9}, {
   3.6901606916248827`*^9, 3.690160703047408*^9}, {3.6906510812862787`*^9, 
   3.690651090124456*^9}, {3.690651424011907*^9, 3.690651449084846*^9}, {
   3.690812296616352*^9, 3.6908122979440947`*^9}, {3.690812341279305*^9, 
   3.690812344071602*^9}, {3.69088515979541*^9, 3.690885160394435*^9}, {
   3.6909021327596197`*^9, 3.690902132914939*^9}, {3.6911556026797943`*^9, 
   3.691155602862464*^9}, {3.691155664285757*^9, 3.6911556643727493`*^9}, {
   3.694825754588704*^9, 3.6948257613498096`*^9}, {3.694889638627862*^9, 
   3.6948896752371798`*^9}, {3.694964906150382*^9, 3.694964940126052*^9}, 
   3.694968833709598*^9, 3.694969040343054*^9, {3.701788083677425*^9, 
   3.701788116410138*^9}, 3.7017882100014677`*^9, {3.7020591068620367`*^9, 
   3.7020591069537897`*^9}, {3.702319748955813*^9, 3.702319781588395*^9}, {
   3.70277206024437*^9, 3.702772063149825*^9}, {3.702908509864789*^9, 
   3.702908517160088*^9}, {3.703251988257679*^9, 3.703251993944016*^9}, {
   3.703514702833995*^9, 3.703514705005673*^9}, {3.7041151347534103`*^9, 
   3.70411515429786*^9}, {3.70411674637719*^9, 3.704116746641876*^9}, {
   3.704116794144679*^9, 3.704116805352098*^9}, 3.704116983028138*^9, {
   3.704566151242791*^9, 3.704566152882312*^9}, {3.704646623219447*^9, 
   3.704646644016918*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Figure 1", "Section",
 CellChangeTimes->{{3.681474515871564*^9, 3.681474531413069*^9}}],

Cell[CellGroupData[{

Cell["D", "Subsection",
 CellChangeTimes->{{3.681474802912915*^9, 3.681474809174946*^9}}],

Cell["\<\
Requires first running 
\ttxt2dat.py [fig1dir <> \"/slide/\"] links
\ttxt2dat.py [fig1dir <> \"/slide/\"] amotors\
\>", "Text",
 CellChangeTimes->{{3.681474989734313*^9, 3.681475117090761*^9}, {
  3.681475585030953*^9, 3.681475591926611*^9}, {3.69495490801373*^9, 
  3.6949549090530863`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig1dir", "<>", "\"\</slide\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<links\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"amots", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<amotors\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fs", "=", 
   RowBox[{"draw", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "lks", ",", "amots", ",", 
     RowBox[{"{", "}"}], ",", "dir", ",", "False", ",", 
     RowBox[{"{", 
      RowBox[{"20", ",", "5"}], "}"}], ",", "True", ",", " ", "21", ",", 
     "Red", ",", "\"\<movie\>\"", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"GraphicsColumn", "[", 
  RowBox[{
   RowBox[{"fs", "[", 
    RowBox[{"[", 
     RowBox[{"{", 
      RowBox[{"1", ",", "31", ",", "101"}], "}"}], "]"}], "]"}], ",", 
   RowBox[{"Spacings", "\[Rule]", "10"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.670165926937613*^9, 3.6701659321620903`*^9}, {
   3.670166012363283*^9, 3.670166014287451*^9}, {3.6701661748971243`*^9, 
   3.6701661757503147`*^9}, {3.670166240834035*^9, 3.6701662779544897`*^9}, {
   3.67016653575146*^9, 3.670166542328537*^9}, {3.68147490995327*^9, 
   3.681474981082556*^9}, {3.681475131618991*^9, 3.681475231888111*^9}, {
   3.681475304862589*^9, 3.681475342174045*^9}, {3.681475561881065*^9, 
   3.6814755620722723`*^9}, {3.681476239650838*^9, 3.681476240234116*^9}, {
   3.681476283136438*^9, 3.681476306438661*^9}, 3.704134030412121*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["E", "Subsection",
 CellChangeTimes->{{3.681474811267931*^9, 3.6814748186149387`*^9}}],

Cell["\<\
Requires first running 
\ttxt2dat.py [fig1dir <> \"/buckle/\"] links
\ttxt2dat.py [fig1dir <> \"/buckle/\"] amotors
\ttxt2dat.py [fig1dir <> \[OpenCurlyDoubleQuote]/buckle/\
\[CloseCurlyDoubleQuote]] pmotors\
\>", "Text",
 CellChangeTimes->{{3.681474989734313*^9, 3.681475117090761*^9}, {
  3.681475531224627*^9, 3.68147553843192*^9}, {3.681475570975587*^9, 
  3.6814756123102417`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig1dir", "<>", "\"\</buckle\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<links\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"amots", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<amotors\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pmots", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<pmotors\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fs", "=", 
   RowBox[{"draw", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "lks", ",", "amots", ",", "pmots", ",", "dir", 
     ",", "False", ",", 
     RowBox[{"{", 
      RowBox[{"20", ",", "5"}], "}"}], ",", "True", ",", " ", "21", ",", 
     "Red", ",", "\"\<movie\>\"", ",", 
     RowBox[{"-", "1"}], ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"GraphicsColumn", "[", 
  RowBox[{
   RowBox[{"fs", "[", 
    RowBox[{"[", 
     RowBox[{"{", 
      RowBox[{"1", ",", "31", ",", "101"}], "}"}], "]"}], "]"}], ",", 
   RowBox[{"Spacings", "\[Rule]", "10"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.670165926937613*^9, 3.6701659321620903`*^9}, {
  3.670166012363283*^9, 3.670166014287451*^9}, {3.6701661748971243`*^9, 
  3.6701661757503147`*^9}, {3.670166240834035*^9, 3.6701662779544897`*^9}, {
  3.67016653575146*^9, 3.670166542328537*^9}, {3.68147490995327*^9, 
  3.681474981082556*^9}, {3.681475131618991*^9, 3.681475231888111*^9}, {
  3.681475304862589*^9, 3.681475354565733*^9}, {3.681475549808961*^9, 
  3.681475557752612*^9}, {3.6814763184610023`*^9, 3.6814763255466003`*^9}, {
  3.704134052955875*^9, 3.7041340530980463`*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Figure 2", "Section",
 CellChangeTimes->{{3.681474533401927*^9, 3.681474539670686*^9}, {
  3.6814753629647617`*^9, 3.681475365028473*^9}}],

Cell["\<\
For these plots, you have to measure the wall clock time of the simulation. I \
did this with the following Bash script, where $cmd was my simulation command \
(see versatile_framework_paper/README.md) and $data_dr was the \
\[OpenCurlyDoubleQuote]data\[CloseCurlyDoubleQuote] output directory of the \
simulation

startt = `date + % s`
./$cmd
endt = `date + % s`
tott = $ (echo $startt $endt | awk' {printf \"%d\", $2 - $1}')
echo $tott > \"$data_dr/time.txt\"\
\>", "Text",
 CellChangeTimes->{{3.681475779711831*^9, 3.68147589346342*^9}, {
  3.6814759839177933`*^9, 3.681476023647291*^9}, {3.681476061276106*^9, 
  3.681476067725877*^9}, {3.681476109250834*^9, 3.681476112786581*^9}, {
  3.694825059951573*^9, 3.694825078541663*^9}, {3.6949549453982697`*^9, 
  3.694954946197813*^9}}],

Cell[CellGroupData[{

Cell["A", "Subsection",
 CellChangeTimes->{{3.6814754228005953`*^9, 3.681475440237659*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nfil0", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nm0", "=", "11"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]0", "=", "0.1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"area", "=", 
   RowBox[{"50", "^", "2"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.690120276681992*^9, 3.690120359298409*^9}}],

Cell[CellGroupData[{

Cell["Red Curve", "Subsubsection",
 CellChangeTimes->{{3.681476367285337*^9, 3.681476376771762*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig2dir", "<>", "\"\</2A/npoly_arr\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ns", "=", 
   RowBox[{"Sort", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{"Outer", "[", 
      RowBox[{"Times", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", 
       RowBox[{"10", "^", 
        RowBox[{"Range", "[", 
         RowBox[{"0", ",", "3"}], "]"}]}]}], "]"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"dir", "<>", "\"\</dens\>\"", "<>", 
      RowBox[{"ToString", "[", "n", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "ns"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"times", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</data/time.txt\>\""}], ",", "\"\<TSV\>\""}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"nparts", "=", 
    RowBox[{
     RowBox[{"ns", " ", "nm0"}], "+", 
     RowBox[{"4", "\[Rho]0", " ", "area"}]}]}], ";"}], 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"in", " ", "paper"}], ",", " ", 
    RowBox[{"constant", " ", "addition", " ", "was", " ", 
     RowBox[{"off", " ", "--"}], " ", "4", "*", "0.1", "*", 
     RowBox[{"75", "^", "2"}], "\"\<\>"}]}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reddat", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"nparts", ",", "times"}], "}"}], "\[Transpose]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.681476386134028*^9, 3.68147642842765*^9}, {
   3.681476486514718*^9, 3.6814764966742153`*^9}, {3.6814766009123077`*^9, 
   3.681476633750849*^9}, {3.681476685413859*^9, 3.6814767949555902`*^9}, {
   3.681477737424013*^9, 3.6814777749229813`*^9}, {3.681478244311326*^9, 
   3.6814782469190397`*^9}, {3.681478327493196*^9, 3.681478336484725*^9}, {
   3.681478380379526*^9, 3.681478401787073*^9}, {3.6899604145520277`*^9, 
   3.6899604512336683`*^9}, {3.689960630577012*^9, 3.6899606557434683`*^9}, 
   3.689961015071436*^9, {3.690032715822194*^9, 3.690032716172482*^9}, {
   3.6900327671722403`*^9, 3.690032772778667*^9}, {3.690032860249407*^9, 
   3.690032863264776*^9}, {3.690120029115752*^9, 3.690120030164071*^9}, {
   3.6901200881790323`*^9, 3.6901200952655287`*^9}, {3.6901203160449266`*^9, 
   3.690120371889987*^9}, {3.6901204067223063`*^9, 3.690120407961038*^9}, {
   3.7027720379040747`*^9, 3.702772042931501*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Blue Curve", "Subsubsection",
 CellChangeTimes->{{3.681476875108006*^9, 3.681476898583172*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig2dir", "<>", "\"\</2A/mdens_arr\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ns", "=", 
   RowBox[{"Sort", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{"Outer", "[", 
      RowBox[{"Times", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", 
       RowBox[{"10", "^", 
        RowBox[{"Range", "[", 
         RowBox[{"0", ",", "4"}], "]"}]}]}], "]"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"denss", "=", 
   RowBox[{"ns", "*", "0.001"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"dir", "<>", "\"\</dens\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"d", ",", 
         RowBox[{"{", 
          RowBox[{"3", ",", "3"}], "}"}]}], "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"d", ",", "denss"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"times", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</data/time.txt\>\""}], ",", "\"\<TSV\>\""}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nparts", "=", 
   RowBox[{
    RowBox[{"nfil0", " ", "nm0"}], "+", 
    RowBox[{"2", "\[Rho]0", " ", "area"}], " ", "+", " ", 
    RowBox[{"2", " ", "denss", " ", "area"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bluedat", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"nparts", ",", "times"}], "}"}], "\[Transpose]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.681476912656404*^9, 3.6814769761110353`*^9}, {
   3.681477692213187*^9, 3.681477693853415*^9}, {3.681478426314803*^9, 
   3.6814784418738737`*^9}, {3.681478509828867*^9, 3.681478514079924*^9}, {
   3.681478770762281*^9, 3.681478790873348*^9}, {3.681478825424696*^9, 
   3.681478829128394*^9}, {3.6814790486343412`*^9, 3.68147904953022*^9}, 
   3.689960842428093*^9, 3.690033108890318*^9, {3.690033885798409*^9, 
   3.6900338861664267`*^9}, {3.690120108865387*^9, 3.690120109304825*^9}, {
   3.690120143817687*^9, 3.690120150536376*^9}, {3.6901201895197067`*^9, 
   3.6901201909267693`*^9}, {3.6901203861547832`*^9, 3.690120442689094*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Black Curve", "Subsubsection",
 CellChangeTimes->{{3.681478715429463*^9, 3.681478728674872*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig2dir", "<>", "\"\</2A/dens_arr\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ns", "=", 
   RowBox[{"Sort", "[", 
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"Flatten", "[", 
       RowBox[{"Outer", "[", 
        RowBox[{"Times", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", 
         RowBox[{"10", "^", 
          RowBox[{"Range", "[", 
           RowBox[{"0", ",", "2"}], "]"}]}]}], "]"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"1000", ",", "2000"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"dir", "<>", "\"\</dens\>\"", "<>", 
      RowBox[{"ToString", "[", "n", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "ns"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"times", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</data/time.txt\>\""}], ",", "\"\<TSV\>\""}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nparts", "=", 
   RowBox[{
    RowBox[{"ns", " ", "nm0"}], "+", 
    RowBox[{"4", " ", "\[Rho]0", " ", 
     RowBox[{"ns", "/", "nfil0"}], " ", "area"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"blackdat", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"nparts", ",", "times"}], "}"}], "\[Transpose]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{
  3.681478739675144*^9, {3.681478856655671*^9, 3.6814788600151253`*^9}, 
   3.681478948149765*^9, {3.681479003939927*^9, 3.681479099192231*^9}, {
   3.68147930005154*^9, 3.681479301274852*^9}, 3.68996092873757*^9, 
   3.69003389336626*^9, {3.690120207374202*^9, 3.690120258598753*^9}, {
   3.690120459112686*^9, 3.6901205079185057`*^9}, {3.702903243861994*^9, 
   3.702903267973051*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ticksb", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", "2", ",", "6"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tickst", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", "2", ",", "6", ",", 
     RowBox[{"ShowTickLabels", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ticksl", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", "0", ",", "10"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ticksr", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", "0", ",", "10", ",", 
     RowBox[{"ShowTickLabels", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.676130679605392*^9, 3.676130764924192*^9}, {
  3.6761308284283123`*^9, 3.676130846403639*^9}, {3.6761309081464157`*^9, 
  3.676130910624511*^9}, {3.676131055046412*^9, 3.676131074564437*^9}, {
  3.676131105821548*^9, 3.67613112162722*^9}, {3.676131176236372*^9, 
  3.676131220272684*^9}, {3.681479405784541*^9, 3.6814794202632217`*^9}, {
  3.681479467839087*^9, 3.681479469693809*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"npartplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Log10", "[", 
          RowBox[{"reddat", "[", 
           RowBox[{"[", 
            RowBox[{"4", ";;"}], "]"}], "]"}], "]"}], ",", 
         RowBox[{"Log10", "[", 
          RowBox[{"bluedat", "[", 
           RowBox[{"[", 
            RowBox[{"6", ";;"}], "]"}], "]"}], "]"}], ",", 
         RowBox[{"Log10", "[", 
          RowBox[{"blackdat", "[", 
           RowBox[{"[", 
            RowBox[{"4", ";;"}], "]"}], "]"}], "]"}]}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "@", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Rectangle", "[", "]"}]}], "}"}]}], ",", "0.05"}], 
           "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "@", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Rotate", "[", 
                RowBox[{
                 RowBox[{"Rectangle", "[", "]"}], ",", 
                 RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}]}], ",", "0.06"}], 
           "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "@", 
             RowBox[{"{", 
              RowBox[{"Black", ",", 
               RowBox[{"Disk", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}]}], 
         "}"}]}], 
       RowBox[{"(*", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "15"}], "}"}], "*)"}], 
       RowBox[{"(*", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"\[FilledSquare]", ",", "15"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[FilledUpTriangle]", ",", "15"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "10"}], "}"}]}], "}"}], "*)"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Red", ",", "Blue", ",", "Black"}], "}"}]}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"ticksl", ",", "ticksr"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ticksb", ",", "tickst"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"2.7", ",", "5.55"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "5.2"}], "}"}]}], "}"}]}], ",", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Number of Particles\>\"", ",", "\"\<Wall Clock Time (s)\>\""}], 
         "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"Placed", "[", 
         RowBox[{
          RowBox[{"PointLegend", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
             "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(motor\)]\) const.\>\"", 
              ",", "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(actin\)]\) \
const.\>\""}], "}"}], ",", 
            RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"FontSize", "\[Rule]", "18"}]}], ",", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"LegendMarkerSize", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"13", ",", "16"}], "}"}]}], ","}], "*)"}], 
            RowBox[{"Spacings", "\[Rule]", "0.1"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"0.7", ",", "0.2"}], "}"}]}], "]"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"x", "-", "2.3"}], ",", 
         RowBox[{
          RowBox[{"2", "x"}], "-", "4.7"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "1", ",", "7"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Blue", ",", "Thick", ",", "Dashed"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"Black", ",", "Thick", ",", "Dashed"}], "}"}]}], "}"}]}]}],
       "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.676128283329194*^9, 3.6761283877100573`*^9}, {
   3.676128429761056*^9, 3.6761284711606283`*^9}, {3.676128510751542*^9, 
   3.6761285644845047`*^9}, {3.6761287675209417`*^9, 3.676128794198489*^9}, {
   3.6761292738519382`*^9, 3.6761293006099854`*^9}, {3.676129522102024*^9, 
   3.676129709151939*^9}, {3.676130569915503*^9, 3.676130575034172*^9}, {
   3.676130744118842*^9, 3.676130752829135*^9}, {3.676130784670869*^9, 
   3.6761308155478277`*^9}, {3.6761308537892437`*^9, 3.676130894034782*^9}, {
   3.67613103036864*^9, 3.676131030476986*^9}, 3.676131091438188*^9, {
   3.676131195787313*^9, 3.6761312882851954`*^9}, {3.676131320329096*^9, 
   3.676131521387967*^9}, {3.6761318305384083`*^9, 3.676131863226449*^9}, {
   3.67613288329213*^9, 3.676132924254941*^9}, {3.676132962670244*^9, 
   3.676132964365451*^9}, {3.676133088856654*^9, 3.6761331679024963`*^9}, {
   3.6761332228814917`*^9, 3.6761333258292522`*^9}, {3.676137037712901*^9, 
   3.676137048678743*^9}, 3.676137442206388*^9, {3.6761997915628967`*^9, 
   3.676199829858016*^9}, 3.676199984919189*^9, {3.676200241121209*^9, 
   3.67620024165924*^9}, {3.676202339155657*^9, 3.6762026407129908`*^9}, {
   3.676202700635233*^9, 3.67620275581758*^9}, {3.676202892996317*^9, 
   3.6762029155243177`*^9}, {3.6814793133400593`*^9, 3.68147932666695*^9}, {
   3.681482255492866*^9, 3.681482289618526*^9}, {3.681482332634503*^9, 
   3.681482446573894*^9}, 3.690032835149521*^9, 3.6900331522262783`*^9, {
   3.6900331880815287`*^9, 3.6900331881780767`*^9}, {3.690033219584766*^9, 
   3.690033286294039*^9}, {3.690033325761455*^9, 3.6900333547766037`*^9}, {
   3.690033385420825*^9, 3.690033438770608*^9}, {3.6901205543638153`*^9, 
   3.690120567345052*^9}, {3.6901209340386257`*^9, 3.690121033730053*^9}, {
   3.690121065298997*^9, 3.690121071505611*^9}, {3.6901213460755444`*^9, 
   3.690121406266819*^9}, {3.69012153858393*^9, 3.690121545637739*^9}, {
   3.70689481706734*^9, 3.706894871699085*^9}, {3.706894930776389*^9, 
   3.706894953676257*^9}, {3.70689498438375*^9, 3.706895013303618*^9}, {
   3.706895526033318*^9, 3.7068956236305923`*^9}, {3.7068957213328657`*^9, 
   3.706895871504738*^9}, {3.7068959025602083`*^9, 3.706895926449387*^9}, {
   3.706895962495061*^9, 3.706895988328813*^9}, {3.706896041564953*^9, 
   3.706896122448619*^9}, {3.707488457042803*^9, 3.7074884580975847`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</2A.eps\>\""}], ",", "npartplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.706896514230826*^9, 3.7068965372537518`*^9}}],

Cell[BoxData[" "], "Input",
 CellChangeTimes->{3.706894974901039*^9}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsection",
 CellChangeTimes->{{3.68147542498995*^9, 3.681475440238608*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ticksl", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", "0", ",", "4"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ticksr", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", "0", ",", "4", ",", 
     RowBox[{"ShowTickLabels", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"yrng", "=", 
   RowBox[{"{", 
    RowBox[{"0.1", ",", "3.2"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"flabel", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<System Size (\!\(\*SuperscriptBox[\(\[Mu]m\), \(2\)]\)) (\
\[FilledCircle])\>\"", ",", "\"\<Wall Clock Time (s)\>\"", ",", 
     "\"\<Grid Density (\!\(\*SuperscriptBox[\(\[Mu]m\), \(-2\)]\)) (\
\[EmptyCircle])\>\""}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.704134198904769*^9, 3.704134201030349*^9}, {
  3.7041342877914047`*^9, 3.704134302148431*^9}, {3.704134379754178*^9, 
  3.7041343799414253`*^9}, {3.704134447954443*^9, 3.704134450399783*^9}, {
  3.706896826887587*^9, 3.706896887558042*^9}, {3.706924033886236*^9, 
  3.706924040424102*^9}}],

Cell[CellGroupData[{

Cell["blue curve", "Subsubsection",
 CellChangeTimes->{{3.7041340892933893`*^9, 3.704134109037644*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig2dir", "<>", "\"\</sys_size_arr\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ns", "=", 
   RowBox[{"10", 
    RowBox[{"Range", "[", "10", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"dir", "<>", "\"\</sz\>\"", "<>", 
      RowBox[{"ToString", "[", "n", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "ns"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"times", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</data/time.txt\>\""}], ",", "\"\<TSV\>\""}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"szs", "=", 
   RowBox[{"ns", "^", "2"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bluedat", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"szs", ",", "times"}], "}"}], "\[Transpose]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{
  3.681478739675144*^9, {3.681478856655671*^9, 3.6814788600151253`*^9}, 
   3.681478948149765*^9, {3.681479003939927*^9, 3.681479099192231*^9}, {
   3.68147930005154*^9, 3.681479301274852*^9}, {3.681483399367694*^9, 
   3.681483414294413*^9}, {3.68148351121964*^9, 3.681483519987196*^9}, {
   3.681483551658101*^9, 3.681483571480762*^9}, {3.681483610423649*^9, 
   3.6814836107042303`*^9}, 3.681483673240704*^9, {3.681483790723737*^9, 
   3.681483801681988*^9}, {3.7041342168377457`*^9, 3.704134217605804*^9}}],

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{
    RowBox[{"ticksb", "=", 
     RowBox[{"LogTicks", "[", 
      RowBox[{"10", ",", "1", ",", "5", ",", 
       RowBox[{"TickLabelStep", "\[Rule]", "0.25"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"tickst", "=", 
     RowBox[{"LogTicks", "[", 
      RowBox[{"10", ",", "1", ",", "5", ",", 
       RowBox[{"ShowTickLabels", "\[Rule]", "False"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"szplt", "=", 
     RowBox[{"Show", "[", 
      RowBox[{
       RowBox[{"ListPlot", "[", 
        RowBox[{
         RowBox[{"Log10", "[", "bluedat", "]"}], ",", 
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "14"}], "}"}]}], ",", 
         RowBox[{"Axes", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
         RowBox[{"PlotStyle", "\[Rule]", "Blue"}], ",", 
         RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
         RowBox[{"FrameTicks", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"ticksl", ",", "ticksr"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"ticksb", ",", "ticksb"}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"FrameStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"Black", ",", "Black"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", "White"}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotRange", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Automatic", ",", "yrng"}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"FrameLabel", "\[Rule]", "flabel"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Plot", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"x", "-", "1.5"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", "2", ",", "5"}], "}"}], ",", 
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Blue", ",", "Thick", ",", "Dashed"}], "}"}]}]}], "]"}]}], 
      "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.68148369869444*^9, 3.6814836996700277`*^9}, {
   3.70277233842768*^9, 3.702772414505404*^9}, 3.704134125363151*^9, {
   3.704134213694482*^9, 3.704134242701565*^9}, {3.704134275372917*^9, 
   3.704134361129793*^9}, {3.704134460873234*^9, 3.704134461425782*^9}, {
   3.704134640149694*^9, 3.704134641843886*^9}, 3.706896795777892*^9, 
   3.708682949384626*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["red curve", "Subsubsection",
 CellChangeTimes->{{3.7041340956915083`*^9, 3.7041341159157543`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig2dir", "<>", "\"\</grid_factor/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ns", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Range", "[", 
      RowBox[{"0", ",", "10", ",", "2"}], "]"}], ",", 
     RowBox[{"10", 
      RowBox[{"Range", "[", "10", "]"}]}], ",", 
     RowBox[{"Range", "[", 
      RowBox[{"150", ",", "500", ",", "50"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gfs", "=", 
   RowBox[{"ns", "*", "0.01"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"dir", "<>", "\"\<gf\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"gf", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "2"}], "}"}]}], "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"gf", ",", "gfs"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"times", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</data/time.txt\>\""}], ",", "\"\<TSV\>\""}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reddat", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"gfs", "^", "2"}], ",", "times"}], "}"}], "\[Transpose]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{
  3.681478739675144*^9, {3.681478856655671*^9, 3.6814788600151253`*^9}, 
   3.681478948149765*^9, {3.681479003939927*^9, 3.681479099192231*^9}, {
   3.68147930005154*^9, 3.681479301274852*^9}, {3.681483399367694*^9, 
   3.681483414294413*^9}, {3.68148351121964*^9, 3.681483519987196*^9}, {
   3.681483551658101*^9, 3.681483571480762*^9}, {3.681483610423649*^9, 
   3.6814836107042303`*^9}, 3.681483673240704*^9, {3.681483790723737*^9, 
   3.681483801681988*^9}, {3.6814848456018753`*^9, 3.681484978646256*^9}, {
   3.702772646749374*^9, 3.7027727266217117`*^9}, {3.704134396746089*^9, 
   3.704134402065688*^9}, {3.704134644764771*^9, 3.7041346455555887`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tickst", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", 
     RowBox[{"-", "3"}], ",", "3", ",", 
     RowBox[{"TickLabelStep", "\[Rule]", "0.25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gfplt", "=", 
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"Log10", "[", "reddat", "]"}], ",", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\[EmptyCircle]", ",", "15"}], "}"}]}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", "Red"}], ",", 
       RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"ticksl", ",", "ticksr"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"tickst", ",", "tickst"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Black", ",", "Black"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"White", ",", "Red"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "False"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"False", ",", "True"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Full", ",", "yrng"}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", "flabel"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "x"}], "+", "0.3"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "2.6"}], ",", 
         RowBox[{"-", "0.5"}]}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Red", ",", "Thick", ",", "Dashed"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"Blue", ",", "Thick", ",", "Dashed"}], "}"}]}], "}"}]}]}], 
      "]"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.68148369869444*^9, 3.6814836996700277`*^9}, {
   3.681484994021861*^9, 3.68148501329976*^9}, {3.702772753165908*^9, 
   3.702772764807103*^9}, {3.702772795917489*^9, 3.702773001697381*^9}, {
   3.704134409156184*^9, 3.7041344370006227`*^9}, {3.704134468852759*^9, 
   3.7041345476340113`*^9}, {3.7041345788385963`*^9, 3.704134590229027*^9}, {
   3.704134650940688*^9, 3.7041346547796307`*^9}, {3.704134688291628*^9, 
   3.7041347149312572`*^9}, {3.704134950702256*^9, 3.704134985099147*^9}, {
   3.7068967094951267`*^9, 3.706896765506463*^9}, 3.708682955358487*^9}],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{"Manipulate", "[", 
   RowBox[{
    RowBox[{"Overlay", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"szplt", ",", 
        RowBox[{"Show", "[", 
         RowBox[{"gfplt", ",", 
          RowBox[{"ImageSize", "\[Rule]", "s1"}]}], "]"}]}], "}"}], ",", 
      RowBox[{"Alignment", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"x1", ",", "y1"}], "}"}]}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "100", ",", "400"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x1", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y1", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}], "*)"}]], "Input",
 CellChangeTimes->{{3.704134776687901*^9, 3.704134832782509*^9}}],

Cell[BoxData[
 RowBox[{"gfszplt", "=", 
  RowBox[{"Overlay", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"szplt", ",", 
      RowBox[{"Show", "[", 
       RowBox[{"gfplt", ",", 
        RowBox[{"ImageSize", "\[Rule]", "274"}]}], "]"}]}], "}"}], ",", 
    RowBox[{"Alignment", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0.72", ",", "1"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.704135043625774*^9, 3.704135066624403*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</2B.eps\>\""}], ",", "gfszplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.689961332613821*^9, 3.689961337997651*^9}, 
   3.7041350810560913`*^9, 3.706896928780139*^9, {3.7068970813764877`*^9, 
   3.70689708163243*^9}}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Figures 3, S2", "Section",
 CellChangeTimes->{{3.681485057577465*^9, 3.6814850654009237`*^9}, {
  3.681485557523642*^9, 3.681485561563706*^9}, {3.704799157539839*^9, 
  3.704799157694456*^9}}],

Cell[CellGroupData[{

Cell["3 B", "Subsection",
 CellChangeTimes->{{3.681485636900084*^9, 3.681485651208992*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dx", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nfils", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nlks", "=", 
   RowBox[{"L", "/", "dx"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"teq", "=", "100"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lp", "=", "17"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6888435981492767`*^9, 3.688843654597272*^9}, 
   3.688844308423531*^9, {3.688902095248651*^9, 3.688902096487885*^9}, {
   3.6889070000723553`*^9, 3.688907003022193*^9}, {3.688907063245473*^9, 
   3.688907067813278*^9}, {3.688907312806796*^9, 3.6889073138151007`*^9}, {
   3.6899613973799953`*^9, 3.6899613982522383`*^9}, {3.689962152418592*^9, 
   3.689962152913419*^9}, 3.689962229118877*^9, 3.68996333720368*^9, {
   3.689963707665883*^9, 3.6899637186738853`*^9}, 3.6899637545449*^9, {
   3.6948252368567877`*^9, 3.6948252480243464`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"fig3Adir", "=", 
   RowBox[{"fig3dir", "<>", "\"\</kl_arr/kl1.00\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"Partition", "[", 
      RowBox[{"#", ",", "nlks"}], "]"}], "&"}], "/@", 
    RowBox[{"pts", "[", 
     RowBox[{"fig3Adir", ",", "\"\<links\>\""}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"angs", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"anglesT", ",", "lks", ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"angsbw", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"anglesBwT", ",", "angs", ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ccfs", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"ccf", ",", "angsbw", ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"th2s", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"th2", ",", "angsbw", ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.688843242504636*^9, 3.688843250502825*^9}, {
   3.6888432854643917`*^9, 3.688843355740748*^9}, {3.688843387484618*^9, 
   3.688843460835856*^9}, {3.688843616070531*^9, 3.688843616478195*^9}, {
   3.6889017253568573`*^9, 3.688901726513508*^9}, {3.6889021032794733`*^9, 
   3.688902128312624*^9}, {3.688907012224152*^9, 3.688907014261779*^9}, {
   3.6889070759578743`*^9, 3.6889070761245613`*^9}, {3.6889072934487762`*^9, 
   3.688907294600191*^9}, {3.6899617213481216`*^9, 3.689961772651939*^9}, 
   3.6899618272661543`*^9, {3.6899628681453733`*^9, 3.689962868288459*^9}, {
   3.6899632272141857`*^9, 3.6899632737353487`*^9}, {3.7020589896799717`*^9, 
   3.702059008447254*^9}, {3.7069590245209293`*^9, 3.7069590321682796`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"muths", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"th2s", "[", 
     RowBox[{"[", 
      RowBox[{"teq", ";;"}], "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"muthsmu", "=", 
   RowBox[{"Mean", "[", "muths", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muthserr", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", "muths", "]"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"Length", "[", "muths", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.688836018561613*^9, 3.688836181654455*^9}, {
  3.688836327819538*^9, 3.688836336683172*^9}, {3.688839943304529*^9, 
  3.6888399605386667`*^9}, {3.688840101024988*^9, 3.688840102519063*^9}, {
  3.6888435601517572`*^9, 3.688843581623076*^9}, {3.6888436340299053`*^9, 
  3.688843710244451*^9}, {3.688844244239398*^9, 3.688844252005994*^9}, {
  3.6899636625711603`*^9, 3.689963716161921*^9}, {3.692627396304717*^9, 
  3.692627497643024*^9}, {3.694825218480102*^9, 3.694825230296645*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"muccfs", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"ccfs", "[", 
     RowBox[{"[", 
      RowBox[{"teq", ";;"}], "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"muccfsmu", "=", 
   RowBox[{"Mean", "[", "muccfs", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muccfserr", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", "muccfs", "]"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"Length", "[", "muccfs", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.688844265045203*^9, 3.688844282636593*^9}, {
   3.688844339053536*^9, 3.688844339363995*^9}, {3.688907616447357*^9, 
   3.6889076178706293`*^9}, {3.6899618549369307`*^9, 
   3.6899618561364822`*^9}, {3.689963672115369*^9, 3.6899636862589903`*^9}, {
   3.6899637310178223`*^9, 3.689963731480876*^9}, {3.692627502265068*^9, 
   3.692627528861187*^9}, 3.6948252238482647`*^9, 3.706961466882884*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"xs", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"dx", ",", 
     RowBox[{"L", "-", "dx"}], ",", "dx"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6881405241305847`*^9, 3.688140557760262*^9}, {
  3.688843859399308*^9, 3.688843877262681*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"th2plt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", "muthsmu"}], "}"}], "\[Transpose]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muthsmu", "-", "muthserr"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muthsmu", "+", "muthserr"}]}], "}"}], "\[Transpose]"}]}],
         "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
          RowBox[{"Rotate", "[", 
           RowBox[{
            RowBox[{"Style", "[", 
             RowBox[{
             "\"\<\[LeftAngleBracket]\!\(\*SuperscriptBox[\(\[Theta]\), \
\(2\)]\)(\[ScriptL])\[RightAngleBracket] (\[FilledSquare])\>\"", ",", 
              "Blue"}], "]"}], ",", "Pi"}], "]"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameStyle", "\[Rule]", "Blue"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "True"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"False", ",", "False"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"None", ",", "All"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"None", ",", "None"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Comment", " ", "out", " ", "next", " ", "three", " ", "lines", " ", 
         "if", " ", "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Blue", ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "5"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Joined", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"False", ",", "True", ",", "True"}], "}"}]}], ","}], 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Uncomment", " ", "next", " ", "four", " ", "line", " ", "if", " ", 
         "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "12"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ","}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Rectangle", "[", "]"}]}], "}"}], "]"}], ",", 
            "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"0.75", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"0.75", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", "Blue"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"2", "\[Rule]", 
          RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "20"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", "/", 
         RowBox[{"(", "lp", ")"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "50"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Thick", ",", 
          RowBox[{"Dashing", "[", "Large", "]"}], ",", "Blue"}], "}"}]}]}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ccfplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListLogPlot", "[", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", "muccfsmu"}], "}"}], "\[Transpose]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muccfsmu", "-", "muccfserr"}]}], "}"}], "\[Transpose]"}],
          ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muccfsmu", "+", "muccfserr"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<\[ScriptL] (\[Mu]m)\>\"", ",", 
          RowBox[{"Style", "[", 
           RowBox[{
           "\"\<\[LeftAngleBracket]cos\[Theta](\[ScriptL])\[RightAngleBracket]\
 (\[FilledCircle])\>\"", ",", "Red"}], "]"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Red", ",", "Automatic"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"Black", ",", "Black"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"True", ",", "False"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"True", ",", "True"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"All", ",", "None"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"All", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Comment", " ", "out", " ", "next", " ", "three", " ", "lines", " ", 
         "if", " ", "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Red", ",", 
            RowBox[{"{", 
             RowBox[{"Red", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Red", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "5"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Joined", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"False", ",", "True", ",", "True"}], "}"}]}], ","}], 
        "\[IndentingNewLine]", "*)"}], 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Uncomment", " ", "next", " ", "four", " ", "line", " ", "if", " ", 
         "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Disk", "[", "]"}]}], "}"}], "]"}], ",", "0.045"}], 
           "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", "Red"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"2", "\[Rule]", 
          RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "20"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"LogPlot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "x"}], "/", 
          RowBox[{"(", 
           RowBox[{"2", "lp"}], ")"}]}], "]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "50"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Thick", ",", 
          RowBox[{"Dashing", "[", "Large", "]"}], ",", "Red"}], "}"}]}]}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.668427445277066*^9, 3.668427486875223*^9}, {
   3.668427521157255*^9, 3.668427527602787*^9}, {3.668427925859844*^9, 
   3.668427928592771*^9}, 3.6690295624217243`*^9, 3.66903615163303*^9, {
   3.669036923248163*^9, 3.669036931030847*^9}, {3.669042061073771*^9, 
   3.669042065880272*^9}, {3.6690422178006773`*^9, 3.669042217929419*^9}, {
   3.669042354772778*^9, 3.669042383106879*^9}, {3.669042416811346*^9, 
   3.669042438086665*^9}, 3.669042562020055*^9, {3.670194484092348*^9, 
   3.6701945327415867`*^9}, {3.670194609249497*^9, 3.670194636131197*^9}, {
   3.670194977018643*^9, 3.670194990837554*^9}, 3.6701950444269876`*^9, {
   3.688140567857744*^9, 3.6881405852347183`*^9}, 3.688219692712883*^9, {
   3.688235084651927*^9, 3.688235086032837*^9}, {3.688843888855977*^9, 
   3.6888440266919317`*^9}, 3.6888441160052233`*^9, {3.688844165555574*^9, 
   3.688844187952065*^9}, {3.6889021987668858`*^9, 3.688902211870509*^9}, {
   3.688902281828388*^9, 3.688902282452259*^9}, {3.688903386244416*^9, 
   3.6889033917797203`*^9}, {3.688907532434621*^9, 3.6889075426574497`*^9}, {
   3.689962214798121*^9, 3.689962215992632*^9}, {3.692627537011492*^9, 
   3.692627575002136*^9}, {3.692627671835803*^9, 3.6926277041175823`*^9}, {
   3.692628057022195*^9, 3.692628081081669*^9}, {3.692628331203144*^9, 
   3.692628337495387*^9}, {3.692628428330727*^9, 3.692628724955847*^9}, {
   3.6926287714217787`*^9, 3.692628777900751*^9}, {3.694825321408148*^9, 
   3.6948253349011173`*^9}, {3.706959880678811*^9, 3.706959921643188*^9}, {
   3.706961607697465*^9, 3.706961630936215*^9}, {3.706961781477497*^9, 
   3.706961786793128*^9}, {3.706970365252192*^9, 3.706970365875758*^9}, {
   3.706970408291597*^9, 3.706970409561246*^9}, {3.706970444370699*^9, 
   3.706970450913087*^9}, 3.706970556084227*^9}],

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"{", 
   RowBox[{"ccfplt", ",", "th2plt"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.668428347267419*^9, 3.668428362876987*^9}, {
  3.668428545209313*^9, 3.668428547608214*^9}, {3.681487836453225*^9, 
  3.681487881018704*^9}, {3.6926278816854362`*^9, 3.692627886603735*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</3BRed.eps\>\""}], ",", "ccfplt"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</3BBlue.eps\>\""}], ",", "th2plt"}], "]"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6888454322637253`*^9, 3.68884543302357*^9}, {
  3.688902342513777*^9, 3.688902370056594*^9}, {3.689963795367483*^9, 
  3.6899638346544647`*^9}, {3.692628382647876*^9, 3.692628391352021*^9}, {
  3.692628897780369*^9, 3.692628902651881*^9}, {3.702059204236408*^9, 
  3.702059206334394*^9}, {3.70696015205122*^9, 3.706960157602888*^9}, {
  3.7069616520737123`*^9, 3.706961656511554*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["S2", "Subsection",
 CellChangeTimes->{{3.6814860715100307`*^9, 3.681486081997136*^9}, {
  3.6814879430246477`*^9, 3.681487943344756*^9}, {3.704799163971861*^9, 
  3.70479916416142*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig3dir", "<>", "\"\</decor\>\""}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<links\>\""}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"angs", "=", 
   RowBox[{"anglesT", "/@", "lks"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"angsbw", "=", 
   RowBox[{"anglesBwT", "/@", "angs"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ccs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"CorrelationFunction", "[", 
      RowBox[{
       RowBox[{"angsbw", "[", 
        RowBox[{"[", 
         RowBox[{"All", ",", "i"}], "]"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "19999"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"angsbw", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"mccs", "=", 
   RowBox[{"Mean", "[", "ccs", "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"cf", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"0.005", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"Length", "[", "mccs", "]"}], "]"}], "-", "1"}], ")"}]}], 
      ",", "mccs"}], "}"}], "\[Transpose]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"decortimes", "=", 
   RowBox[{"0.01", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Total", "[", 
       RowBox[{"mccs", "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", "i"}], "]"}], "]"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "2", ",", "20000"}], "}"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.679247740748741*^9, 3.679247745882997*^9}, {
   3.679308124936595*^9, 3.679308125855735*^9}, {3.6814861473664427`*^9, 
   3.6814861537899933`*^9}, 3.681486202030113*^9, {3.6814879162340117`*^9, 
   3.681487916569923*^9}, {3.6814880332723417`*^9, 3.6814880758714848`*^9}, 
   3.689966696391391*^9, {3.689966813099277*^9, 3.689966817043273*^9}, {
   3.6899668474119797`*^9, 3.6899668631384172`*^9}, {3.689966928840221*^9, 
   3.689966934753706*^9}, 3.7023165775927963`*^9}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.7069707115352907`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["S1A", "Subsubsection",
 CellChangeTimes->{{3.681488082876646*^9, 3.681488089192833*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ps", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Range", "[", 
      RowBox[{"1", ",", "5"}], "]"}], ",", 
     RowBox[{"Range", "[", 
      RowBox[{"7", ",", "15", ",", "2"}], "]"}], ",", 
     RowBox[{"Range", "[", 
      RowBox[{"20", ",", "100", ",", "20"}], "]"}], ",", 
     RowBox[{"Range", "[", 
      RowBox[{"150", ",", "2000", ",", "50"}], "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.694203942126101*^9, 3.694203986820271*^9}, {
  3.694204029788004*^9, 3.694204057938751*^9}, {3.694204091274929*^9, 
  3.694204143416222*^9}}],

Cell[BoxData[
 RowBox[{"plta", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"ListLogPlot", "[", 
     RowBox[{"cf", ",", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "0.1"}], ",", "10.1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0.005", ",", "1.2"}], "}"}]}], "}"}]}], ",", 
      RowBox[{"Joined", "\[Rule]", "True"}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<s  (sec)\>\"", ",", 
         RowBox[{"Style", "[", 
          RowBox[{"\"\<R(\[Theta],s)\>\"", ",", 
           RowBox[{"SingleLetterItalics", "\[Rule]", "False"}]}], "]"}]}], 
        "}"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"ListLogPlot", "[", 
     RowBox[{
      RowBox[{"cf", "[", 
       RowBox[{"[", "ps", "]"}], "]"}], ",", 
      RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
      RowBox[{"PlotMarkers", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\[FilledCircle]", ",", "7"}], "}"}]}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}]}], "]"}]}]], "Input",\

 CellChangeTimes->{{3.6899671431951857`*^9, 3.6899673786705437`*^9}, {
  3.6899675277621603`*^9, 3.689967710452628*^9}, {3.6899677676525793`*^9, 
  3.689967769003866*^9}, {3.689968155882427*^9, 3.6899681568496027`*^9}, {
  3.694203917250765*^9, 3.6942039248023663`*^9}, {3.694204185313566*^9, 
  3.694204188255865*^9}, {3.6942042574466257`*^9, 3.694204292490054*^9}, {
  3.6942207205540667`*^9, 3.6942207254050694`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S1A.eps\>\""}], ",", "plta"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6899680893063507`*^9, 3.689968092986697*^9}, {
  3.689968141443097*^9, 3.689968162209289*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["S1B", "Subsubsection",
 CellChangeTimes->{{3.681488092541636*^9, 3.681488099394761*^9}}],

Cell[BoxData[
 RowBox[{"pltb", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"decortimes", ",", " ", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "15"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1.12"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Black", ",", "Thick"}], "}"}]}], ",", 
    RowBox[{"DataRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "100"}], "}"}]}], ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\!\(\*SubscriptBox[\(t\), \(final\)]\) (sec)\>\"", ",", 
       "\"\<\[Tau] (sec)\>\""}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.679314695210397*^9, 3.679314716837257*^9}, {
   3.67931542775768*^9, 3.679315428819701*^9}, {3.679315482090784*^9, 
   3.679315483221333*^9}, {3.679315515171554*^9, 3.679315681029505*^9}, {
   3.6793157237107286`*^9, 3.679315733388226*^9}, {3.679316210177846*^9, 
   3.6793162129518433`*^9}, {3.67931786390441*^9, 3.6793178732620296`*^9}, {
   3.67931798892556*^9, 3.6793180726371403`*^9}, {3.679318346851409*^9, 
   3.679318361603799*^9}, {3.67931932769026*^9, 3.6793193476099663`*^9}, {
   3.67932108351818*^9, 3.679321090422134*^9}, 3.681488106254443*^9, {
   3.689967480219969*^9, 3.689967513226961*^9}, {3.6899681200117064`*^9, 
   3.689968133754859*^9}, {3.689968173665724*^9, 3.689968174465423*^9}, {
   3.694204320630706*^9, 3.694204325449699*^9}, {3.7023168035084343`*^9, 
   3.7023168444353952`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S1B.eps\>\""}], ",", "pltb"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.689968183304413*^9, 3.689968186704307*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3 C", "Subsection",
 CellChangeTimes->{{3.6814881596778793`*^9, 3.681488170077593*^9}, 
   3.702319806851655*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"seeds", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"1", ",", "1000"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"fig3Cdir", "<>", "\"\<seed\>\"", "<>", 
      RowBox[{"ToString", "[", "s", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", "seeds"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nfils", "=", "100"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"tf", "=", "2001"}], ";"}]}], "Input",
 CellChangeTimes->{
  3.688867569399598*^9, {3.6899682198163977`*^9, 3.6899682210318737`*^9}, {
   3.690109046015226*^9, 3.69010910432218*^9}, 3.690109740853941*^9, {
   3.6901098878660316`*^9, 3.690109890657546*^9}, {3.694825689291568*^9, 
   3.6948257407641077`*^9}, {3.694825789269104*^9, 3.694825790268907*^9}, 
   3.7023198099479837`*^9}],

Cell[CellGroupData[{

Cell["Evaluate the eigenvalues for the first time", "Subsubsection",
 CellChangeTimes->{3.6948256698368893`*^9}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"fstreams", "=", 
     RowBox[{
      RowBox[{
       RowBox[{"OpenRead", "[", 
        RowBox[{"#", "<>", "\"\</txt_stack/actins_ext.dat\>\""}], "]"}], 
       "&"}], "/@", "dirs"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"evs", "=", 
     RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Do", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"actsextt", "=", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"readline3", "[", 
           RowBox[{"f", ",", 
            RowBox[{"2", "*", "nfils"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"f", ",", "fstreams"}], "}"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"evs", ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"Eigenvalues", "[", 
            RowBox[{"Covariance", "[", 
             RowBox[{"actsextt", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", "i", ",", 
                RowBox[{"1", ";;", "2"}]}], "]"}], "]"}], "]"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", 
             RowBox[{"2", "*", "nfils"}]}], "}"}]}], "]"}]}], "]"}], ";"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"{", "tf", "}"}]}], "\[IndentingNewLine]", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{
      RowBox[{"fig3Cdir", "<>", "\"\</all_evs.dat\>\""}], ",", "evs"}], "]"}],
     ";"}], "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.690109749716836*^9, 3.6901097568893147`*^9}, {
   3.690111776674369*^9, 3.690111829193057*^9}, {3.690111899415036*^9, 
   3.690111917590077*^9}, {3.690111977293144*^9, 3.690112143712899*^9}, 
   3.690112210308506*^9, {3.690112240926971*^9, 3.690112254061735*^9}, 
   3.690112592166883*^9, 3.690112651443818*^9, {3.690112827076489*^9, 
   3.69011283828749*^9}, {3.690113542344779*^9, 3.6901135575651197`*^9}, 
   3.6901137696960993`*^9, {3.690114128222259*^9, 3.690114137926394*^9}, {
   3.6926293785144663`*^9, 3.6926293989786587`*^9}, {3.694825599299136*^9, 
   3.694825687374854*^9}, {3.694825797124634*^9, 3.694825798884211*^9}, 
   3.702319828251762*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Import Previously evaluated eigenvalues", "Subsubsection",
 CellChangeTimes->{{3.6948401023700457`*^9, 3.6948401234395733`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"evs", "=", 
   RowBox[{"ToExpression", "[", 
    RowBox[{"Import", "[", 
     RowBox[{
      RowBox[{"fig3Cdir", "<>", "\"\</all_evs.dat\>\""}], ",", 
      "\"\<TSV\>\""}], "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6926293356350613`*^9, 3.692629365303092*^9}, {
  3.694825802796508*^9, 3.694825804204021*^9}, {3.706960677080464*^9, 
  3.706960677421598*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ts", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0.001", ",", 
     RowBox[{"tf", "*", "0.001"}], ",", "0.001"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"maxevs", "=", 
   RowBox[{"evs", "[", 
    RowBox[{"[", 
     RowBox[{"All", ",", "All", ",", "1"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"minevs", "=", 
   RowBox[{"evs", "[", 
    RowBox[{"[", 
     RowBox[{"All", ",", "All", ",", "2"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"mumaxevs", "=", 
   RowBox[{"Mean", "/@", "maxevs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muminevs", "=", 
   RowBox[{"Mean", "/@", "minevs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"errmaxevs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "maxevs"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{
      RowBox[{"Dimensions", "[", "evs", "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"errminevs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "minevs"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{
      RowBox[{"Dimensions", "[", "evs", "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.661691949002371*^9, 3.661691963379101*^9}, {
   3.661692035441389*^9, 3.661692071288414*^9}, {3.681488316698078*^9, 
   3.681488396255754*^9}, {3.681489146273534*^9, 3.681489180573453*^9}, {
   3.681490634766802*^9, 3.681490636526746*^9}, {3.681649598135318*^9, 
   3.681649599194922*^9}, {3.688867341148519*^9, 3.688867456939609*^9}, {
   3.6888675441168947`*^9, 3.688867549503434*^9}, 3.690109748620873*^9, {
   3.690112399802515*^9, 3.690112401849997*^9}, 3.690113621275536*^9, {
   3.690118015246152*^9, 3.690118015981584*^9}, {3.69262975732336*^9, 
   3.692629827905003*^9}, {3.692629874772706*^9, 3.692629879732862*^9}, 
   3.6948258111410522`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"maxmaxevs", "=", 
   RowBox[{"Max", "/@", "maxevs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"maxminevs", "=", 
   RowBox[{"Max", "/@", "minevs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"minmaxevs", "=", 
   RowBox[{"Min", "/@", "maxevs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"minminevs", "=", 
   RowBox[{"Min", "/@", "minevs"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.688868603711191*^9, 3.688868712534042*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"maxnorm", "=", 
   RowBox[{"mumaxevs", "[", 
    RowBox[{"[", "1000", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"minnorm", "=", 
   RowBox[{"muminevs", "[", 
    RowBox[{"[", "1000", "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.690118489656066*^9, 3.6901185079929657`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"idxs", "=", 
   RowBox[{"Union", "[", 
    RowBox[{"Round", "[", 
     RowBox[{"10", "^", 
      RowBox[{"Range", "[", 
       RowBox[{
        RowBox[{"Log10", "[", "2", "]"}], ",", 
        RowBox[{"Log10", "[", "2000", "]"}], ",", "0.05"}], "]"}]}], "]"}], 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.68886897480028*^9, 3.68886902636089*^9}, {
   3.690112417649403*^9, 3.6901124274001303`*^9}, {3.69011717972276*^9, 
   3.690117225456943*^9}, 3.6926296416256332`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ticksx", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", 
     RowBox[{"-", "3"}], ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"msz", "=", "0.04"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fluctplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{"ts", ",", 
              RowBox[{"mumaxevs", "/", "maxnorm"}]}], "}"}], "\[Transpose]"}],
            "[", 
           RowBox[{"[", "idxs", "]"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"ts", ",", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"mumaxevs", "-", "errmaxevs"}], ")"}], "/", 
                 "maxnorm"}]}], "}"}], "\[Transpose]"}], "[", 
             RowBox[{"[", "idxs", "]"}], "]"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"ts", ",", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"mumaxevs", "+", "errmaxevs"}], ")"}], "/", 
                 "maxnorm"}]}], "}"}], "\[Transpose]"}], "[", 
             RowBox[{"[", "idxs", "]"}], "]"}], ","}], "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{"ts", ",", 
              RowBox[{"muminevs", "/", "minnorm"}]}], "}"}], "\[Transpose]"}],
            "[", 
           RowBox[{"[", "idxs", "]"}], "]"}]}], 
         RowBox[{"(*", 
          RowBox[{",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"ts", ",", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"muminevs", "-", "errminevs"}], ")"}], "/", 
                "minnorm"}]}], "}"}], "\[Transpose]"}], "[", 
            RowBox[{"[", "idxs", "]"}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"ts", ",", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"muminevs", "+", "errminevs"}], ")"}], "/", 
                "minnorm"}]}], "}"}], "\[Transpose]"}], "[", 
            RowBox[{"[", "idxs", "]"}], "]"}]}], "*)"}], 
         "\[IndentingNewLine]", "}"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "2.5"}], ",", 
            RowBox[{"Log10", "[", "1.2", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "2.5"}], ",", "0.8"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Blue", ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Opacity", "[", "0.1", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Opacity", "[", "0.1", "]"}]}], "}"}], ",", "Red", ",", 
            
            RowBox[{"{", 
             RowBox[{"Red", ",", 
              RowBox[{"Opacity", "[", "0.1", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Red", ",", 
              RowBox[{"Opacity", "[", "0.1", "]"}]}], "}"}]}], "}"}]}], ","}],
         "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "7"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "7"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}]}], "}"}]}], ","}],
         "*)"}], "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Circle", "[", "]"}]}], "}"}], "]"}], ",", "msz"}], 
           "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Graphics", "[", 
               RowBox[{"{", 
                RowBox[{"Blue", ",", 
                 RowBox[{"Line", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], 
                "}"}], "]"}], ",", "msz"}], "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Graphics", "[", 
               RowBox[{"{", 
                RowBox[{"Blue", ",", 
                 RowBox[{"Line", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], 
                "}"}], "]"}], ",", "msz"}], "}"}], ","}], "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Disk", "[", "]"}]}], "}"}], "]"}], ",", "msz"}], 
           "}"}]}], 
         RowBox[{"(*", 
          RowBox[{",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Graphics", "[", 
              RowBox[{"{", 
               RowBox[{"Red", ",", 
                RowBox[{"Line", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], 
               "}"}], "]"}], ",", "msz"}], "}"}], ",", "\[IndentingNewLine]", 
           
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Graphics", "[", 
              RowBox[{"{", 
               RowBox[{"Red", ",", 
                RowBox[{"Line", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], 
               "}"}], "]"}], ",", "msz"}], "}"}]}], "*)"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Filling", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"2", "\[Rule]", 
             RowBox[{"{", "3", "}"}]}], ",", 
            RowBox[{"5", "\[Rule]", 
             RowBox[{"{", "6", "}"}]}]}], "}"}]}], ","}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<t (s)\>\"", ",", "\"\<\[Lambda](t)/\[Lambda](1)\>\""}], 
         "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"ticksx", ",", "Automatic"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ticksx", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], 
       RowBox[{"(*", 
        RowBox[{"{", 
         RowBox[{
         "False", ",", "True", ",", "True", ",", "False", ",", "True", ",", 
          "True"}], "}"}], "*)"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Axes", "\[Rule]", "None"}]}], "\[IndentingNewLine]", "]"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"LogLogPlot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"t", "^", 
          RowBox[{"{", 
           RowBox[{"7", "/", "8"}], "}"}]}], ",", 
         RowBox[{"1.05", 
          RowBox[{"t", "^", 
           RowBox[{"{", "0.75", "}"}]}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"t", ",", "0.001", ",", "10"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Blue", ",", "Thick", ",", "Dashed"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"Red", ",", "Thick", ",", "Dashed"}], "}"}]}], "}"}]}]}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.692629947333927*^9, 3.692630006420792*^9}, {
  3.692630086168386*^9, 3.69263008639898*^9}, {3.6926301992350483`*^9, 
  3.69263022711547*^9}, {3.6926303687958384`*^9, 3.692630445778343*^9}, {
  3.692630511508078*^9, 3.6926305441945763`*^9}, {3.692630938759447*^9, 
  3.692630958697639*^9}, {3.6948258928313093`*^9, 3.694825913991007*^9}, {
  3.706970995572629*^9, 3.7069711629850483`*^9}, {3.706971262394327*^9, 
  3.706971363765369*^9}, {3.7069715932790833`*^9, 3.706971593987377*^9}}],

Cell[BoxData["fluctplt"], "Input",
 CellChangeTimes->{{3.692630013577462*^9, 3.692630015692322*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</3C.eps\>\""}], ",", "fluctplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.68886967913385*^9, 3.688869679942526*^9}, {
  3.6888699193595552`*^9, 3.688869940192646*^9}, {3.690118759902116*^9, 
  3.690118783162641*^9}, {3.690118958095358*^9, 3.690118960221488*^9}, {
  3.692630947324191*^9, 3.6926309693484707`*^9}, {3.694955405962553*^9, 
  3.6949554069857273`*^9}}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Figure 4", "Section",
 CellChangeTimes->{{3.681490709459614*^9, 3.681490719268773*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nlks", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dx", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"teq", "=", "100"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nfils", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"teq", "=", "100"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Lp", "=", "17"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"maxl", "=", "5"}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ticksx", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", 
     RowBox[{"-", "7"}], ",", "3"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rngy", "=", 
   RowBox[{"{", 
    RowBox[{"15.9", ",", "18.1"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.694827488081492*^9, 3.69482754539447*^9}, {
   3.6948275771706543`*^9, 3.694827577605371*^9}, {3.694827898436797*^9, 
   3.694827902597581*^9}, 3.702322616682626*^9}],

Cell[CellGroupData[{

Cell["A", "Subsection",
 CellChangeTimes->{{3.673690546043909*^9, 3.6736905483629208`*^9}, 
   3.681491535693219*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kbdir", "=", 
   RowBox[{"fig3dir", "<>", "\"\</kb_arr\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kbs", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.005", ",", "0.01", ",", "0.02", ",", "0.05", ",", "0.1", ",", "0.2", 
     ",", "0.5", ",", "1.0", ",", "2.0", ",", "5", ",", "10", ",", "20", ",", 
     "50", ",", "100", ",", "200", ",", "500", ",", "1000"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kbdirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"kbdir", "<>", "\"\</kb\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "3"}], "}"}], ",", 
         RowBox[{"ExponentFunction", "\[Rule]", 
          RowBox[{"(", 
           RowBox[{"Null", "&"}], ")"}]}]}], "]"}], "]"}]}], "&"}], "/@", 
    "kbs"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.688909031207704*^9, 3.6889090634199743`*^9}, {
   3.68890921290136*^9, 3.688909214521461*^9}, {3.688909248367621*^9, 
   3.688909250845167*^9}, 3.6889094257456017`*^9, 3.688909713826535*^9, {
   3.688909876310413*^9, 3.6889098800451813`*^9}, {3.689970410605195*^9, 
   3.689970417705097*^9}, {3.6948274139613447`*^9, 3.694827419553361*^9}, {
   3.69482746636882*^9, 3.6948274752736397`*^9}, {3.702059780627728*^9, 
   3.702059780709993*^9}, 3.702059899850852*^9, {3.7020599551244993`*^9, 
   3.702059956391706*^9}, {3.7026384712105227`*^9, 3.702638473142873*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Partition", "[", 
        RowBox[{"#", ",", "nlks"}], "]"}], "&"}], "/@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"pts", "[", 
         RowBox[{
          RowBox[{"kbdirs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "\"\<links\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"teq", ";;", 
          RowBox[{"-", "2"}], ";;", "2"}], "]"}], "]"}], ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "kbs", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lps", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"plMeanThsq", "[", 
        RowBox[{
         RowBox[{"lks", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "All", ",", "j", ",", "All", ",", 
            RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", "dx", ",", "L", 
         ",", "maxl"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"lks", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "lks", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lpmeans", "=", 
   RowBox[{"Mean", "/@", "lps"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "lps"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6889092263583097`*^9, 3.688909231277938*^9}, {
   3.688909390923856*^9, 3.688909397489203*^9}, {3.688909450401243*^9, 
   3.688909450744975*^9}, 3.688912650834285*^9, 3.6948275810358047`*^9, {
   3.6949589693604183`*^9, 3.694958975664885*^9}, {3.7045593906413927`*^9, 
   3.704559391216786*^9}, {3.7069611014681053`*^9, 3.706961107588023*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lplpplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListLogLogPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"kbs", "/", "0.004"}], ",", "lpmeans"}], "}"}], 
         "\[Transpose]"}], 
        RowBox[{"(*", 
         RowBox[{",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"kbs", "/", "0.004"}], ",", 
             RowBox[{"lpmeans", "-", "lperrs"}]}], "}"}], "\[Transpose]"}], 
          ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"kbs", "/", "0.004"}], ",", 
             RowBox[{"lpmeans", "+", "lperrs"}]}], "}"}], "\[Transpose]"}]}], 
         "*)"}], "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "[", 
           RowBox[{"{", 
            RowBox[{"Disk", "[", "]"}], "}"}], "]"}], ",", "0.05"}], "}"}]}], 
       
       RowBox[{"(*", 
        RowBox[{"{", 
         RowBox[{",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}]}], "}"}], "*)"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ","}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Black", ",", "Black"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"2", "\[Rule]", 
            RowBox[{"{", "3", "}"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"5", "\[Rule]", 
            RowBox[{"{", "6", "}"}]}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"LogLogPlot", "[", 
      RowBox[{"x", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0.1", ",", 
         RowBox[{"10", "^", "6"}]}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", 
          RowBox[{"Dashing", "[", "0.02", "]"}]}], 
         RowBox[{"(*", 
          RowBox[{",", "Thick"}], "*)"}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(\[Kappa]\), \(B\)]\) (\!\(\*SubscriptBox[\(k\
\), \(B\)]\)T\[Times]\[Mu]m)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(L\), \(\(p\)\(\\\ \)\)]\)(\[Mu]m)\>\""}], 
       "}"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.673699939102161*^9, 3.673700011867363*^9}, {
   3.673700352501178*^9, 3.673700376970888*^9}, {3.673700443211453*^9, 
   3.673700492096118*^9}, {3.6737005832553673`*^9, 3.673700798192556*^9}, {
   3.673700972293968*^9, 3.6737010583513193`*^9}, {3.673701109946662*^9, 
   3.6737011497922583`*^9}, {3.673701198712723*^9, 3.673701207038722*^9}, {
   3.6793386008167973`*^9, 3.6793386673352823`*^9}, {3.6793933330419283`*^9, 
   3.6793933339930687`*^9}, {3.688909926757944*^9, 3.688910115144164*^9}, {
   3.688910146425112*^9, 3.688910204333703*^9}, {3.6889103440035343`*^9, 
   3.688910391825008*^9}, {3.702638830022729*^9, 3.702638831214315*^9}, {
   3.706964141132222*^9, 3.706964162775062*^9}, {3.706964396962512*^9, 
   3.706964402099533*^9}, 3.706964472871117*^9, {3.706964530008128*^9, 
   3.706964530150751*^9}}],

Cell[BoxData["lplpplt"], "Input",
 CellChangeTimes->{{3.679339354022794*^9, 3.679339355484564*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</4A_dt2.eps\>\""}], ",", "lplpplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.689970567721046*^9, 3.689970579508089*^9}, {
  3.7045596866410227`*^9, 3.704559687136984*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsection",
 CellChangeTimes->{{3.673690546043909*^9, 3.6736905483629208`*^9}, {
   3.673703705247267*^9, 3.673703705294692*^9}, 3.681491172652049*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kldir", "=", 
   RowBox[{"fig3dir", "<>", "\"\</kl_arr/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kls", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.01", ",", "0.02", ",", "0.05", ",", "0.1", ",", "0.2", ",", "0.5", ",",
      "1.0", ",", "2", ",", "5", ",", "10", ",", "20", ",", "50", ",", "100", 
     ",", "200"}], 
    RowBox[{"(*", 
     RowBox[{",", "500", ",", "1000"}], "*)"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kldirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"kldir", "<>", "\"\</kl\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "2"}], "}"}], ",", 
         RowBox[{"ExponentFunction", "\[Rule]", 
          RowBox[{"(", 
           RowBox[{"Null", "&"}], ")"}]}]}], "]"}], "]"}]}], "&"}], "/@", 
    "kls"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.688912702925228*^9, 3.688912777908717*^9, {3.688912836186125*^9, 
   3.688912872664366*^9}, {3.6889141888718233`*^9, 3.688914192078882*^9}, {
   3.688914232999997*^9, 3.68891423819844*^9}, {3.6899706570759478`*^9, 
   3.6899706675464277`*^9}, {3.689970781632534*^9, 3.689970788561544*^9}, {
   3.6899709458298683`*^9, 3.689970949626053*^9}, {3.690046539613387*^9, 
   3.690046539719692*^9}, {3.690046655479246*^9, 3.69004665867372*^9}, {
   3.6900467364483757`*^9, 3.690046736711817*^9}, 3.690046917500307*^9, {
   3.69004880609512*^9, 3.690048834452615*^9}, {3.694827508706661*^9, 
   3.694827554594385*^9}, 3.6949590502082987`*^9, {3.702060309967326*^9, 
   3.702060311094297*^9}, 3.7020603420833693`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Partition", "[", 
        RowBox[{"#", ",", "nlks"}], "]"}], "&"}], "/@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"pts", "[", 
         RowBox[{
          RowBox[{"kldirs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "\"\<links\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"teq", ";;", 
          RowBox[{"-", "1"}]}], "]"}], "]"}], ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "kls", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lps", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"plMeanThsq", "[", 
        RowBox[{
         RowBox[{"lks", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "All", ",", "j", ",", "All", ",", 
            RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", "dx", ",", "L", 
         ",", "maxl"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"lks", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "lks", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lpmeans", "=", 
   RowBox[{"Mean", "/@", "lps"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "lps"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.6889128469143953`*^9, {3.688912881104621*^9, 3.6889128813852463`*^9}, 
   3.690046899139833*^9, {3.694827755511664*^9, 3.6948277971819277`*^9}, 
   3.694827871986713*^9, {3.6948794221873827`*^9, 3.694879424771328*^9}, {
   3.694959024823882*^9, 3.694959055496361*^9}, 3.694959434300005*^9, {
   3.704559905643922*^9, 3.704559906267598*^9}, 3.7045604875025578`*^9, {
   3.706962091377963*^9, 3.7069620942187366`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"kllpplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Log10", "[", "kls", "]"}], ",", "lpmeans"}], "}"}], 
          "\[Transpose]"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Log10", "[", "kls", "]"}], ",", 
            RowBox[{"lpmeans", "-", "lperrs"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Log10", "[", "kls", "]"}], ",", 
            RowBox[{"lpmeans", "+", "lperrs"}]}], "}"}], "\[Transpose]"}]}], 
        "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ","}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Disk", "[", "]"}], "}"}], "]"}], ",", "0.045"}], "}"}],
           ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Black", ",", "Black"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "rngy"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"2", "\[Rule]", 
            RowBox[{"{", "3", "}"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"5", "\[Rule]", 
            RowBox[{"{", "6", "}"}]}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ticksx", ",", "Automatic"}], "}"}]}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", 
      RowBox[{"Lp", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "3"}], ",", "4"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", 
          RowBox[{"Dashing", "[", "0.02", "]"}]}], 
         RowBox[{"(*", 
          RowBox[{",", "Thick"}], "*)"}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(k\), \(a\)]\) (pN/\[Mu]m)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(L\), \(\(p\)\(\\\ \)\)]\)(\[Mu]m)\>\""}], 
       "}"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.68891375364622*^9, 3.6889137765017567`*^9}, {
  3.688913809513376*^9, 3.6889138839512367`*^9}, {3.688914155250214*^9, 
  3.688914160944457*^9}, {3.6901222056298637`*^9, 3.690122361592531*^9}, {
  3.690122398256054*^9, 3.690122408792276*^9}, {3.694827617276205*^9, 
  3.694827617723269*^9}, {3.706962376029489*^9, 3.706962457539626*^9}}],

Cell[BoxData["kllpplt"], "Input",
 CellChangeTimes->{{3.673706769985362*^9, 3.673706787642914*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</4B.pdf\>\""}], ",", "kllpplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.688914065362533*^9, 3.688914094065682*^9}, {
   3.69004793295574*^9, 3.6900479475546713`*^9}, {3.7023227741018047`*^9, 
   3.702322774276829*^9}, {3.7045601684930573`*^9, 3.704560169404831*^9}, 
   3.7045604924961243`*^9, {3.7069624742482977`*^9, 3.706962487490484*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["C", "Subsection",
 CellChangeTimes->{{3.673690546043909*^9, 3.6736905483629208`*^9}, {
   3.673703705247267*^9, 3.673703705294692*^9}, {3.673709005096376*^9, 
   3.6737090053761044`*^9}, 3.6814912924097013`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dxdir", "=", 
   RowBox[{"vfdir", "<>", "\"\</pl/dx_arr_kldx1/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dxs", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.08", ",", "0.1", ",", "0.2", ",", "0.25", ",", "0.4", ",", "0.5", ",", 
     "0.8", ",", "1", ",", "2", ",", "2.5", ",", "5"}], "}"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"Ls", "=", 
   RowBox[{"nlks", "*", "dxs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dxdirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dxdir", "<>", "\"\</dx\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "2"}], "}"}], ",", 
         RowBox[{"ExponentFunction", "\[Rule]", 
          RowBox[{"(", 
           RowBox[{"Null", "&"}], ")"}]}]}], "]"}], "]"}]}], "&"}], "/@", 
    "dxs"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Partition", "[", 
        RowBox[{"#", ",", "nlks"}], "]"}], "&"}], "/@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"pts", "[", 
         RowBox[{
          RowBox[{"dxdirs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "\"\<links\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"teq", ";;", 
          RowBox[{"-", "2"}], ";;", "2"}], "]"}], "]"}], ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "dxs", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"lps", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"plMeanThsq", "[", 
         RowBox[{
          RowBox[{"lks", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "All", ",", "j", ",", "All", ",", 
             RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", 
          RowBox[{"dxs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", 
          RowBox[{"Ls", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "maxl"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", 
          RowBox[{"Length", "[", 
           RowBox[{"lks", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "lks", "]"}]}], "}"}]}], "]"}]}], ";"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"lpmeans", "=", 
   RowBox[{"Mean", "/@", "lps"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "lps"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.688912702925228*^9, 3.688912777908717*^9, {3.688912836186125*^9, 
   3.688912872664366*^9}, {3.6889141888718233`*^9, 3.688914192078882*^9}, {
   3.688914232999997*^9, 3.68891423819844*^9}, {3.6889156079958963`*^9, 
   3.688915628410925*^9}, {3.688915712241868*^9, 3.6889157910153093`*^9}, {
   3.688916481431432*^9, 3.688916481702526*^9}, {3.688917041432994*^9, 
   3.688917045319569*^9}, {3.688920465913409*^9, 3.6889204823842688`*^9}, 
   3.688920688443486*^9, 3.690047959754725*^9, {3.690048864204405*^9, 
   3.690048927851812*^9}, {3.6907135094297943`*^9, 3.6907135096447783`*^9}, {
   3.690715087404726*^9, 3.690715087611973*^9}, {3.690715944335157*^9, 
   3.690715973365397*^9}, {3.690716200968707*^9, 3.690716256390687*^9}, {
   3.690716574055274*^9, 3.690716581275573*^9}, {3.690718523429895*^9, 
   3.690718526157374*^9}, {3.690718648683062*^9, 3.690718654809513*^9}, 
   3.690718748600213*^9, {3.694827643074876*^9, 3.6948276656675997`*^9}, 
   3.694827952485569*^9, 3.6949594067003107`*^9, 3.7023217103286333`*^9, 
   3.70462525349681*^9, {3.706962502280634*^9, 3.706962505081155*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dxlpplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListLogLinearPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", "lpmeans"}], "}"}], "\[Transpose]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", 
            RowBox[{"lpmeans", "-", "lperrs"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", 
            RowBox[{"lpmeans", "+", "lperrs"}]}], "}"}], "\[Transpose]"}]}], 
        "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Disk", "[", "]"}], "}"}], "]"}], ",", "0.045"}], "}"}],
           ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Black", ",", "Black"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Black", ",", "Red", ",", "Red"}], "}"}]}], ","}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0.09", ",", "1.2"}], "}"}], ",", "rngy"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"2", "\[Rule]", 
            RowBox[{"{", "3", "}"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"5", "\[Rule]", 
            RowBox[{"{", "6", "}"}]}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"LogLinearPlot", "[", 
      RowBox[{"Lp", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0.001", ",", "10000"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", 
          RowBox[{"Dashing", "[", "0.02", "]"}]}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(l\), \(a\)]\) (\[Mu]m)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(L\), \(\(p\)\(\\\ \)\)]\)(\[Mu]m)\>\""}], 
       "}"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.688915903573367*^9, 3.688915957026971*^9}, {
  3.68891659836347*^9, 3.68891663991546*^9}, {3.690123543371933*^9, 
  3.6901235504030657`*^9}, {3.690715379838122*^9, 3.690715439348922*^9}, {
  3.69071651445778*^9, 3.690716528856823*^9}, {3.694827960550668*^9, 
  3.694827982199066*^9}, {3.6949597004147263`*^9, 3.69495970074914*^9}, {
  3.6949597643754797`*^9, 3.694959780165606*^9}, {3.7048008806262207`*^9, 
  3.704800881299675*^9}, {3.706963324238494*^9, 3.706963337729466*^9}}],

Cell[BoxData["dxlpplt"], "Input",
 CellChangeTimes->{{3.6949596619064913`*^9, 3.694959663684731*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</4C.pdf\>\""}], ",", "dxlpplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.694959802179838*^9, {3.702322737623075*^9, 3.7023227660775547`*^9}, {
   3.704560720550856*^9, 3.7045607216263103`*^9}, {3.706963362193578*^9, 
   3.706963374841194*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["D", "Subsection",
 CellChangeTimes->{{3.673690546043909*^9, 3.6736905483629208`*^9}, {
   3.673703705247267*^9, 3.673703705294692*^9}, 3.681491172652049*^9, 
   3.70232214653354*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dtdir", "=", 
   RowBox[{"fig3dir", "<>", "\"\</dt_arr/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dts", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"(*", 
      RowBox[{"1", ",", "2", ",", "5", ","}], "*)"}], 
     RowBox[{
     "10", ",", "20", ",", "50", ",", "100", ",", "200", ",", "500", ",", 
      "1000", ",", "2000", ",", "5000", ",", "10000", ",", "20000", ",", 
      "50000"}], "}"}], "*", 
    RowBox[{"10.", "^", 
     RowBox[{"(", 
      RowBox[{"-", "7"}], ")"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dtdirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dtdir", "<>", "\"\</dt\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "7"}], "}"}], ",", 
         RowBox[{"ExponentFunction", "\[Rule]", 
          RowBox[{"(", 
           RowBox[{"Null", "&"}], ")"}]}]}], "]"}], "]"}]}], "&"}], "/@", 
    "dts"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.688912702925228*^9, 3.688912777908717*^9, {3.688912836186125*^9, 
   3.688912872664366*^9}, {3.6889141888718233`*^9, 3.688914192078882*^9}, {
   3.688914232999997*^9, 3.68891423819844*^9}, {3.6899706570759478`*^9, 
   3.6899706675464277`*^9}, {3.689970781632534*^9, 3.689970788561544*^9}, {
   3.6899709458298683`*^9, 3.689970949626053*^9}, {3.690046539613387*^9, 
   3.690046539719692*^9}, {3.690046655479246*^9, 3.69004665867372*^9}, {
   3.6900467364483757`*^9, 3.690046736711817*^9}, 3.690046917500307*^9, {
   3.69004880609512*^9, 3.690048834452615*^9}, {3.694827508706661*^9, 
   3.694827554594385*^9}, 3.6949590502082987`*^9, {3.702060309967326*^9, 
   3.702060311094297*^9}, 3.7020603420833693`*^9, {3.70232214902923*^9, 
   3.702322175148139*^9}, {3.7023222201800327`*^9, 3.702322292946456*^9}, {
   3.7041133780053864`*^9, 3.704113392037348*^9}, 3.704800988791394*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Partition", "[", 
        RowBox[{"#", ",", "nlks"}], "]"}], "&"}], "/@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"pts", "[", 
         RowBox[{
          RowBox[{"dtdirs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "\"\<links\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"teq", ";;", 
          RowBox[{"-", "1"}], ";;", "1"}], "]"}], "]"}], ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "dts", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lps", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"plMeanThsq", "[", 
        RowBox[{
         RowBox[{"lks", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "All", ",", "j", ",", "All", ",", 
            RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", "dx", ",", "L", 
         ",", "maxl"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"lks", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "lks", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lpmeans", "=", 
   RowBox[{"Mean", "/@", "lps"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "lps"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.6889128469143953`*^9, {3.688912881104621*^9, 3.6889128813852463`*^9}, 
   3.690046899139833*^9, {3.694827755511664*^9, 3.6948277971819277`*^9}, 
   3.694827871986713*^9, {3.6948794221873827`*^9, 3.694879424771328*^9}, {
   3.694959024823882*^9, 3.694959055496361*^9}, 3.694959434300005*^9, {
   3.702322306971385*^9, 3.702322309808751*^9}, {3.704114033320684*^9, 
   3.70411403602185*^9}, 3.704114707573202*^9, {3.7041149429268417`*^9, 
   3.7041149434696836`*^9}, {3.704560735264328*^9, 3.704560735983107*^9}, 
   3.704561168220978*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dtlpplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Log10", "[", "dts", "]"}], ",", "lpmeans"}], "}"}], 
          "\[Transpose]"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Log10", "[", "dts", "]"}], ",", 
            RowBox[{"lpmeans", "-", "lperrs"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Log10", "[", "dts", "]"}], ",", 
            RowBox[{"lpmeans", "+", "lperrs"}]}], "}"}], "\[Transpose]"}]}], 
        "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Disk", "[", "]"}], "}"}], "]"}], ",", "0.045"}], "}"}],
           ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Black", ",", "Black"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Black", ",", "Red", ",", "Red"}], "}"}]}], ","}], 
        "\[IndentingNewLine]", "*)"}], 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "6.1"}], ",", 
            RowBox[{"-", "2.6"}]}], "}"}], ",", "rngy"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"2", "\[Rule]", 
            RowBox[{"{", "3", "}"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"5", "\[Rule]", 
            RowBox[{"{", "6", "}"}]}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ticksx", ",", "Automatic"}], "}"}]}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", 
      RowBox[{"Lp", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "7"}], ",", "4"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", 
          RowBox[{"Dashing", "[", "0.02", "]"}]}], 
         RowBox[{"(*", 
          RowBox[{",", "Thick"}], "*)"}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\[CapitalDelta]t (s)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(L\), \(\(p\)\(\\\ \)\)]\)(\[Mu]m)\>\""}], 
       "}"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.68891375364622*^9, 3.6889137765017567`*^9}, {
   3.688913809513376*^9, 3.6889138839512367`*^9}, {3.688914155250214*^9, 
   3.688914160944457*^9}, {3.6901222056298637`*^9, 3.690122361592531*^9}, {
   3.690122398256054*^9, 3.690122408792276*^9}, {3.694827617276205*^9, 
   3.694827617723269*^9}, {3.702322368648869*^9, 3.702322395624017*^9}, {
   3.702322637075436*^9, 3.70232269980949*^9}, 3.7041146822594013`*^9, 
   3.704561147486559*^9, 3.7045613984962263`*^9, {3.704801205226903*^9, 
   3.704801213559792*^9}, {3.70696361947075*^9, 3.7069636259228277`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</4D.pdf\>\""}], ",", "dtlpplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.688914065362533*^9, 3.688914094065682*^9}, {
  3.69004793295574*^9, 3.6900479475546713`*^9}, {3.702322713983315*^9, 
  3.702322727902278*^9}, {3.7045611564121523`*^9, 3.704561156835608*^9}, {
  3.7069636403069773`*^9, 3.706963664240369*^9}}],

Cell[BoxData["dtlpplt"], "Input",
 CellChangeTimes->{{3.7069636474297256`*^9, 3.706963649272931*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Figures 5, S3, S5", "Section",
 CellChangeTimes->{{3.681491555839826*^9, 3.681491567795499*^9}, {
  3.681495644388301*^9, 3.681495650533968*^9}, {3.694828717467196*^9, 
  3.694828719428822*^9}, {3.704799209616989*^9, 3.704799212479219*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig5dir", "<>", "\"\</kcl1000.0\>\""}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Gamma]s", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0.001", ",", "0.5", ",", "0.001"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"s", "=", "0.5"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"boxsz", "=", "75"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fov", "=", 
   RowBox[{"boxsz", 
    RowBox[{"{", 
     RowBox[{"1", ",", "1"}], "}"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"pr", "=", 
   RowBox[{"boxsz", "*", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.5"}], ",", "0.5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.5"}], ",", "0.5"}], "}"}]}], "}"}]}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.7032520460400953`*^9, 3.703252053087636*^9}}],

Cell[CellGroupData[{

Cell["A", "Subsection",
 CellChangeTimes->{{3.68149340936126*^9, 3.681493422710394*^9}}],

Cell[BoxData[
 RowBox[{"\n", 
  RowBox[{
   RowBox[{
    RowBox[{"lks", "=", 
     RowBox[{"pts", "[", 
      RowBox[{"dir", ",", "\"\<links\>\""}], "]"}]}], ";"}], "\n", "\n", 
   RowBox[{
    RowBox[{"engs", "=", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Norm", "/@", 
        RowBox[{"Flatten", "[", 
         RowBox[{
          RowBox[{"lks", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "All", ",", 
             RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", "1"}], "]"}]}], 
       "-", "1"}], ")"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"englims", "=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.025"}], ",", "0.025"}], "}"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.6814934584469213`*^9, 3.681493464006775*^9}, {
   3.6814984787599497`*^9, 3.681498479279214*^9}, {3.68149851228747*^9, 
   3.681498524550028*^9}, {3.6814988029272547`*^9, 3.681498803334526*^9}, {
   3.682158820793499*^9, 3.682158827700979*^9}, {3.682197752770117*^9, 
   3.6821977697626953`*^9}, 3.6821978403759823`*^9, {3.690887965868389*^9, 
   3.6908879730351686`*^9}, {3.690888683930387*^9, 3.6908886840571833`*^9}, {
   3.694893989182786*^9, 3.694893997646483*^9}, {3.694964302769702*^9, 
   3.694964302956278*^9}, {3.702818064662015*^9, 3.702818066277692*^9}, {
   3.703252021426206*^9, 3.7032520430872927`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"fs0", "=", 
   RowBox[{"drawStretch", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"lks", "[", 
       RowBox[{"[", "21", "]"}], "]"}], "}"}], ",", 
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}], ",", "\"\<\>\"", ",", "False", ",", "fov", ",", "1", 
     ",", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", "s", ",", "1", ",", "True", ",", 
     "englims"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fs1", "=", 
   RowBox[{"drawStretch", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"lks", "[", 
       RowBox[{"[", "51", "]"}], "]"}], "}"}], ",", 
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}], ",", "\"\<\>\"", ",", "False", ",", "fov", ",", "1", 
     ",", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", "s", ",", "1", ",", "True", ",", 
     "englims"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fs2", "=", 
   RowBox[{"drawStretch", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"lks", "[", 
       RowBox[{"[", 
        RowBox[{"-", "1"}], "]"}], "]"}], "}"}], ",", 
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", "}"}], ",", "\"\<\>\"", ",", "False", ",", "fov", ",", "1", 
     ",", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", "s", ",", "1", ",", "True", ",", 
     "englims"}], "]"}]}], ";"}], "\n", 
 RowBox[{"{", 
  RowBox[{"fs0", ",", "fs1", ",", "fs2"}], "}"}]}], "Input",
 CellChangeTimes->{{3.6585921315493717`*^9, 3.658592151882937*^9}, {
   3.658592200282206*^9, 3.6585922005455227`*^9}, {3.658676168823683*^9, 
   3.65867616915211*^9}, {3.66887173911327*^9, 3.668871745441071*^9}, 
   3.6691127853308687`*^9, {3.669112831519607*^9, 3.6691128343403883`*^9}, {
   3.669486122450152*^9, 3.66948614882583*^9}, {3.669486193059435*^9, 
   3.669486216659259*^9}, {3.670081566706023*^9, 3.670081591433515*^9}, {
   3.675689414197875*^9, 3.6756894195441713`*^9}, {3.675690864611155*^9, 
   3.6756908754349947`*^9}, {3.681493485671204*^9, 3.681493501945582*^9}, 
   3.6814935412741947`*^9, {3.6814936597946987`*^9, 3.681493687921405*^9}, {
   3.681496235831003*^9, 3.681496239199255*^9}, 3.681498492881922*^9, 
   3.682158799806699*^9, {3.68219751772723*^9, 3.682197556055252*^9}, {
   3.682197710594503*^9, 3.682197728016655*^9}, {3.682197831475944*^9, 
   3.682197854081263*^9}, {3.690053138344769*^9, 3.690053149007868*^9}, {
   3.690053294701885*^9, 3.690053295916079*^9}, {3.6900533978343353`*^9, 
   3.690053398601807*^9}, {3.702818027792087*^9, 3.702818029318384*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Atop.pdf\>\""}], ",", 
    RowBox[{"Show", "[", "fs0", "]"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Amid.pdf\>\""}], ",", 
    RowBox[{"Show", "[", "fs1", "]"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Abot.pdf\>\""}], ",", 
    RowBox[{"Show", "[", "fs2", "]"}]}], "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.690053414607089*^9, 3.69005348374329*^9}, {
  3.69005354168607*^9, 3.6900535545332623`*^9}, {3.702817992375635*^9, 
  3.702818001622951*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsection",
 CellChangeTimes->{{3.681493412272726*^9, 3.6814934279417753`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"strain", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"ConstantArray", "[", 
       RowBox[{
        RowBox[{"0.001", "*", "i"}], ",", "20"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", "500"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"te", "=", 
   RowBox[{"Total", "[", 
    RowBox[{"Transpose", "[", 
     RowBox[{"Import", "[", 
      RowBox[{
       RowBox[{"dir", "<>", "\"\</data/pe.txt\>\""}], ",", "\"\<TSV\>\""}], 
      "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"rng0", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"1", ",", "101"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rng0b", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"1", ",", "101", ",", "5"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rng1", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"2000", ",", "2100", ",", "5"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rng2", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"5000", ",", "5100", ",", "5"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rng3", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"8000", ",", "8100", ",", "5"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ClearAll", "[", "tickfcn", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tickfcn", "[", 
   RowBox[{"min_", ",", "max_", ",", 
    RowBox[{"nticks_:", "50"}], ",", 
    RowBox[{"nlabels_:", "5"}]}], "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"x", " ", 
            RowBox[{"nlabels", "/", 
             RowBox[{"(", 
              RowBox[{"max", "-", "min"}], ")"}]}]}], ",", "1"}], "]"}], 
         "\[Equal]", "0"}], ",", 
        RowBox[{"ToString", "[", 
         RowBox[{"N", "[", "x", "]"}], "]"}], ",", "\"\<\>\""}], "]"}]}], 
     "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "min", ",", "max", ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"max", "-", "min"}], ")"}], "/", 
       RowBox[{"(", "nticks", ")"}]}]}], "}"}]}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{"ts", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", "0.49999", ",", 
     RowBox[{"0.5", "/", "10000"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"p1", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"ts", "[", 
         RowBox[{"[", "rng0b", "]"}], "]"}], ",", 
        RowBox[{"te", "[", 
         RowBox[{"[", "rng1", "]"}], "]"}]}], "}"}], "\[Transpose]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", "Blue"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\[FilledCircle]", ",", "10"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"None", ",", 
          RowBox[{"{", 
           RowBox[{"58", ",", "63"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"None", ",", "None"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Frame", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"False", ",", "True"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"False", ",", "False"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "350"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", 
      RowBox[{"0.8", "/", "3"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Thick", ",", "Blue"}], "}"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"p2", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"ts", "[", 
         RowBox[{"[", "rng0b", "]"}], "]"}], ",", 
        RowBox[{"te", "[", 
         RowBox[{"[", "rng2", "]"}], "]"}]}], "}"}], "\[Transpose]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", "Blue"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\[FilledSquare]", ",", "10"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"None", ",", 
          RowBox[{"{", 
           RowBox[{"1480", ",", "1520", ",", "1560"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"None", ",", "None"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Frame", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"False", ",", "True"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"False", ",", "False"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "350"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", 
      RowBox[{"0.8", "/", "3"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
        RowBox[{"Style", "[", 
         RowBox[{
          RowBox[{"Rotate", "[", 
           RowBox[{
           "\"\<\!\(\*SubscriptBox[\(U\), \(tot\)]\) (pN\[Mu]m)\>\"", ",", 
            "Pi"}], "]"}], ",", "Blue"}], "]"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Thick", ",", "Blue"}], "}"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"p3", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"ts", "[", 
         RowBox[{"[", "rng0b", "]"}], "]"}], ",", 
        RowBox[{"te", "[", 
         RowBox[{"[", "rng3", "]"}], "]"}]}], "}"}], "\[Transpose]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", "Automatic"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", "Blue"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\[FilledUpTriangle]", ",", "10"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"None", ",", 
          RowBox[{"{", 
           RowBox[{"8850", ",", "9000", ",", "9150"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"None", ",", "None"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Frame", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"False", ",", "True"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"False", ",", "False"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "350"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", 
      RowBox[{"0.8", "/", "3"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Thick", ",", "Blue"}], "}"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"strplt", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"ts", "[", 
          RowBox[{"[", "rng0", "]"}], "]"}], "*", "1000"}], ",", 
        RowBox[{"strain", "[", 
         RowBox[{"[", "rng0", "]"}], "]"}]}], "}"}], "\[Transpose]"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Black", ",", "Dashed", ",", "Thick"}], "}"}]}], ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"FontSize", "\[Rule]", "22"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"tickfcn", "[", 
           RowBox[{"0.000", ",", "0.006", ",", "30", ",", "6"}], "]"}], ",", 
          "None"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"(*", 
          RowBox[{"tickfcn", "[", 
           RowBox[{"0", ",", "10", ",", "50", ",", "5"}], "]"}], "*)"}], 
         RowBox[{"All", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Style", "[", 
         RowBox[{"\"\<t-\!\(\*SubscriptBox[\(t\), \(0\)]\)(ms)\>\"", ",", 
          RowBox[{"SingleLetterItalics", "\[Rule]", "False"}]}], "]"}], ",", 
        "\"\<\[Gamma]-\!\(\*SubscriptBox[\(\[Gamma]\), \(0\)]\)\>\"", ",", 
        "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Black", ",", 
          RowBox[{"{", 
           RowBox[{"Blue", ",", 
            RowBox[{"Dashing", "[", "0.01", "]"}]}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Black"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"AspectRatio", "\[Rule]", "1"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Frame", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"True", ",", "True"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"True", ",", "True"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImagePadding", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "100"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "400"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.669114076343569*^9, 3.669114155201116*^9}, {
   3.669486864391608*^9, 3.669486877871217*^9}, {3.669487069255987*^9, 
   3.669487070391588*^9}, {3.6814938785079823`*^9, 3.681493913961915*^9}, {
   3.6814986240418043`*^9, 3.681498655445121*^9}, 3.681498730517428*^9, {
   3.681498832101923*^9, 3.68149883254891*^9}, {3.690053917571404*^9, 
   3.690053944361063*^9}, {3.690054094123358*^9, 3.69005419603797*^9}, {
   3.690054250773628*^9, 3.69005436790626*^9}, {3.6901348878570213`*^9, 
   3.690134912137517*^9}, 3.6901405573524837`*^9, {3.690142440706418*^9, 
   3.69014244930867*^9}, {3.694894066655815*^9, 3.694894144787331*^9}, {
   3.702813488750195*^9, 3.702813512514016*^9}, {3.7028135811978416`*^9, 
   3.702813602940271*^9}, {3.703253390017007*^9, 3.703253472840579*^9}, {
   3.703253525477181*^9, 3.7032536387921352`*^9}, {3.704801488503518*^9, 
   3.704801509942954*^9}}],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Overlay", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Overlay", "[", 
         RowBox[{
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Overlay", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"strplt", ",", "\[IndentingNewLine]", 
                RowBox[{"Show", "[", 
                 RowBox[{"p1", ",", 
                  RowBox[{"ImageSize", "\[Rule]", "s1"}]}], "]"}]}], "}"}], 
              ",", 
              RowBox[{"Alignment", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"x1", ",", "y1"}], "}"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Show", "[", 
             RowBox[{"p2", ",", 
              RowBox[{"ImageSize", "\[Rule]", "s2"}]}], "]"}]}], "}"}], ",", 
          RowBox[{"Alignment", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"x2", ",", "y2"}], "}"}]}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Show", "[", 
         RowBox[{"p3", ",", 
          RowBox[{"ImageSize", "\[Rule]", "s3"}]}], "]"}]}], "}"}], ",", 
      RowBox[{"Alignment", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"x3", ",", "y3"}], "}"}]}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"s1", ",", "100", ",", "400"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x1", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y1", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"s2", ",", "100", ",", "400"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x2", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y2", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"s3", ",", "100", ",", "400"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x3", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y3", ",", 
      RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "\[IndentingNewLine]", "]"}], 
  "\[IndentingNewLine]", "*)"}]], "Input",
 CellChangeTimes->{{3.690141111758563*^9, 3.6901413022917967`*^9}, {
  3.690141970844597*^9, 3.690142019235182*^9}}],

Cell[BoxData[
 RowBox[{"plt", "=", 
  RowBox[{
   RowBox[{"Overlay", "[", 
    RowBox[{
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Overlay", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Overlay", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"strplt", ",", "\[IndentingNewLine]", 
               RowBox[{"Show", "[", 
                RowBox[{"p1", ",", 
                 RowBox[{"ImageSize", "\[Rule]", "s1"}]}], "]"}]}], "}"}], 
             ",", 
             RowBox[{"Alignment", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"x1", ",", "y1"}], "}"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Show", "[", 
            RowBox[{"p2", ",", 
             RowBox[{"ImageSize", "\[Rule]", "s2"}]}], "]"}]}], "}"}], ",", 
         RowBox[{"Alignment", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"x2", ",", "y2"}], "}"}]}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Show", "[", 
        RowBox[{"p3", ",", 
         RowBox[{"ImageSize", "\[Rule]", "s3"}]}], "]"}]}], "}"}], ",", 
     RowBox[{"Alignment", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"x3", ",", "y3"}], "}"}]}]}], "]"}], "/.", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"s1", "\[Rule]", "235"}], ",", 
     RowBox[{"s2", "\[Rule]", "298"}], ",", 
     RowBox[{"s3", "\[Rule]", "262"}], ",", "\[IndentingNewLine]", 
     RowBox[{"x1", "\[Rule]", "0.165"}], ",", 
     RowBox[{"x2", "\[Rule]", "0.84"}], ",", 
     RowBox[{"x3", "\[Rule]", "0.365"}], ",", "\[IndentingNewLine]", 
     RowBox[{"y1", "\[Rule]", 
      RowBox[{"-", "0.49"}]}], ",", 
     RowBox[{"y2", "\[Rule]", "0.155"}], ",", 
     RowBox[{"y3", "\[Rule]", "0.7"}]}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.690141630611163*^9, 3.6901417003737926`*^9}, {
  3.690141800459571*^9, 3.690141800896302*^9}, {3.690142227270644*^9, 
  3.6901422634516153`*^9}, {3.694964511100893*^9, 3.694964515466428*^9}, {
  3.694964686813381*^9, 3.694964696923881*^9}, {3.694964756599517*^9, 
  3.694964762682787*^9}, {3.702812999857071*^9, 3.702813036734953*^9}, {
  3.7028137908849783`*^9, 3.7028138395470047`*^9}, {3.7032537238933353`*^9, 
  3.7032537309084797`*^9}, {3.703253866834004*^9, 3.703253874697195*^9}, {
  3.703253968471147*^9, 3.703254006639045*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5B.png\>\""}], ",", "plt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6901405696515923`*^9, 3.690140590669272*^9}, {
  3.6901407354339457`*^9, 3.690140737737679*^9}, {3.690141810463744*^9, 
  3.69014181066609*^9}, {3.7028138561867323`*^9, 3.702813858049505*^9}, {
  3.703254022533486*^9, 3.703254022749073*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5B.svg\>\""}], ",", "plt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.703254029028881*^9, 3.7032540292295427`*^9}, {
  3.7032630734095173`*^9, 3.7032630746968193`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Bleft.eps\>\""}], ",", "strplt"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Bbot.eps\>\""}], ",", "p1"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Bmid.eps\>\""}], ",", "p2"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Btop.eps\>\""}], ",", "p3"}], "]"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.690054393804923*^9, 3.690054467743232*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["C", "Subsection",
 CellChangeTimes->{{3.681493413362709*^9, 3.681493432386705*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kls", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.1", ",", "0.2", ",", "0.5", ",", "1", ",", "2", ",", "5", ",", "10", 
     ",", "20", ",", "50", ",", "100", ",", "200", ",", "500", ",", "1000"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"fig5dir", "<>", "\"\</kcl\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1"}], "}"}]}], "]"}], "]"}]}], "&"}], "/@", 
    "kls"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dt", "=", 
   RowBox[{"10", "^", 
    RowBox[{"(", 
     RowBox[{"-", "6"}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tfs", "=", "0.5"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"engs", "=", 
   RowBox[{"Transpose", "/@", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</data/pe.txt\>\""}], ",", "\"\<TSV\>\""}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"te", "=", 
   RowBox[{"Total", "/@", "engs"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ts", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", "0.49999", ",", 
     RowBox[{"0.5", "/", "10000"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Gamma]s", "=", 
   RowBox[{
    RowBox[{"Range", "[", 
     RowBox[{"1", ",", "500"}], "]"}], "0.001"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"eqengs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Mean", "/@", 
      RowBox[{"Partition", "[", 
       RowBox[{
        RowBox[{"te", "[", 
         RowBox[{"[", "i", "]"}], "]"}], ",", "20"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "te", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dats", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Gamma]s", "[", 
          RowBox[{"[", 
           RowBox[{"1", ";;", 
            RowBox[{"Length", "[", "#", "]"}]}], "]"}], "]"}], ",", 
         RowBox[{"#", "/", 
          RowBox[{"75", "^", "2"}]}]}], "}"}], "\[Transpose]"}], ")"}], "&"}],
     "/@", "eqengs"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.67014814229431*^9, 3.670148142983172*^9}, {
   3.681496858903926*^9, 3.681496870823559*^9}, 3.681496956996936*^9, 
   3.681498878935136*^9, {3.689971093761306*^9, 3.689971141951737*^9}, {
   3.690812353015519*^9, 3.690812357311297*^9}, {3.690885686526586*^9, 
   3.690885696461915*^9}, {3.690886950501401*^9, 3.6908869607573843`*^9}, {
   3.694964975391388*^9, 3.694965028593811*^9}, {3.704032359452381*^9, 
   3.704032439752611*^9}}],

Cell[BoxData[
 RowBox[{"leg", "=", 
  RowBox[{"logbarleg", "[", 
   RowBox[{"1", ",", "1000", ",", "3", ",", "\"\<BlueGreenYellow\>\"", ",", 
    RowBox[{"ExponentFunction", "\[Rule]", "1"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"LegendLabel", "\[Rule]", 
       RowBox[{"Placed", "[", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(k\), \(xl\)]\)\\t\>\"", ",", "Bottom"}], 
        "]"}]}], ",", 
      RowBox[{"LegendMarkerSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"250", ",", "20"}], "}"}]}], ",", 
      RowBox[{"LabelStyle", "\[Rule]", 
       RowBox[{"Directive", "[", 
        RowBox[{"Black", ",", 
         RowBox[{"FontSize", "\[Rule]", "28"}]}], "]"}]}], ",", 
      RowBox[{"TicksStyle", "\[Rule]", "Black"}], ",", 
      RowBox[{"LegendLayout", "\[Rule]", "\"\<Row\>\""}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.704037290978511*^9, 3.704037437483684*^9}, {
  3.704037736189455*^9, 3.704037736300439*^9}, {3.7040378052622128`*^9, 
  3.704037807451126*^9}, {3.704037865762041*^9, 3.704037881553137*^9}, {
  3.704037948176235*^9, 3.704037948287627*^9}, {3.704037988951354*^9, 
  3.704038086515657*^9}, {3.704038124308465*^9, 3.704038124482617*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"idxs", "=", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "6", ",", "7", ",", "10", ",", "13"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"plt", "=", "\[IndentingNewLine]", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListLogLogPlot", "[", 
      RowBox[{
       RowBox[{"dats", "[", 
        RowBox[{"[", 
         RowBox[{"idxs", ",", 
          RowBox[{"200", ";;"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Frame", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<\[Gamma]\>\"", ",", "\"\<w (pN/\[Mu]m)\>\""}], "}"}]}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{",", 
        RowBox[{"FrameTicks", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"LogTicks", "[", 
              RowBox[{"10", ",", 
               RowBox[{"-", "3"}], ",", "3"}], "]"}], ",", 
             RowBox[{"LogTicks", "[", 
              RowBox[{"10", ",", 
               RowBox[{"-", "3"}], ",", "3", ",", 
               RowBox[{"ShowTickLabels", "\[Rule]", "False"}]}], "]"}]}], 
            "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"Automatic", ",", "Automatic"}], "}"}]}], "}"}]}], ","}], 
       "*)"}], "\[IndentingNewLine]", "]"}], ",", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"2", "x"}], "-", "1.7"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"7", "/", "2"}], "x"}], "+", "2.5"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "4"}], ",", "10"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Darker", "[", "Blue", "]"}], ",", "Dashed", ",", 
            "Thick"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Darker", "[", "Green", "]"}], ",", "Dashed", ",", 
            "Thick"}], "}"}]}], "}"}]}]}], "]"}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
        RowBox[{"FontSize", "\[Rule]", "22"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"AspectRatio", "\[Rule]", "0.8"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.704032497119323*^9, 3.704032501366767*^9}, {
  3.704032585998131*^9, 3.70403266044351*^9}, {3.704032740194627*^9, 
  3.704032798776361*^9}, {3.704037444737568*^9, 3.7040374802023783`*^9}, {
  3.7040388128269167`*^9, 3.704038904791246*^9}, {3.704038952560042*^9, 
  3.7040389868542223`*^9}, {3.704801548522107*^9, 3.704801562436038*^9}}],

Cell[BoxData["plt"], "Input",
 CellChangeTimes->{{3.704032571072145*^9, 3.704032571160887*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5C.eps\>\""}], ",", "plt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.690054491310685*^9, 3.690054492001759*^9}, {
  3.6900545315247*^9, 3.690054539932158*^9}, {3.702813901220479*^9, 
  3.702813920025543*^9}, {3.7040375148181334`*^9, 3.704037515017578*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5Cleg.png\>\""}], ",", "leg"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.7040374978305607`*^9, 3.704037522129353*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["D (continued from C)", "Subsection",
 CellChangeTimes->{{3.6814934151232243`*^9, 3.681493440722681*^9}, {
  3.6814969795562353`*^9, 3.6814969829161987`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"fs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"Fit", "[", 
      RowBox[{"#", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "x"}], "}"}], ",", "x"}], "]"}], "&"}], "/@", 
    RowBox[{"Log", "[", 
     RowBox[{"dats", "[", 
      RowBox[{"[", 
       RowBox[{
        RowBox[{"1", ";;"}], ",", 
        RowBox[{"200", ";;"}]}], "]"}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"exps", "=", 
   RowBox[{"D", "[", 
    RowBox[{"fs", ",", "x"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"expdatsp", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Log10", "[", "kls", "]"}], ",", "exps"}], "}"}], 
    "\[Transpose]"}]}], ";"}], "\n", 
 RowBox[{"plt", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"expdatsp", ",", "\[IndentingNewLine]", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0.7", ",", "4.2"}], "}"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotMarkers", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\[FilledCircle]", ",", "10"}], "}"}]}], ",", 
    RowBox[{"BaseStyle", "\[Rule]", 
     RowBox[{"FontSize", "\[Rule]", "22"}]}], ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\!\(\*SubscriptBox[\(k\), \(xl\)]\) (pN/\[Mu]m)\>\"", ",", 
       "\"\<scaling of w(\[Gamma])\>\""}], "}"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"FrameTicks", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"LogTicks", "[", 
          RowBox[{"10", ",", 
           RowBox[{"-", "2"}], ",", "4"}], "]"}], ",", 
         RowBox[{"LogTicks", "[", 
          RowBox[{"10", ",", 
           RowBox[{"-", "2"}], ",", "4", ",", 
           RowBox[{"ShowTickLabels", "\[Rule]", "False"}]}], "]"}]}], "}"}]}],
       "}"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"AspectRatio", "\[Rule]", "0.8"}], ",", 
    RowBox[{"Axes", "\[Rule]", "None"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.669569247884862*^9, 3.669569275453822*^9}, {
  3.669569427947051*^9, 3.669569476856819*^9}, {3.669570043123513*^9, 
  3.669570045426097*^9}, {3.670155792257201*^9, 3.6701557925523157`*^9}, {
  3.6814969957515173`*^9, 3.6814969990446043`*^9}, {3.689971228722972*^9, 
  3.689971229629282*^9}, {3.6900545610210733`*^9, 3.690054561283923*^9}, {
  3.690902805762682*^9, 3.690902856377733*^9}, {3.6929341931352663`*^9, 
  3.692934194084177*^9}, {3.704038425219461*^9, 3.7040384873380136`*^9}, {
  3.704038555528338*^9, 3.704038581938553*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</5D.eps\>\""}], ",", "plt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6900545480273657`*^9, 3.69005456745955*^9}, {
  3.702813934208479*^9, 3.702813934361259*^9}, {3.704038700732732*^9, 
  3.704038700923842*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["S5 (Continued from C)", "Subsection",
 CellChangeTimes->{{3.69066692281921*^9, 3.690666933423665*^9}, {
  3.694893965670472*^9, 3.694893966974483*^9}, {3.694965159654757*^9, 
  3.694965163014793*^9}, {3.704799221397287*^9, 3.70479922145474*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"stprop", "=", 
   RowBox[{
    RowBox[{"engs", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "1", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "/", 
    RowBox[{"te", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bendprop", "=", 
   RowBox[{
    RowBox[{"engs", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "2", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "/", 
    RowBox[{"te", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xlprop", "=", 
   RowBox[{
    RowBox[{"engs", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "4", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "/", 
    RowBox[{"te", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.69090364707609*^9, 3.690903695858974*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"plt1", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"stprop", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "100"}]}], "]"}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Automatic", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<strain\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(w\), \(actin_stretch\)]\) / w\>\""}], 
       "}"}]}], ",", 
     RowBox[{"DataRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0.5"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"plt2", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"bendprop", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "100"}]}], "]"}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Automatic", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<strain\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(w\), \(actin_bend\)]\) / w\>\""}], "}"}]}], 
     ",", 
     RowBox[{"DataRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0.5"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"plt3", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"xlprop", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "100"}]}], "]"}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Automatic", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<strain\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(w\), \(xl\)]\) / w\>\""}], "}"}]}], ",", 
     RowBox[{"DataRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0.5"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.690903722658888*^9, 3.6909037924971952`*^9}, {
  3.690903850376369*^9, 3.690903905861209*^9}, {3.6909039602768927`*^9, 
  3.6909039647644053`*^9}, {3.690904120753973*^9, 3.690904139825639*^9}, {
  3.690904819607747*^9, 3.690904829203141*^9}, {3.691160053140439*^9, 
  3.6911600685077543`*^9}, {3.703254068244976*^9, 3.7032541381703033`*^9}, {
  3.704801589808386*^9, 3.704801607781682*^9}}],

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"plt1", ",", "plt2", ",", "plt3"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "1000"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.691160085659528*^9, 3.6911601004744883`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5D.pdf\>\""}], ",", "plt3"}], "]"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5E.pdf\>\""}], ",", "plt1"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5F.pdf\>\""}], ",", "plt2"}], "]"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6909048503590393`*^9, 3.690904851261355*^9}, 
   3.6909049748018017`*^9, 3.69116012044575*^9, {3.6949651994567003`*^9, 
   3.6949652668960257`*^9}, {3.702814039574119*^9, 3.702814044425991*^9}, {
   3.70480162938552*^9, 3.7048016320100937`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rngy", "=", 
   RowBox[{"{", 
    RowBox[{"5*^-4", ",", "3.6"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"engdens", "=", 
   RowBox[{"engs", "/", 
    RowBox[{"75", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"p1", " ", "=", " ", 
   RowBox[{"ListLogPlot", "[", 
    RowBox[{
     RowBox[{"engdens", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "1", ",", 
        RowBox[{"2", ";;", 
         RowBox[{"-", "1"}], ";;", "100"}]}], "]"}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<strain\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(w\), \(actin_stretch\)]\) \
(pN/\[Mu]m)\>\""}], "}"}]}], ",", 
     RowBox[{"DataRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0.5"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0.5"}], "}"}], ",", "rngy"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"p2", "=", 
   RowBox[{"ListLogPlot", "[", 
    RowBox[{
     RowBox[{"engdens", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "2", ",", 
        RowBox[{"2", ";;", 
         RowBox[{"-", "1"}], ";;", "100"}]}], "]"}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<strain\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(w\), \(actin_bend\)]\) (pN/\[Mu]m)\>\""}], 
       "}"}]}], ",", 
     RowBox[{"DataRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0.5"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0.5"}], "}"}], ",", "rngy"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"p3", "=", 
   RowBox[{"ListLogPlot", "[", 
    RowBox[{
     RowBox[{"engdens", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "4", ",", 
        RowBox[{"2", ";;", 
         RowBox[{"-", "1"}], ";;", "100"}]}], "]"}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<strain\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(w\), \(xl\)]\) (pN/\[Mu]m)\>\""}], "}"}]}], 
     ",", 
     RowBox[{"DataRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0.5"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0.5"}], "}"}], ",", "rngy"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.691158391899535*^9, 3.691158481852099*^9}, {
  3.691158528261483*^9, 3.691158536836042*^9}, {3.6911587132803583`*^9, 
  3.691158719174069*^9}, {3.691159072949398*^9, 3.691159201547295*^9}, {
  3.691159844601882*^9, 3.6911599636962337`*^9}, {3.691163149698633*^9, 
  3.691163151754773*^9}, {3.691163188074333*^9, 3.691163329821616*^9}, {
  3.694965281961193*^9, 3.694965282321558*^9}, {3.70325418245854*^9, 
  3.7032542042747183`*^9}, {3.7048016170371637`*^9, 3.70480164954401*^9}}],

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p3", ",", "p1", ",", "p2"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "1000"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.691159969485916*^9, 3.691159998581101*^9}, {
  3.694965305930889*^9, 3.694965310470834*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5A.pdf\>\""}], ",", "p3"}], "]"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5B.pdf\>\""}], ",", "p1"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5C.pdf\>\""}], ",", "p2"}], "]"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6909048503590393`*^9, 3.690904851261355*^9}, 
   3.6909049748018017`*^9, 3.69116012044575*^9, {3.6949651994567003`*^9, 
   3.6949652668960257`*^9}, {3.6949653240055933`*^9, 3.694965334960208*^9}, {
   3.702814151803722*^9, 3.702814156488052*^9}, {3.704801660784555*^9, 
   3.704801663115161*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</shear_actin_bend.eps\>\""}], ",", "p2"}], 
   "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.690904056065847*^9, 3.69090405617813*^9}, 
   3.690905009345394*^9, {3.691159862464807*^9, 3.6911598821675386`*^9}, {
   3.691160019149242*^9, 3.69116003124442*^9}, {3.691163361020588*^9, 
   3.6911633725316563`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["S3", "Subsection",
 CellChangeTimes->{{3.681495691520013*^9, 3.681495701735938*^9}, 
   3.69066691767852*^9, 3.6948939633505783`*^9, {3.7047992347148647`*^9, 
   3.704799235084537*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nshears", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{"Outer", "[", 
      RowBox[{"Times", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "10", ",", "100", ",", "1000"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "5", ",", "10"}], "}"}]}], "]"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"figS2dir", "<>", "\"\</nbw\>\"", "<>", 
      RowBox[{"ToString", "[", "n", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "nshears"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dt", "=", 
   RowBox[{"10", "^", 
    RowBox[{"(", 
     RowBox[{"-", "7"}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tfs", "=", 
   RowBox[{"500", "*", "dt", "*", "nshears"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Delta]t", "=", 
   RowBox[{"dt", "*", "1000"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"engs", "=", 
   RowBox[{"Transpose", "/@", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</data/pe.txt\>\""}], ",", "\"\<TSV\>\""}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"te", "=", 
   RowBox[{"Total", "/@", "engs"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ts", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Range", "[", 
      RowBox[{"0", ",", "tf", ",", 
       RowBox[{"Max", "[", 
        RowBox[{"dt", ",", 
         RowBox[{"tf", "/", "10000"}]}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"tf", ",", "tfs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"ts", "[", 
    RowBox[{"[", "5", "]"}], "]"}], "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", 
     RowBox[{"tfs", "[", 
      RowBox[{"[", "5", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"tfs", "[", 
       RowBox[{"[", "5", "]"}], "]"}], "/", "12500"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ls", "=", 
   RowBox[{"Floor", "[", 
    RowBox[{
     RowBox[{"Length", "/@", "ts"}], ",", "100"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"eng1dat", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"ts", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", 
           RowBox[{";;", 
            RowBox[{"ls", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}]}], "]"}], "]"}], ",", 
        RowBox[{"te", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", 
           RowBox[{";;", 
            RowBox[{"ls", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}]}], "]"}], "]"}]}], "}"}], 
      "\[Transpose]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "ls", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dts", "=", 
   RowBox[{
    RowBox[{"eng1dat", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "2", ",", "1"}], "]"}], "]"}], "-", 
    RowBox[{"eng1dat", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "1", ",", "1"}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dshears", "=", 
   RowBox[{"0.5", "/", 
    RowBox[{"Length", "/@", "eng1dat"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"eng2dat", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"ts", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", 
            RowBox[{";;", 
             RowBox[{"ls", "[", 
              RowBox[{"[", "i", "]"}], "]"}]}]}], "]"}], "]"}], "*", 
         RowBox[{
          RowBox[{"dshears", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "/", 
          RowBox[{"dts", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}]}], ",", 
        RowBox[{"te", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", 
           RowBox[{";;", 
            RowBox[{"ls", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}]}], "]"}], "]"}]}], "}"}], 
      "\[Transpose]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "ls", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"logeng2dat", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"ts", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", 
            RowBox[{";;", 
             RowBox[{"ls", "[", 
              RowBox[{"[", "i", "]"}], "]"}]}]}], "]"}], "]"}], "*", 
         RowBox[{
          RowBox[{"dshears", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "/", 
          RowBox[{"dts", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}]}], ",", 
        RowBox[{"Log10", "[", 
         RowBox[{"te", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", 
            RowBox[{";;", 
             RowBox[{"ls", "[", 
              RowBox[{"[", "i", "]"}], "]"}]}]}], "]"}], "]"}], "]"}]}], 
       "}"}], "\[Transpose]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "ls", "]"}]}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6676742929962597`*^9, 3.667674477668132*^9}, {
   3.6676745121897297`*^9, 3.667674550973604*^9}, {3.667675995737681*^9, 
   3.6676760120643806`*^9}, {3.667676072432158*^9, 3.6676761343060513`*^9}, 
   3.6677459885641127`*^9, 3.6677499008736153`*^9, {3.667899629369495*^9, 
   3.6678996341209717`*^9}, {3.668185243457621*^9, 3.6681852494062843`*^9}, {
   3.668185569686734*^9, 3.668185571341756*^9}, 3.676048328945562*^9, {
   3.676048510669063*^9, 3.676048553618248*^9}, 3.6760494134136267`*^9, {
   3.68149582740998*^9, 3.681495869249237*^9}, {3.681496215202948*^9, 
   3.681496221007638*^9}, {3.681498977283567*^9, 3.6814990049294777`*^9}, {
   3.690126568018598*^9, 3.690126569048764*^9}, {3.6901266022716103`*^9, 
   3.690126668749785*^9}, {3.694964883071889*^9, 3.6949648842937326`*^9}, {
   3.694965052352687*^9, 3.694965082286852*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tixy", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", "1", ",", "5"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"etplt2", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"logeng2dat", ",", "\[IndentingNewLine]", 
    RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0.5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1.2", ",", "4.9"}], "}"}]}], "}"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<Strain\>\"", ",", "\"\<Energy (pN\[Mu]m)\>\""}], "}"}]}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"FrameTicks", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"tixy", ",", "Automatic"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"Automatic", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"BaseStyle", "\[Rule]", 
     RowBox[{"FontSize", "\[Rule]", "21"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0.001", ",", "0.5"}], "}"}], ",", "Automatic"}], "}"}]}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.6901266739873753`*^9, 3.690126703675268*^9}, {
  3.6901268099199963`*^9, 3.6901269408770247`*^9}, {3.704801683954442*^9, 
  3.7048016877560883`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S3.pdf\>\""}], ",", "etplt2"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.690126950710375*^9, 3.69012697773464*^9}, {
  3.7028141980090513`*^9, 3.7028141990022593`*^9}, {3.7048016998042183`*^9, 
  3.704801700338776*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Figure 6", "Section",
 CellChangeTimes->{{3.681497027542276*^9, 3.681497045722228*^9}, {
  3.694829435672793*^9, 3.6948294414564*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"kon0", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Rho]0", "=", "4"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"L0", "=", "15"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Lm", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"koff0", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"kend0", "=", "10"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rd0", "=", 
     RowBox[{"kon0", "/", 
      RowBox[{"(", 
       RowBox[{"kon0", "+", "koff0"}], ")"}]}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{"fig6dir", "=", 
     RowBox[{"outdir", "<>", "\"\</vf_r2/motility/koff1_kend10/sd1\>\""}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{
     RowBox[{"fig6dir", "=", 
      RowBox[{
      "outdir", "<>", "\"\</vf_r2/testing/motility/kend100_koff0.1/\>\""}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"fig6dir", "=", 
      RowBox[{
      "outdir", "<>", 
       "\"\</vf_r2/testing/motility/kend10_koff0.1_rc0.03/\>\""}]}], ";"}], 
    "*)"}]}]}]], "Input",
 CellChangeTimes->{{3.675612558758986*^9, 3.675612572014722*^9}, {
   3.6756126893944807`*^9, 3.675612697112899*^9}, {3.675618744809306*^9, 
   3.675618754631495*^9}, {3.675618828599145*^9, 3.6756188299991293`*^9}, {
   3.675618875646193*^9, 3.6756188772686253`*^9}, {3.675618964052663*^9, 
   3.675618964314345*^9}, {3.675679649250039*^9, 3.675679652985635*^9}, {
   3.680368402222104*^9, 3.680368404077057*^9}, {3.680368434574329*^9, 
   3.680368435853072*^9}, {3.680368511038554*^9, 3.6803685174311953`*^9}, 
   3.681638770383143*^9, {3.681641035619134*^9, 3.681641050897395*^9}, {
   3.70178806284307*^9, 3.701788063108918*^9}, {3.7021253551712112`*^9, 
   3.702125361323183*^9}, {3.7022263695164213`*^9, 3.7022263889012537`*^9}, 
   3.7022306978919764`*^9, {3.702230838657016*^9, 3.70223083893617*^9}, {
   3.702296909770903*^9, 3.702296913570635*^9}, {3.702299373332787*^9, 
   3.702299374964019*^9}, 3.702376392460175*^9, 3.702377109900634*^9, {
   3.7026567988352747`*^9, 3.7026567992797213`*^9}, {3.702657002640463*^9, 
   3.702657012209594*^9}, {3.702662290469716*^9, 3.702662321252269*^9}, {
   3.702662603493663*^9, 3.702662612745934*^9}, {3.702662914886793*^9, 
   3.702662915685145*^9}}],

Cell[CellGroupData[{

Cell["A", "Subsection",
 CellChangeTimes->{{3.6814983603361387`*^9, 3.6814983709557962`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"fig6dir", "<>", "\"\</kon_arr/kon\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ks", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.001", ",", "0.002", ",", "0.005", ",", "0.01", ",", "0.02", ",", 
     "0.05", ",", "0.1", ",", "0.2", ",", "0.5", ",", "1", ",", "2"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"idxs", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "6", ",", "9", ",", "11"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"dirs", "=", 
    RowBox[{
     RowBox[{
      RowBox[{"dir", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"NumberForm", "[", 
         RowBox[{"#", ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "3"}], "}"}]}], "]"}], "]"}]}], "&"}], "/@", 
     RowBox[{"(", 
      RowBox[{"ks", "[", 
       RowBox[{"[", "idxs", "]"}], "]"}], ")"}]}]}], ";"}], "\n", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"ks", "=", 
     RowBox[{"Join", "[", 
      RowBox[{"ks1", ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4"}], "}"}]}], "]"}]}], ";"}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"konarr", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"pts", "[", 
      RowBox[{"#", ",", "\"\<actins\>\""}], "]"}], "&"}], "/@", "dirs"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"maxf", "=", "401"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cf", "=", 
   RowBox[{"ColorData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<DeepSeaColors\>\"", ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "maxf"}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"cf2", "[", "i_", "]"}], ":=", 
   RowBox[{"Blend", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Pink", ",", 
       RowBox[{"Darker", "[", "Red", "]"}]}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"i", "-", "1"}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"maxf", "-", "1"}], ")"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"start", "=", "5"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"fs", "=", 
   RowBox[{"Table", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"draw", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"konarr", "[", 
        RowBox[{"[", 
         RowBox[{"j", ",", 
          RowBox[{"{", 
           RowBox[{"i", "+", "start"}], "}"}]}], "]"}], "]"}], ",", 
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "}"}], ",", 
       RowBox[{"dirs", "[", 
        RowBox[{"[", 
         RowBox[{"-", "1"}], "]"}], "]"}], ",", "False", ",", 
       RowBox[{"25", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}]}], ",", "True", 
       ",", "16", ",", 
       RowBox[{"cf2", "[", "i", "]"}]}], "\[IndentingNewLine]", "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", "konarr", "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "maxf", ",", "40"}], "}"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"pics", "=", 
   RowBox[{"Show", "/@", "fs"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"grow", "=", 
   RowBox[{"GraphicsRow", "[", 
    RowBox[{"pics", ",", 
     RowBox[{"Spacings", "\[Rule]", "20"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.668474291818268*^9, 3.668474309594727*^9}, {
   3.668474379163718*^9, 3.668474448837059*^9}, {3.6684744835126553`*^9, 
   3.668474489656275*^9}, {3.675593391741249*^9, 3.6755934465634604`*^9}, {
   3.675598899483252*^9, 3.675598914009637*^9}, {3.675598952850383*^9, 
   3.675598953080596*^9}, 3.675599420549582*^9, 3.6756012399104013`*^9, {
   3.680030120489949*^9, 3.680030160201604*^9}, {3.680030466930999*^9, 
   3.6800305054982862`*^9}, {3.6800305394515457`*^9, 3.680030567827417*^9}, 
   3.680032459645234*^9, {3.6800324975442953`*^9, 3.680032538308353*^9}, {
   3.680032871638942*^9, 3.6800328746149054`*^9}, {3.680449858118917*^9, 
   3.680449864350855*^9}, {3.680449910717959*^9, 3.680449928226161*^9}, {
   3.680608967398597*^9, 3.680608983492928*^9}, {3.68149910201788*^9, 
   3.6814991203021793`*^9}, {3.681499154152726*^9, 3.6814992031243067`*^9}, 
   3.68752781136968*^9, {3.687528026316728*^9, 3.687528033914567*^9}, {
   3.688124159335102*^9, 3.688124201821739*^9}, {3.688124239511241*^9, 
   3.688124268612493*^9}, {3.688124431825286*^9, 3.688124456984035*^9}, {
   3.690076491701336*^9, 3.690076513515217*^9}, {3.690076556475416*^9, 
   3.6900765658324966`*^9}, 3.690076612827382*^9, {3.694828799418363*^9, 
   3.6948288022854023`*^9}, {3.7017881378660707`*^9, 3.701788177616283*^9}, {
   3.702124016709733*^9, 3.7021240402140427`*^9}, {3.702124076588359*^9, 
   3.7021240783396263`*^9}, {3.702227261250248*^9, 3.702227262616824*^9}, 
   3.7022273306967287`*^9, {3.70283048769413*^9, 3.702830488312613*^9}, {
   3.7034617891462183`*^9, 3.703461789302409*^9}, {3.7034618283092813`*^9, 
   3.7034619289717903`*^9}, {3.7048018114205713`*^9, 3.704801811667625*^9}, {
   3.704801880977337*^9, 3.704801888273041*^9}, {3.7048019716080303`*^9, 
   3.704801975095364*^9}, {3.7048020128752604`*^9, 3.704802017235656*^9}, {
   3.704802076990247*^9, 3.704802144785075*^9}, {3.7048021829620113`*^9, 
   3.704802191765074*^9}}],

Cell[BoxData["grow"], "Input",
 CellChangeTimes->{{3.7048020663274918`*^9, 3.704802066912856*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</6A_kon\>\"", "<>", 
     RowBox[{"StringRiffle", "[", 
      RowBox[{
       RowBox[{"ToString", "/@", 
        RowBox[{"ks", "[", 
         RowBox[{"[", "idxs", "]"}], "]"}]}], ",", "\"\<_\>\""}], "]"}], 
     "<>", "\"\<.pdf\>\""}], ",", "grow"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.687531250143922*^9, 3.687531275730743*^9}, {
  3.688124588139453*^9, 3.688124598499362*^9}, {3.6900769306494007`*^9, 
  3.69007697010964*^9}, {3.7026571096795464`*^9, 3.702657115686883*^9}, {
  3.702657253171813*^9, 3.702657274930999*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsection",
 CellChangeTimes->{{3.68149922441538*^9, 3.681499236235417*^9}}],

Cell[CellGroupData[{

Cell["Green", "Subsubsection",
 CellChangeTimes->{{3.6814996892082767`*^9, 3.681499705793477*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir1", "=", 
   RowBox[{"fig6dir", "<>", "\"\</dens_arr/dens\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]s1", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.01", ",", "0.02", ",", "0.05", ",", "0.1", ",", "0.2", ",", "0.5", ",",
      "1", ",", "2", ",", "5"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dir2", "=", 
   RowBox[{"fig6dir", "<>", "\"\</dens_arr_dt2e-5/dens\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]s2", "=", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "6", ",", "7", ",", "9"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dir1", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "2"}], "}"}]}], "]"}], "]"}]}], "&"}], "/@", 
    "\[Rho]s1"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs2", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dir2", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "2"}], "}"}]}], "]"}], "]"}]}], "&"}], "/@", 
    "\[Rho]s2"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"dirs1", "[", 
      RowBox[{"[", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}], "]"}], "]"}], ",", 
     RowBox[{"dirs2", "[", 
      RowBox[{"[", 
       RowBox[{"1", ";;", "2"}], "]"}], "]"}], ",", 
     RowBox[{"dirs1", "[", 
      RowBox[{"[", 
       RowBox[{"{", 
        RowBox[{"-", "1"}], "}"}], "]"}], "]"}], ",", 
     RowBox[{"dirs2", "[", 
      RowBox[{"[", 
       RowBox[{"3", ";;"}], "]"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]s", "=", 
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"\[Rho]s1", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "2"}]}], "]"}], "]"}], ",", 
      RowBox[{"\[Rho]s2", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", "2"}], "]"}], "]"}], ",", 
      RowBox[{"\[Rho]s1", "[", 
       RowBox[{"[", 
        RowBox[{"{", 
         RowBox[{"-", "1"}], "}"}], "]"}], "]"}], ",", 
      RowBox[{"\[Rho]s2", "[", 
       RowBox[{"[", 
        RowBox[{"3", ";;"}], "]"}], "]"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lks1", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"pts", "[", 
      RowBox[{"dr", ",", "\"\<links\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"drs", "=", 
   RowBox[{
    RowBox[{"lks1", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;", 
        RowBox[{"-", "1"}]}], ",", "All", ",", 
       RowBox[{"1", ";;", "2"}]}], "]"}], "]"}], "-", 
    RowBox[{"lks1", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}], ",", "All", ",", 
       RowBox[{"1", ";;", "2"}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"drps1", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"rijP", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", 
         RowBox[{"50", ",", "50"}], "}"}]}], "]"}], "&"}], ",", "drs", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"vs1", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Norm", "[", "#", "]"}], "&"}], ",", "drps1", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"norms", "=", 
    RowBox[{"Map", "[", 
     RowBox[{"Norm", ",", 
      RowBox[{"lks1", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "2"}]}], ",", "All", ",", 
         RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", 
      RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rspar", "=", 
   RowBox[{
    RowBox[{"lks1", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}], ",", "All", ",", 
       RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], "/", "norms"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rsperp", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j", ",", "k", ",", "2"}], "]"}], "]"}]}], ",", 
        RowBox[{"rspar", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j", ",", "k", ",", "1"}], "]"}], "]"}]}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "rspar", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vspar", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"MapThread", "[", 
      RowBox[{"Dot", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"drps1", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", "2"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "drps1", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vsperp", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"MapThread", "[", 
       RowBox[{"Dot", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"drps1", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", 
          RowBox[{"rsperp", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", "2"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "drps1", "]"}]}], "}"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dt", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muvs1", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vs1"}]}], "/", "dt"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"par1", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vspar"}]}], "/", "dt"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"perp1", "=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"par1", "^", "2"}]}], "+", 
     RowBox[{"muvs1", "^", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.667915209112094*^9, 3.6679152112007313`*^9}, {
   3.667915424739209*^9, 3.667915428458887*^9}, {3.667915505552581*^9, 
   3.667915657796867*^9}, {3.667922986161195*^9, 3.6679229928804913`*^9}, {
   3.66834267027363*^9, 3.668342744071603*^9}, {3.668342808806497*^9, 
   3.6683428105821047`*^9}, 3.668343999040887*^9, {3.668380563568387*^9, 
   3.66838056394344*^9}, {3.668380747753365*^9, 3.668380830539136*^9}, {
   3.675602228700328*^9, 3.675602230083921*^9}, 3.675602301466354*^9, {
   3.675602869939434*^9, 3.6756028814910192`*^9}, {3.675602933426928*^9, 
   3.675602951217174*^9}, 3.6756030342163563`*^9, {3.680367548389927*^9, 
   3.680367551206134*^9}, 3.6803680141795*^9, {3.680370386651848*^9, 
   3.6803703890718403`*^9}, {3.6804385137471867`*^9, 3.680438515213071*^9}, {
   3.680549211368311*^9, 3.680549371076475*^9}, {3.681499880850184*^9, 
   3.681499891959676*^9}, {3.6814999582702103`*^9, 3.681499979118753*^9}, {
   3.6815000734514847`*^9, 3.681500078282009*^9}, {3.681500119360219*^9, 
   3.681500135952963*^9}, {3.6815763316817923`*^9, 3.6815764129986753`*^9}, {
   3.6875280878791313`*^9, 3.687528138613545*^9}, {3.6881264095230913`*^9, 
   3.6881264265467663`*^9}, {3.6901272067541018`*^9, 3.690127236129705*^9}, {
   3.692647457513588*^9, 3.692647459234321*^9}, 3.692648472401219*^9, {
   3.692652124691498*^9, 3.6926521311094303`*^9}, {3.694828954965605*^9, 
   3.694828957205843*^9}, {3.701803166670101*^9, 3.7018031696885557`*^9}, {
   3.7018034285701723`*^9, 3.701803463625931*^9}, {3.70212419941276*^9, 
   3.702124208634211*^9}, 3.702125212112328*^9, {3.70222752718828*^9, 
   3.702227573010467*^9}, {3.702230891112865*^9, 3.702230895695334*^9}, 
   3.702296938729805*^9, 3.702301154857498*^9, {3.702376629761366*^9, 
   3.702376629877202*^9}, {3.7026574305206337`*^9, 3.702657439919791*^9}, {
   3.702830543901593*^9, 3.7028305453191767`*^9}, 3.702853890399931*^9, 
   3.702853986992709*^9, 3.7034620405965767`*^9, {3.703462085495676*^9, 
   3.703462119612239*^9}, {3.703462153261634*^9, 3.703462177128085*^9}, {
   3.703462214038376*^9, 3.703462330862363*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Red Curve", "Subsubsection",
 CellChangeTimes->{{3.681500080359632*^9, 3.681500099144232*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir2", "=", 
   RowBox[{"fig6dir", "<>", "\"\</length_arr/nm\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nms", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"2", ",", "26", ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"dirs", "=", 
    RowBox[{
     RowBox[{
      RowBox[{"dir2", "<>", 
       RowBox[{"ToString", "[", "#", "]"}]}], "&"}], "/@", "nms"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"lks2", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"pts", "[", 
       RowBox[{"dr", ",", "\"\<links\>\""}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"drs", "=", 
   RowBox[{
    RowBox[{"lks2", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;", 
        RowBox[{"-", "1"}]}], ",", "All", ",", 
       RowBox[{"1", ";;", "2"}]}], "]"}], "]"}], "-", 
    RowBox[{"lks2", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}], ",", "All", ",", 
       RowBox[{"1", ";;", "2"}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"drps2", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"rijP", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", 
         RowBox[{"50", ",", "50"}], "}"}]}], "]"}], "&"}], ",", "drs", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"vs2", "=", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Norm", "[", "#", "]"}], "&"}], ",", "drps2", ",", 
      RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"norms", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Norm", ",", 
     RowBox[{"lks2", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "2"}]}], ",", "All", ",", 
        RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vspar", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"drps2", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "t", ",", "j"}], "]"}], "]"}], ".", 
       RowBox[{"lks2", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "t", ",", "j", ",", 
          RowBox[{"3", ";;", "4"}]}], "]"}], "]"}]}], "/", 
      RowBox[{"norms", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "t", ",", "j"}], "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "drps2", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "1", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"drps2", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"drps2", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "t"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"muvs2", "=", 
    RowBox[{
     RowBox[{"Mean", "/@", 
      RowBox[{"Flatten", "/@", "vs2"}]}], "/", "dt"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"par2", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vspar"}]}], "/", "dt"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"perp2", "=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"par2", "^", "2"}]}], "+", 
     RowBox[{"muvs2", "^", "2"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.667915209112094*^9, 3.6679152112007313`*^9}, {
   3.667915424739209*^9, 3.667915428458887*^9}, {3.667915505552581*^9, 
   3.667915657796867*^9}, {3.667922986161195*^9, 3.6679229928804913`*^9}, {
   3.668214053860446*^9, 3.6682140573861923`*^9}, {3.6682140997083797`*^9, 
   3.668214187029709*^9}, {3.675603479756071*^9, 3.675603481107647*^9}, {
   3.680367731223073*^9, 3.680367733319559*^9}, {3.680370455152523*^9, 
   3.6803705200803137`*^9}, {3.680371167472022*^9, 3.6803712574238453`*^9}, 
   3.680371313369973*^9, {3.680371488164864*^9, 3.680371513415758*^9}, {
   3.680371545550068*^9, 3.680371547681169*^9}, {3.680613369381633*^9, 
   3.6806133811422253`*^9}, 3.6815001525132627`*^9, {3.681576494069491*^9, 
   3.681576516788685*^9}, {3.681638436405954*^9, 3.681638458212126*^9}, 
   3.687528207204154*^9, {3.6901274423812323`*^9, 3.690127444423362*^9}, {
   3.6926474742091*^9, 3.692647476484346*^9}, 3.6926484663766403`*^9, 
   3.692652105932928*^9, {3.6948289966781187`*^9, 3.694829004140869*^9}, {
   3.7018034956505737`*^9, 3.701803530154902*^9}, {3.7018035676908083`*^9, 
   3.7018035769442263`*^9}, {3.701803657119739*^9, 3.7018036589313507`*^9}, 
   3.702125228159459*^9, {3.702125280940558*^9, 3.7021252826678123`*^9}, {
   3.7022280755501757`*^9, 3.702228084693469*^9}, {3.702230999613474*^9, 
   3.702231014084243*^9}, {3.7022970111681013`*^9, 3.702297045471347*^9}, 
   3.7023011707250233`*^9, {3.702657539508993*^9, 3.702657545810041*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Blue Curve", "Subsubsection",
 CellChangeTimes->{{3.681500091319038*^9, 3.681500102463667*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir1", "=", 
   RowBox[{"fig6dir", "<>", "\"\</kon_arr/kon\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ks1", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.001", ",", "0.002", ",", "0.005", ",", "0.01", ",", "0.02", ",", 
     "0.05", ",", "0.1", ",", "0.2", ",", "0.5", ",", "1", ",", "2"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dir2", "=", 
   RowBox[{"fig6dir", "<>", "\"\</kon_arr_dt1e-5/kon3.000\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dir3", "=", 
   RowBox[{"fig6dir", "<>", "\"\</kon_arr_dt2e-5/kon4.000\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"dir1", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"NumberForm", "[", 
          RowBox[{"#", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "3"}], "}"}]}], "]"}], "]"}]}], "&"}], "/@", 
      "ks1"}], ",", 
     RowBox[{"{", 
      RowBox[{"dir2", ",", "dir3"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ks", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"ks1", ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "4"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"lks3", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"pts", "[", 
       RowBox[{"dr", ",", "\"\<links\>\""}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"drs", "=", 
   RowBox[{
    RowBox[{"lks3", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;", 
        RowBox[{"-", "1"}]}], ",", "All", ",", 
       RowBox[{"1", ";;", "2"}]}], "]"}], "]"}], "-", 
    RowBox[{"lks3", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}], ",", "All", ",", 
       RowBox[{"1", ";;", "2"}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"drps3", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"rijP", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", 
         RowBox[{"50", ",", "50"}], "}"}]}], "]"}], "&"}], ",", "drs", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"vs3", "=", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Norm", "[", "#", "]"}], "&"}], ",", "drps3", ",", 
      RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"norms", "=", 
    RowBox[{"Map", "[", 
     RowBox[{"Norm", ",", 
      RowBox[{"lks3", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "2"}]}], ",", "All", ",", 
         RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", 
      RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"vspar", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"MapThread", "[", 
      RowBox[{"Dot", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"drps3", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"lks3", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", 
             RowBox[{"1", ";;", 
              RowBox[{"-", "2"}]}], ",", "All", ",", 
             RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], "/", 
          RowBox[{"norms", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}]}], "}"}], ",", "2"}], "]"}], ",", 
     
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "drps3", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muvs3", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vs3"}]}], "/", "dt"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"par3", "=", 
    RowBox[{
     RowBox[{"Mean", "/@", 
      RowBox[{"Flatten", "/@", "vspar"}]}], "/", "dt"}]}], ";"}], 
  "\n"}], "\n", 
 RowBox[{
  RowBox[{"perp3", "=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"par3", "^", "2"}]}], "+", 
     RowBox[{"muvs3", "^", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.667915209112094*^9, 3.6679152112007313`*^9}, {
   3.667915424739209*^9, 3.667915428458887*^9}, {3.667915505552581*^9, 
   3.667915657796867*^9}, {3.667922986161195*^9, 3.6679229928804913`*^9}, {
   3.66834267027363*^9, 3.668342744071603*^9}, {3.668342808806497*^9, 
   3.6683428105821047`*^9}, 3.668343999040887*^9, {3.668358108957509*^9, 
   3.668358182781155*^9}, 3.668358232592617*^9, {3.668358264178584*^9, 
   3.6683582715125017`*^9}, {3.668387636216672*^9, 3.668387638184024*^9}, {
   3.675604082596591*^9, 3.675604083572019*^9}, {3.6756041457309723`*^9, 
   3.6756041788844433`*^9}, {3.680367931121681*^9, 3.680367932015705*^9}, {
   3.680367974993856*^9, 3.6803679864740467`*^9}, {3.680368092340272*^9, 
   3.680368187653327*^9}, {3.680371936700832*^9, 3.680371947453092*^9}, {
   3.680371996398251*^9, 3.6803720649852333`*^9}, {3.6806133954057083`*^9, 
   3.6806134008402987`*^9}, 3.681500176672111*^9, {3.681638514333529*^9, 
   3.681638528550832*^9}, {3.681638664640223*^9, 3.681638716446854*^9}, {
   3.687528259242612*^9, 3.687528260090386*^9}, {3.68812498300463*^9, 
   3.68812499376427*^9}, 3.6901274991955223`*^9, {3.692647487369032*^9, 
   3.692647488648117*^9}, 3.692648452937454*^9, 3.692652152260558*^9, {
   3.694966006859468*^9, 3.694966010247151*^9}, {3.701803671052133*^9, 
   3.701803672635474*^9}, 3.701803703156925*^9, {3.7018037643533697`*^9, 
   3.701803765469137*^9}, {3.701803874789773*^9, 3.7018038755426407`*^9}, {
   3.7021253084455442`*^9, 3.702125336489143*^9}, {3.702228541834597*^9, 
   3.702228572336919*^9}, {3.702228630039996*^9, 3.702228632471566*^9}, {
   3.702228740389821*^9, 3.702228800739848*^9}, {3.702231036766732*^9, 
   3.7022310484911747`*^9}, {3.702231079666581*^9, 3.702231129569875*^9}, {
   3.702297080486505*^9, 3.70229708148629*^9}, {3.702297116245792*^9, 
   3.702297137460888*^9}, {3.702297170164551*^9, 3.702297202166655*^9}, 
   3.702297658483254*^9, {3.70229963073497*^9, 3.702299647535821*^9}, 
   3.7023012043897133`*^9, {3.702376725332375*^9, 3.7023767730111837`*^9}, {
   3.702657577772485*^9, 3.702657680468602*^9}, 3.703461974774353*^9, {
   3.703462529667777*^9, 3.70346255670744*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Whole Plot", "Subsubsection",
 CellChangeTimes->{{3.6815002234258423`*^9, 3.681500237705695*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals1", "=", 
   RowBox[{"\[Rho]s", " ", "Lm", " ", "L0", " ", 
    RowBox[{"rd0", "^", "a"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]0", " ", "Lm", 
      RowBox[{"(", 
       RowBox[{"nms", "-", "1"}], ")"}]}], ")"}], " ", 
    RowBox[{"rd0", "^", "a"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals3", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]0", " ", "Lm", " ", "L0"}], ")"}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"ks", "/", 
       RowBox[{"(", 
        RowBox[{"ks", "+", "koff0"}], ")"}]}], ")"}], "^", "a"}]}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"sz", "=", "10"}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tab", "=", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(||\)]\)\>\"", ",", 
       "\"\<\!\(\*SubscriptBox[\(v\), \(\[Perpendicular]\)]\)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Green", ",", 
           RowBox[{"Rectangle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"Directive", "[", 
             RowBox[{"Thick", ",", "Green"}], "]"}], "]"}], ",", "White", ",", 
           RowBox[{"Rectangle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<L\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Red", ",", 
           RowBox[{"Rotate", "[", 
            RowBox[{
             RowBox[{"Rectangle", "[", "]"}], ",", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{"Red", ",", "Thick"}], "}"}], "]"}], ",", "White", ",", 
           RowBox[{"Rotate", "[", 
            RowBox[{
             RowBox[{"Rectangle", "[", "]"}], ",", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(r\), \(D\)]\)\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Disk", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Circle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}]}], "}"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"leg", "=", 
   RowBox[{"Grid", "[", 
    RowBox[{"tab", ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"FontSize", "\[Rule]", "19"}], ",", 
        RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}]}], "}"}]}], ",", 
     RowBox[{"Frame", "\[Rule]", "All"}], ",", 
     RowBox[{"Spacings", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0.2", ",", "0.2"}], "}"}]}], ",", 
     RowBox[{"Background", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"LightGray", ",", "None"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"LightGray", ",", "None"}], "}"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"vplt1", "=", 
   RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals1", ",", "par1"}], "}"}], "\[Transpose]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "2"}]}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals2", ",", "par2"}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals3", ",", "par3"}], "}"}], "\[Transpose]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals1", ",", "perp1"}], "}"}], "\[Transpose]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "2"}]}], "]"}], "]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals2", ",", "perp2"}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals3", ",", "perp3"}], "}"}], "\[Transpose]"}]}], "}"}], 
     ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "->", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Green", ",", 
             RowBox[{"Rectangle", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}],
         ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Red", ",", 
             RowBox[{"Rotate", "[", 
              RowBox[{
               RowBox[{"Rectangle", "[", "]"}], ",", 
               RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}]}], ",", "0.06"}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Blue", ",", 
             RowBox[{"Disk", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}], ",",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{"Green", ",", "Thick"}], "}"}], "]"}], ",", "White", 
             ",", 
             RowBox[{"Rectangle", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}],
         ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{"Red", ",", "Thick"}], "}"}], "]"}], ",", "White", ",", 
             RowBox[{"Rotate", "[", 
              RowBox[{
               RowBox[{"Rectangle", "[", "]"}], ",", 
               RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}]}], ",", "0.06"}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Blue", ",", 
             RowBox[{"Circle", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}]}], 
       "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[EmptySquare]", ",", "15"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[EmptySquare]", ",", "15"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[EmptySquare]", ",", "15"}], "}"}]}], "}"}]}], ","}], 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "Green", ",", "Red", ",", "Blue", ",", "Green", ",", "Red", ",", 
        "Blue"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "27"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1.1"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\!\(\*SubscriptBox[\(l\), \
\(m\)]\)\!\(\*SubscriptBox[\(Lr\), \(D\)]\)\>\"", ",", 
        "\"\<v(\[Mu]m/s)\>\""}], "}"}]}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.669547394333199*^9, 3.669547395335994*^9}, {
   3.669547457910119*^9, 3.6695474619181547`*^9}, {3.669547504293866*^9, 
   3.669547577075108*^9}, {3.66954763002936*^9, 3.669547698552413*^9}, 
   3.669547794719522*^9, {3.669547851732976*^9, 3.669547865916278*^9}, {
   3.669548944417613*^9, 3.669548953800785*^9}, {3.675608241306411*^9, 
   3.675608251281824*^9}, {3.6756082899849358`*^9, 3.6756082947206793`*^9}, {
   3.6756084883561153`*^9, 3.675608497819375*^9}, {3.6756085869548483`*^9, 
   3.675608592088807*^9}, {3.675608721719131*^9, 3.675608723133561*^9}, {
   3.6756087736373672`*^9, 3.67560877929205*^9}, {3.675610506089459*^9, 
   3.675610511831223*^9}, {3.675610548184681*^9, 3.675610550966566*^9}, {
   3.675619149744605*^9, 3.6756192344597*^9}, {3.6756193047316227`*^9, 
   3.675619326497716*^9}, {3.675639750430951*^9, 3.675639804982925*^9}, {
   3.675639835251245*^9, 3.675639856722138*^9}, {3.675639908767729*^9, 
   3.675639920041058*^9}, {3.675639970987195*^9, 3.675639971997177*^9}, {
   3.675640041424684*^9, 3.675640042433385*^9}, {3.675679659211442*^9, 
   3.675679665609633*^9}, 3.675681237937204*^9, {3.67568136630999*^9, 
   3.675681366845221*^9}, 3.675681473683605*^9, {3.6803685073752193`*^9, 
   3.6803685078059797`*^9}, {3.681638741048915*^9, 3.68163877443894*^9}, {
   3.702299668027651*^9, 3.702299686334713*^9}, {3.702299758613212*^9, 
   3.702299807507491*^9}, {3.702299844314913*^9, 3.7022998807616873`*^9}, {
   3.702657816655905*^9, 3.702657826487344*^9}, {3.702773073041666*^9, 
   3.702773088502527*^9}, {3.702830674271605*^9, 3.7028307396932*^9}, 
   3.7034626493818817`*^9, {3.703462688370851*^9, 3.703462688786216*^9}, {
   3.7068991110006847`*^9, 3.70689914560614*^9}, {3.706899261843048*^9, 
   3.706899281675783*^9}, {3.7068995607235737`*^9, 3.706899616633515*^9}, {
   3.706899649650309*^9, 3.706899650704467*^9}, {3.7068997032649193`*^9, 
   3.70689978322962*^9}, {3.706899932459545*^9, 3.706899976185533*^9}, 
   3.70690004360194*^9, {3.70690011446345*^9, 3.7069002007563353`*^9}, {
   3.7069002358767223`*^9, 3.706900256499181*^9}}],

Cell[BoxData["vplt1"], "Input",
 CellChangeTimes->{{3.702297677366201*^9, 3.702297677910553*^9}, {
  3.7048023127263317`*^9, 3.704802313406801*^9}}],

Cell[BoxData[
 RowBox[{"vplt", "=", 
  RowBox[{"Overlay", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"vplt1", ",", "leg"}], "}"}], ",", 
    RowBox[{"Alignment", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0.1", ",", 
       RowBox[{"-", "0.25"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.670250770748762*^9, 3.67025084882266*^9}, {
   3.6702512080113153`*^9, 3.670251208180633*^9}, 3.6702512839694767`*^9, {
   3.670251402266375*^9, 3.6702514153004093`*^9}, {3.675679766135846*^9, 
   3.675679767104607*^9}, {3.675679820910652*^9, 3.6756798668974237`*^9}, {
   3.675680023785091*^9, 3.675680067033328*^9}, {3.680368632409597*^9, 
   3.680368649022732*^9}, {3.680368879915257*^9, 3.6803689516579447`*^9}, {
   3.68036898207446*^9, 3.680368990882536*^9}, {3.68752838021579*^9, 
   3.68752839217188*^9}, 3.68812521777288*^9, {3.6881277389434023`*^9, 
   3.688127758621648*^9}, {3.6901276656993103`*^9, 3.690127703306343*^9}, {
   3.702230749787147*^9, 3.702230772665698*^9}, {3.702231756074345*^9, 
   3.7022317578199244`*^9}, {3.702298321398878*^9, 3.702298333494326*^9}, {
   3.702299907575738*^9, 3.702299934061219*^9}, {3.702377244528946*^9, 
   3.702377277686516*^9}, {3.7026578438046417`*^9, 3.702657857684031*^9}, {
   3.7034626704049797`*^9, 3.703462697082931*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</6B_noleg.eps\>\""}], ",", "vplt1"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.687531320957047*^9, 3.687531356680743*^9}, {
   3.6881277837009983`*^9, 3.688127789396841*^9}, {3.6901277470489264`*^9, 
   3.6901277556082983`*^9}, {3.702230604339601*^9, 3.7022306410785103`*^9}, {
   3.702231765065547*^9, 3.7022317688679543`*^9}, {3.7022319094542913`*^9, 
   3.7022319120373783`*^9}, {3.702298343767055*^9, 3.7022983522611237`*^9}, {
   3.7022999452388906`*^9, 3.702299948957389*^9}, {3.702301327787817*^9, 
   3.702301332474938*^9}, {3.702377297453045*^9, 3.7023773073568068`*^9}, 
   3.702657889467293*^9, {3.702663107816716*^9, 3.702663108040073*^9}, 
   3.702833461595536*^9, 3.7028540346909018`*^9, {3.702854276588339*^9, 
   3.7028542777634497`*^9}, {3.7034627055546083`*^9, 3.703462708496703*^9}, {
   3.7034629069312763`*^9, 3.703462909194191*^9}, {3.706900572338112*^9, 
   3.7069005726327744`*^9}, {3.706900654639339*^9, 3.706900659965949*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["C (Continued from B)", "Subsection",
 CellChangeTimes->{{3.6814992413611526`*^9, 3.681499246798682*^9}, {
  3.6815000286587553`*^9, 3.681500032385903*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.687531320957047*^9, 3.687531356680743*^9}, {
   3.6881277837009983`*^9, 3.688127789396841*^9}, {3.6901277470489264`*^9, 
   3.6901277556082983`*^9}, {3.702230604339601*^9, 3.7022306410785103`*^9}, {
   3.702231765065547*^9, 3.7022317688679543`*^9}, {3.7022319094542913`*^9, 
   3.7022319120373783`*^9}, {3.702298343767055*^9, 3.7022983522611237`*^9}, {
   3.7022999452388906`*^9, 3.702299948957389*^9}, {3.702301327787817*^9, 
   3.702301332474938*^9}, {3.702377297453045*^9, 3.7023773073568068`*^9}, 
   3.702657889467293*^9, 3.702658291153352*^9, 3.7026630868082743`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "If", " ", "Evaluating", " ", "for", " ", "the", " ", "first", " ", 
    "time"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"ms1", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"msdpc", "[", 
        RowBox[{"drps1", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "All", ",", "i"}], "]"}], "]"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", 
         RowBox[{"Length", "[", "drps1", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", "L0"}], "}"}]}], "]"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{
      RowBox[{"fig6dir", "<>", "\"\</dens_msds.dat\>\""}], ",", "ms1"}], 
     "]"}], ";"}], "\n", 
   RowBox[{
    RowBox[{"ms2", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"msdpc", "[", 
        RowBox[{"drps2", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "All", ",", "i"}], "]"}], "]"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", 
         RowBox[{"Length", "[", "drps2", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{
          RowBox[{"nms", "[", 
           RowBox[{"[", "j", "]"}], "]"}], "-", "1"}]}], "}"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{
      RowBox[{"fig6dir", "<>", "\"\</len_msds.dat\>\""}], ",", "ms2"}], "]"}],
     ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ms3", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"msdpc", "[", 
        RowBox[{"drps3", "[", 
         RowBox[{"[", 
          RowBox[{"j", ",", "All", ",", "i"}], "]"}], "]"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", 
         RowBox[{"Length", "[", "drps3", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", "L0"}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{
      RowBox[{"fig6dir", "<>", "\"\</kon_msds.dat\>\""}], ",", "ms3"}], "]"}],
     ";"}]}]}]], "Input",
 CellChangeTimes->{{3.690132841529711*^9, 3.690132855561311*^9}, {
  3.69482904758385*^9, 3.6948290519831142`*^9}, {3.694966193239485*^9, 
  3.6949662010725727`*^9}, {3.701804461474519*^9, 3.701804476875084*^9}, {
  3.701804513874653*^9, 3.701804533104582*^9}, {3.702663119800726*^9, 
  3.70266312711264*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"If", " ", "already", " ", "evaluated"}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"ms1", "=", 
     RowBox[{"ToExpression", "[", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"fig6dir", "<>", "\"\</dens_msds.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ms2", "=", 
     RowBox[{"ToExpression", "[", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"fig6dir", "<>", "\"\</len_msds.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ms3", "=", 
     RowBox[{"ToExpression", "[", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"fig6dir", "<>", "\"\</kon_msds.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.694966140969583*^9, 3.6949662117117147`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "Sort", " ", "MSD", " ", "data", " ", "sets", " ", "by", " ", "their", " ",
     "xval"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"msddats1", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xvals1", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"Range", "[", "2000", "]"}], "*", "dt"}], ",", 
            RowBox[{"Mean", "[", 
             RowBox[{"ms1", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "ms1", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"msddats2", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xvals2", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"Range", "[", "2000", "]"}], "*", "dt"}], ",", 
            RowBox[{"Mean", "[", 
             RowBox[{"ms2", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "ms2", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"msddats3", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xvals3", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"Range", "[", 
              RowBox[{"Length", "[", 
               RowBox[{"ms3", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}], "]"}], "*", 
             "dt"}], ",", 
            RowBox[{"Mean", "[", 
             RowBox[{"ms3", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "ms3", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{"allmsddats", "=", 
     RowBox[{"Sort", "[", 
      RowBox[{"Join", "[", 
       RowBox[{"msddats1", ",", "msddats2", ",", "msddats3"}], "]"}], "]"}]}],
     ";"}], "\n", 
   RowBox[{
    RowBox[{"allxvals", "=", 
     RowBox[{"allmsddats", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "1"}], "]"}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.6695592830546913`*^9, 3.66955938004792*^9}, {
   3.669559747788515*^9, 3.669559780644438*^9}, {3.670252094595261*^9, 
   3.67025212160218*^9}, 3.6756809210013113`*^9, {3.680375396747889*^9, 
   3.6803754921939783`*^9}, {3.6804485571215143`*^9, 3.680448560960495*^9}, {
   3.681640021537374*^9, 3.6816400511485643`*^9}, 3.68164008378819*^9, {
   3.681640123046792*^9, 3.681640143387857*^9}, {3.681641060923131*^9, 
   3.681641067155027*^9}, {3.702773588582571*^9, 3.70277360384953*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xidxs", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"1", ",", "33"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tidxs", "=", 
   RowBox[{"Union", "[", 
    RowBox[{"Round", "[", 
     RowBox[{"10", "^", 
      RowBox[{"Range", "[", 
       RowBox[{"0", ",", 
        RowBox[{"Log10", "[", "200", "]"}], ",", "0.15"}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"logxvals", "=", 
   RowBox[{"Log", "[", 
    RowBox[{"allxvals", "[", 
     RowBox[{"[", "xidxs", "]"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cf", "=", 
   RowBox[{"ColorData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<BlueGreenYellow\>\"", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Min", "[", "logxvals", "]"}], ",", 
        RowBox[{"Max", "[", "logxvals", "]"}]}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"msdplt", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"ListLogLogPlot", "[", 
     RowBox[{
      RowBox[{"allmsddats", "[", 
       RowBox[{"[", 
        RowBox[{"xidxs", ",", "2", ",", "tidxs"}], "]"}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"cf", "/@", "logxvals"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<\[Delta]t (s)\>\"", ",", 
         RowBox[{"Style", "[", 
          RowBox[{
          "\"\<\[LeftAngleBracket]|r(t+\[Delta]t)-r(t)\!\(\*SuperscriptBox[\(|\
\), \(2\)]\)\[RightAngleBracket] (\!\(\*SuperscriptBox[\(\[Mu]m\), \
\(2\)]\))\>\"", ",", 
           RowBox[{"FontSize", "\[Rule]", "20"}]}], "]"}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"PlotMarkers", "\[Rule]", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\[FilledCircle]", ",", "15"}], "}"}], ",", "6"}], "]"}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"Joined", "\[Rule]", "False"}]}], "\[IndentingNewLine]", "]"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"Plot", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"x", "-", "2"}], ",", 
        RowBox[{"2", "x"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"-", "1"}], ",", "200"}], "}"}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", "Thick", ",", 
           RowBox[{"Dashing", "[", "Large", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Orange", ",", "Thick", ",", 
           RowBox[{"Dashing", "[", "Large", "]"}]}], "}"}]}], "}"}]}]}], 
     "]"}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.669559791479904*^9, 3.669559809914703*^9}, {
   3.66955987425764*^9, 3.669559937992004*^9}, {3.66955999335984*^9, 
   3.66956002067796*^9}, {3.6695600555183077`*^9, 3.669560072028809*^9}, {
   3.669560186379463*^9, 3.669560193657913*^9}, {3.669560287464293*^9, 
   3.669560743277512*^9}, {3.669560777591743*^9, 3.669560790388888*^9}, {
   3.6695667581365213`*^9, 3.6695669311940536`*^9}, {3.669566966907544*^9, 
   3.6695669689146*^9}, {3.670252181307631*^9, 3.670252227776977*^9}, {
   3.670252344973756*^9, 3.670252412946159*^9}, 3.670252575976788*^9, {
   3.670252726010165*^9, 3.670252766156967*^9}, 3.670252796423959*^9, {
   3.670252851228488*^9, 3.67025286546266*^9}, 3.670252900118739*^9, 
   3.6702532205690327`*^9, {3.675681046927827*^9, 3.675681049174952*^9}, {
   3.675681293538048*^9, 3.675681295554626*^9}, {3.680373237458084*^9, 
   3.6803732548916407`*^9}, {3.680375716710082*^9, 3.68037573479631*^9}, {
   3.680375799477467*^9, 3.680375959725904*^9}, {3.680376067952094*^9, 
   3.680376082576666*^9}, {3.680376297371682*^9, 3.6803764443712387`*^9}, {
   3.680376652395946*^9, 3.6803767420662403`*^9}, {3.6804485830600986`*^9, 
   3.6804487852280607`*^9}, {3.6804488226675863`*^9, 3.680448886881852*^9}, {
   3.680448937986287*^9, 3.680448987521428*^9}, {3.680449019280162*^9, 
   3.680449079645751*^9}, {3.6816400693473587`*^9, 3.681640072203498*^9}, {
   3.687531089946885*^9, 3.687531095720957*^9}, {3.687531152056962*^9, 
   3.687531155551291*^9}, {3.687531207463706*^9, 3.687531230142337*^9}, {
   3.688128418888012*^9, 3.6881284314295053`*^9}, {3.688131271082498*^9, 
   3.688131301124642*^9}, {3.688131346284482*^9, 3.6881313511796227`*^9}, 
   3.688131395032092*^9, {3.68813147989225*^9, 3.688131589884988*^9}, {
   3.6901335728652067`*^9, 3.690133676455019*^9}, {3.694829060119174*^9, 
   3.694829060894874*^9}, {3.7027736194540663`*^9, 3.702773709296208*^9}, {
   3.7027737395749617`*^9, 3.7027738230520163`*^9}, {3.702774281985703*^9, 
   3.7027743484504213`*^9}, {3.702854451840088*^9, 3.70285448555204*^9}, {
   3.70480239547946*^9, 3.704802398219247*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</6C_0.03-25.eps\>\""}], ",", "msdplt"}], 
   "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.687531381519882*^9, 3.687531389863661*^9}, {
   3.688131657754567*^9, 3.688131672897607*^9}, {3.690133696115766*^9, 
   3.690133727218423*^9}, {3.702774010295281*^9, 3.702774018180828*^9}, {
   3.702774361495964*^9, 3.7027743655832453`*^9}, {3.702854439053763*^9, 
   3.702854440597527*^9}, 3.702854481021689*^9}],

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{"logbarleg", "[", 
   RowBox[{"0.03", ",", "25", ",", "4", ",", "\"\<BlueGreenYellow\>\"", ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"LegendLabel", "\[Rule]", 
       RowBox[{"Placed", "[", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\!\(\*SubscriptBox[\(l\), \
\(m\)]\)\!\(\*SubscriptBox[\(Lr\), \(D\)]\)\>\"", ",", "Bottom"}], "]"}]}], 
      ",", 
      RowBox[{"LegendMarkerSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"40", ",", "250"}], "}"}]}], ",", 
      RowBox[{"LabelStyle", "\[Rule]", 
       RowBox[{"Directive", "[", 
        RowBox[{"Black", ",", 
         RowBox[{"FontSize", "\[Rule]", "30"}]}], "]"}]}], ",", 
      RowBox[{"TicksStyle", "\[Rule]", "Black"}]}], "}"}]}], "]"}]}]], "Input",\

 CellChangeTimes->{{3.702854569948374*^9, 3.702854597405547*^9}, {
  3.702854689118669*^9, 3.7028547072863417`*^9}, {3.70285473751163*^9, 
  3.702854737623871*^9}, {3.7028548052407084`*^9, 3.7028549981434717`*^9}, {
  3.702855075035797*^9, 3.7028550757790327`*^9}, {3.702855118947908*^9, 
  3.702855127283193*^9}, {3.702855169315529*^9, 3.702855171888589*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</6C_leg.pdf\>\""}], ",", "x"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.70285500185756*^9, 3.702855016081697*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["D", "Subsection",
 CellChangeTimes->{{3.694829112125366*^9, 3.694829119313354*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fig6Ddir", "=", 
   RowBox[{"vfdir", "<>", "\"\</motility/koff1_kend10/\>\""}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.7026588988282948`*^9, 3.702658917426676*^9}, 
   3.70265899505588*^9, {3.703268603679715*^9, 3.70326860519925*^9}, 
   3.703465255536841*^9}],

Cell[CellGroupData[{

Cell["simulation", "Subsubsection",
 CellChangeTimes->{{3.694829375930079*^9, 3.694829382765191*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"mdenss", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.01", ",", "0.02", ",", "0.05", ",", "0.1", ",", "0.2", ",", "0.5", ",",
      "1", ",", "2", ",", "5"}], 
    RowBox[{"(*", 
     RowBox[{",", "10"}], "*)"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ddirs", "=", 
    RowBox[{"Flatten", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"fig6Ddir", "<>", "\"\<sd\>\"", "<>", 
        RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\</dens_arr/dens\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"NumberForm", "[", 
          RowBox[{"md", ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", "2"}], "}"}]}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"md", ",", "mdenss"}], "}"}]}], "]"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{"nms", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"2", ",", "26", ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ldirs", "=", 
    RowBox[{"Flatten", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"fig6Ddir", "<>", "\"\<sd\>\"", "<>", 
        RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\</length_arr/nm\>\"", "<>", 
        RowBox[{"ToString", "[", "nm", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"nm", ",", "nms"}], "}"}]}], "]"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ks", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.001", ",", "0.002", ",", "0.005", ",", "0.01", ",", "0.02", ",", 
     "0.05", ",", "0.1", ",", "0.2", ",", "0.5", ",", "1", ",", "2"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"kdirs", "=", 
    RowBox[{"Flatten", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"fig6Ddir", "<>", "\"\<sd\>\"", "<>", 
        RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\</kon_arr/kon\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"NumberForm", "[", 
          RowBox[{"k", ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "ks"}], "}"}]}], "]"}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"ddirs", ",", "ldirs", ",", "kdirs"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"maxX", "=", "50"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6930103193253202`*^9, 3.693010384685337*^9}, {
   3.6930146981175203`*^9, 3.693014699058861*^9}, 3.69301477121972*^9, {
   3.6930589597508173`*^9, 3.6930590275928917`*^9}, {3.6930678061765633`*^9, 
   3.693067811382667*^9}, 3.6948293102434797`*^9, {3.694966742620056*^9, 
   3.6949667429163218`*^9}, {3.702658863947919*^9, 3.70265886536314*^9}, 
   3.702658928994629*^9, {3.702658974058387*^9, 3.702659046056733*^9}, {
   3.70266004584588*^9, 3.70266013701956*^9}, 3.702660173035726*^9, {
   3.702660311479583*^9, 3.702660320007071*^9}, {3.70266050052264*^9, 
   3.702660502554183*^9}, {3.702661384685152*^9, 3.702661385340837*^9}, {
   3.7028233341119413`*^9, 3.702823334297575*^9}, {3.702823431208811*^9, 
   3.702823431518918*^9}, {3.7028267914444027`*^9, 3.702826791706736*^9}, {
   3.702826898249291*^9, 3.7028268983895073`*^9}, 3.7028279468417377`*^9, 
   3.702828106619628*^9, 3.7028281890569487`*^9, {3.703465210496066*^9, 
   3.703465226751453*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"actext", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"pts3", "[", 
      RowBox[{"dir", ",", "\"\<actins_ext\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"cms", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Mean", ",", 
     RowBox[{"actext", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "All", ",", "All", ",", 
        RowBox[{"1", ";;", "2"}]}], "]"}], "]"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cms", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"MovingAverage", "[", 
      RowBox[{"#", ",", "20"}], "]"}], "&"}], "/@", "cms"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lkext", "=", 
   RowBox[{
    RowBox[{"cms", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"cms", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.694966780171879*^9, 3.694966781178989*^9}, 
   3.7028267821911297`*^9, 3.703465170592984*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lps", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"plThsq", "[", 
      RowBox[{
       RowBox[{"lkext", "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", "maxX"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "lkext", "]"}]}], "}"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{3.703465175934803*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xvals1", "=", 
   RowBox[{"mdenss", " ", "Lm", " ", "L0", " ", "rd0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]0", " ", "Lm", 
      RowBox[{"(", 
       RowBox[{"nms", "-", "1"}], ")"}]}], ")"}], " ", "rd0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"xvals3", "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Rho]0", " ", "Lm", " ", "L0"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"ks", "/", 
       RowBox[{"(", 
        RowBox[{"ks", "+", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"0.9", "koff0"}], "+", 
           RowBox[{"0.1", "kend0"}]}], ")"}]}], ")"}]}], ")"}]}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bnd1", "=", 
   RowBox[{"(", 
    RowBox[{"5", 
     RowBox[{"Length", "[", "mdenss", "]"}]}], ")"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bnd2", "=", 
   RowBox[{"bnd1", "+", 
    RowBox[{"5", 
     RowBox[{"Length", "[", "nms", "]"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lps1", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"Partition", "[", 
     RowBox[{
      RowBox[{"lps", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", "bnd1"}], "]"}], "]"}], ",", 
      RowBox[{"Length", "[", "mdenss", "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lps2", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"Partition", "[", 
     RowBox[{
      RowBox[{"lps", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"bnd1", "+", "1"}], ")"}], ";;", "bnd2"}], "]"}], "]"}], 
      ",", 
      RowBox[{"Length", "[", "nms", "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lps3", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"Partition", "[", 
     RowBox[{
      RowBox[{"lps", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"bnd2", "+", "1"}], ")"}], ";;"}], "]"}], "]"}], ",", 
      RowBox[{"Length", "[", "ks", "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.693010253029624*^9, 3.693010255055624*^9}, {
   3.693010409243886*^9, 3.693010410244808*^9}, {3.6930104589704447`*^9, 
   3.693010489849372*^9}, 3.6930147504547367`*^9, {3.693014831420834*^9, 
   3.693014837076696*^9}, {3.6930150030742702`*^9, 3.693015004887475*^9}, {
   3.693015296420745*^9, 3.693015296774139*^9}, 3.693019911318362*^9, 
   3.693021420044187*^9, 3.693068387050994*^9, {3.693095974484981*^9, 
   3.6930960045614433`*^9}, {3.6930981059282312`*^9, 
   3.6930981214135103`*^9}, {3.693098172948015*^9, 3.6930981952346077`*^9}, {
   3.6930999431325903`*^9, 3.6930999439715967`*^9}, 3.693914377217301*^9, {
   3.69482918787183*^9, 3.694829241023943*^9}, 3.6948292961205673`*^9, {
   3.694966683368175*^9, 3.694966771120057*^9}, 3.694967035278393*^9, {
   3.69496727365215*^9, 3.694967277272092*^9}, {3.702662189616343*^9, 
   3.70266219328789*^9}, {3.7026623453328323`*^9, 3.70266234607649*^9}, {
   3.7026627153163958`*^9, 3.702662721986693*^9}, 3.702819213809071*^9, 
   3.7028205200438023`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lperrs1", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", 
     RowBox[{"Partition", "[", 
      RowBox[{
       RowBox[{"lps", "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", "bnd1"}], "]"}], "]"}], ",", 
       RowBox[{"Length", "[", "mdenss", "]"}]}], "]"}], "]"}], "/", 
    RowBox[{"Sqrt", "[", "5", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs2", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", 
     RowBox[{"Partition", "[", 
      RowBox[{
       RowBox[{"lps", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"bnd1", "+", "1"}], ")"}], ";;", "bnd2"}], "]"}], "]"}], 
       ",", 
       RowBox[{"Length", "[", "nms", "]"}]}], "]"}], "]"}], "/", 
    RowBox[{"Sqrt", "[", "5", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs3", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", 
     RowBox[{"Partition", "[", 
      RowBox[{
       RowBox[{"lps", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"bnd2", "+", "1"}], ")"}], ";;"}], "]"}], "]"}], ",", 
       RowBox[{"Length", "[", "ks", "]"}]}], "]"}], "]"}], "/", 
    RowBox[{"Sqrt", "[", "5", "]"}]}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.70282052964819*^9, 3.702820548771785*^9}, {
  3.702820579598654*^9, 3.702820587974255*^9}, {3.702823074762339*^9, 
  3.702823086129855*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["theory: duke holy 1995", "Subsubsection",
 CellChangeTimes->{{3.6948292762794847`*^9, 3.6948292865699053`*^9}, {
  3.6948293686956863`*^9, 3.694829369711838*^9}, {3.694968593778799*^9, 
  3.694968595291177*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"f", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ll", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"x", " ", "f"}], "<", "y"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"gg", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"x", "/", "f"}], ">", "y"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.70346568631074*^9, 3.703465738233995*^9}, {
   3.7034662559516706`*^9, 3.703466258198675*^9}, {3.7034662935447483`*^9, 
   3.7034663069521437`*^9}, 3.70346633882377*^9, {3.703466561571357*^9, 
   3.703466590578101*^9}, {3.70346663149827*^9, 3.703466631642006*^9}, 
   3.7048087845178967`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Lp", "=", "17"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kbT", "=", "0.004"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"visc", "=", "0.001"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"maxBindDist", "=", "0.063"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ka", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v0", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   RowBox[{"kbT", "/", "visc"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ls", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"2", ",", "25"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Sigma]0", "=", "\[Rho]0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Sigma]x", "=", 
   RowBox[{
    RowBox[{"maxBindDist", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "5"}], "/", "3"}], ")"}]}], 
    RowBox[{"Lp", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "/", "3"}], ")"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Sigma]xx", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "/", "v0"}], " ", ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "5"}], "/", "6"}], ")"}]}], 
    RowBox[{"Lp", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "/", "3"}], ")"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"davg0", "=", 
    RowBox[{"\[Sigma]0", " ", 
     RowBox[{"Lp", "^", 
      RowBox[{"(", 
       RowBox[{"1", "/", "5"}], ")"}]}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"davg", ",", "savg", ",", "pcur", ",", "dth", ",", "persist"}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"davg", "[", 
    RowBox[{"\[Sigma]_", ",", 
     RowBox[{"\[Sigma]p_:", "\[Sigma]x"}], ",", " ", 
     RowBox[{"\[Sigma]pp_:", "\[Sigma]xx"}]}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"ll", "[", 
      RowBox[{"\[Sigma]", ",", "\[Sigma]pp"}], "]"}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"\[Sigma]", " ", 
        RowBox[{"Sqrt", "[", 
         RowBox[{"\[Gamma]", "/", "v0"}], "]"}]}], ")"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"gg", "[", 
         RowBox[{"\[Sigma]", ",", "\[Sigma]pp"}], "]"}], "&&", 
        RowBox[{"ll", "[", 
         RowBox[{"\[Sigma]", ",", "\[Sigma]p"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"\[Sigma]", "^", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "2."}], "/", "5"}], ")"}]}], 
        RowBox[{"Lp", "^", 
         RowBox[{"(", 
          RowBox[{"1", "/", "5"}], ")"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"gg", "[", 
          RowBox[{"\[Sigma]", ",", "\[Sigma]p"}], "]"}], ",", 
         RowBox[{"1", "/", 
          RowBox[{"(", 
           RowBox[{"\[Sigma]", " ", "maxBindDist"}], ")"}]}]}], "]"}], ",", 
       RowBox[{"-", "1"}]}], "]"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"savg", "[", 
    RowBox[{"l_", ",", "d_"}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"d", "\[GreaterEqual]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"l", "+", 
         RowBox[{"2", "d"}]}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{"l", "+", 
         RowBox[{"3", "d"}]}], ")"}]}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"d", "^", "2"}], "/", "l"}], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"l", "/", "d"}], "]"}], "-", "1", "-", 
        RowBox[{"l", "/", "d"}]}], ")"}]}], ",", 
     RowBox[{"-", "1"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"pcur", "[", 
    RowBox[{"\[Sigma]_", ",", 
     RowBox[{"\[Sigma]pp_:", "\[Sigma]xx"}]}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"\[Sigma]", ">", "\[Sigma]pp"}], ",", "Lp", ",", 
     RowBox[{"Lp", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"\[Sigma]", "/", "\[Sigma]pp"}], ")"}], "^", "2"}]}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"dth", "[", 
    RowBox[{"\[Rho]_", ",", "L_"}], "]"}], ":=", 
   RowBox[{"3", "/", 
    RowBox[{"(", 
     RowBox[{"\[Rho]", " ", 
      RowBox[{"L", "^", "2"}]}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"persist", "[", 
     RowBox[{"\[Rho]_", ",", "L_", ",", 
      RowBox[{"\[Sigma]p_:", "\[Sigma]x"}], ",", 
      RowBox[{"\[Sigma]pp_:", "\[Sigma]xx"}]}], "]"}], ":=", 
    RowBox[{"With", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"s", "=", 
        RowBox[{"savg", "[", 
         RowBox[{"L", ",", 
          RowBox[{"davg", "[", 
           RowBox[{"\[Rho]", ",", "\[Sigma]p", ",", "\[Sigma]pp"}], "]"}]}], 
         "]"}]}], "}"}], ",", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"s", ">", "0"}], ",", 
        RowBox[{"1", "/", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"1", "/", 
            RowBox[{"pcur", "[", 
             RowBox[{"\[Rho]", ",", "\[Sigma]pp"}], "]"}]}], "+", 
           RowBox[{
            RowBox[{
             RowBox[{"dth", "[", 
              RowBox[{"\[Rho]", ",", "L"}], "]"}], "^", "2"}], "/", "s"}]}], 
          ")"}]}], ",", 
        RowBox[{"-", "1"}]}], "]"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pr", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.5"}], ",", "1.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1.5"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prplt1", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Rho]0", " ", "Lm", " ", "L", " ", "rd0"}], ",", 
          RowBox[{"persist", "[", 
           RowBox[{"\[Rho]0", ",", "L"}], "]"}]}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"L", ",", "0.01", ",", "26", ",", "0.01"}], "}"}]}], "]"}], 
     ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", "Thick", ",", "Dashed"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prplt2", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Rho]0", " ", "Lm", " ", "L0", " ", "rD"}], ",", 
          RowBox[{"persist", "[", 
           RowBox[{
            RowBox[{"\[Rho]0", "*", "rD"}], ",", "L0", ",", "\[Sigma]x", ",", 
            "\[Sigma]xx"}], " ", "]"}]}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"rD", ",", 
         RowBox[{".001", "/", "1.01"}], ",", 
         RowBox[{"3", "/", "4"}], ",", "0.01"}], "}"}]}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", "Thick", ",", "Dotted"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prplt3", "=", 
   RowBox[{"ListPlot", "[", "  ", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Rho]", " ", "Lm", " ", "L0", " ", "rd0"}], ",", 
          RowBox[{"persist", "[", 
           RowBox[{"\[Rho]", ",", "L0"}], "]"}]}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Rho]", ",", "0.001", ",", "6", ",", "0.01"}], "}"}]}], 
      "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Green", ",", "Thick"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6908289518540154`*^9, 3.690829183326593*^9}, {
   3.690829215686035*^9, 3.690829256532992*^9}, {3.6908292891965857`*^9, 
   3.690829339562786*^9}, 3.690829482351533*^9, {3.690829653475766*^9, 
   3.690829832973675*^9}, {3.690831152381*^9, 3.690831165446589*^9}, {
   3.6908312622101183`*^9, 3.690831288865958*^9}, {3.693914563085428*^9, 
   3.693914566692687*^9}, {3.6939147316493063`*^9, 3.693914731806114*^9}, {
   3.693914790903183*^9, 3.693914798238636*^9}, {3.693915167932311*^9, 
   3.693915328228868*^9}, {3.693915508799697*^9, 3.6939155385120564`*^9}, {
   3.693915596954473*^9, 3.693915622854863*^9}, {3.693915657071535*^9, 
   3.6939156932023487`*^9}, {3.6939158269263363`*^9, 3.693915830471766*^9}, {
   3.693915887257145*^9, 3.693915887776473*^9}, 3.693923096387932*^9, {
   3.693923138968843*^9, 3.693923143241933*^9}, {3.693923176283086*^9, 
   3.693923267229465*^9}, {3.693925791938081*^9, 3.69392581235466*^9}, {
   3.693926327318202*^9, 3.693926371493339*^9}, {3.693926412268839*^9, 
   3.6939264530975657`*^9}, {3.6939264838427267`*^9, 3.693926523081642*^9}, 
   3.694829339946741*^9, {3.694968659795073*^9, 3.6949686719262543`*^9}, 
   3.702662762772193*^9, {3.7028195459793262`*^9, 3.702819585728086*^9}, {
   3.7028196693284597`*^9, 3.702819695288974*^9}, {3.702819746269766*^9, 
   3.702819792299273*^9}, {3.702819845490922*^9, 3.702819913417247*^9}, {
   3.702819990414629*^9, 3.7028200182212963`*^9}, {3.7028287249726152`*^9, 
   3.702828742684601*^9}, {3.702828799237197*^9, 3.70282880035021*^9}, {
   3.70282886152954*^9, 3.702828862462503*^9}, {3.703465579322896*^9, 
   3.703465599002852*^9}, {3.703465793140376*^9, 3.703465795251301*^9}, 
   3.7034661888398943`*^9, 3.703466286755658*^9, {3.704808798182254*^9, 
   3.704808815850144*^9}, {3.7069022794163847`*^9, 3.706902288374552*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["plot theory + simulation", "Subsubsection",
 CellChangeTimes->{{3.694829356241735*^9, 3.694829365273306*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"xtix", "=", 
     RowBox[{"ytix", "=", 
      RowBox[{"LogTicks", "[", 
       RowBox[{"10", ",", 
        RowBox[{"-", "3"}], ",", "3"}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"dplt", "=", 
     RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"xvals1", ",", "lps1"}], "}"}], "\[Transpose]"}], ",", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"xvals2", ",", "lps2"}], "}"}], "\[Transpose]"}], ",", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"xvals3", ",", "lps3"}], "}"}], "\[Transpose]"}], ")"}],
            "[", 
           RowBox[{"[", 
            RowBox[{"1", ";;", 
             RowBox[{"-", "3"}]}], "]"}], "]"}]}], "\[IndentingNewLine]", 
         "}"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"ConstantArray", "[", 
           RowBox[{"True", ",", "3"}], "]"}], ",", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"True", ",", "6"}], "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"ConstantArray", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"\[FilledCircle]", ",", "15"}], "}"}], ",", "3"}], 
             "]"}], ",", 
            RowBox[{"ConstantArray", "[", 
             RowBox[{"\"\<\[CenterDot]\>\"", ",", "6"}], "]"}]}], "]"}]}], 
         ","}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "->", 
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "@", 
             RowBox[{"{", 
              RowBox[{"Green", ",", 
               RowBox[{"Rectangle", "[", "]"}]}], "}"}]}], ",", "0.06"}], 
           "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "@", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Rotate", "[", 
                RowBox[{
                 RowBox[{"Rectangle", "[", "]"}], ",", 
                 RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}]}], ",", "0.07"}], 
           "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "@", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Disk", "[", "]"}]}], "}"}]}], ",", "0.06"}], "}"}]}], 
         "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Green", ",", "Red", ",", "Blue"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \
\(m\)]\)\!\(\*SubscriptBox[\(l\), \(m\)]\)\!\(\*SubscriptBox[\(Lr\), \(D\)]\)\
\>\"", ",", "\"\<Path Persistence\\nLength (\[Mu]m)\>\""}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"4", "\[Rule]", 
           RowBox[{"{", "5", "}"}]}], ",", 
          RowBox[{"6", "\[Rule]", 
           RowBox[{"{", "7", "}"}]}], ",", 
          RowBox[{"8", "\[Rule]", 
           RowBox[{"{", "9", "}"}]}]}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"ytix", ",", "Automatic"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"xtix", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.25", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Axes", "\[Rule]", "None"}]}], "]"}]}], ";"}], 
   "\n"}]}]], "Input",
 CellChangeTimes->{{3.693323059013507*^9, 3.693323068945348*^9}, {
   3.6933246748811417`*^9, 3.693324727092619*^9}, 3.693325418615409*^9, {
   3.693914863686348*^9, 3.6939148701403913`*^9}, {3.6939155718285427`*^9, 
   3.693915572300576*^9}, 3.693918147527931*^9, {3.693918203464855*^9, 
   3.6939182473570423`*^9}, {3.693918286191609*^9, 3.6939183072245693`*^9}, 
   3.693923107741583*^9, {3.693923349587538*^9, 3.693923370031493*^9}, 
   3.693923530199808*^9, {3.693923642681055*^9, 3.69392365947845*^9}, {
   3.693923751381466*^9, 3.6939238464905233`*^9}, {3.6939256126825447`*^9, 
   3.6939256130203238`*^9}, {3.693925833636763*^9, 3.693925976057938*^9}, {
   3.6939263913982267`*^9, 3.693926402019898*^9}, {3.694175416742619*^9, 
   3.694175438190942*^9}, {3.694175479929654*^9, 3.694175531170727*^9}, {
   3.694175594946813*^9, 3.694175595513085*^9}, {3.694175740575204*^9, 
   3.694175776164175*^9}, {3.6941758645799913`*^9, 3.694175903666292*^9}, 
   3.694176192108038*^9, {3.694829350410486*^9, 3.694829351395665*^9}, {
   3.6949672103937283`*^9, 3.694967230424769*^9}, {3.694968656838273*^9, 
   3.6949686658844347`*^9}, {3.7028288203325787`*^9, 
   3.7028288309131823`*^9}, {3.7069021578181953`*^9, 
   3.7069021711037617`*^9}, {3.706902208876255*^9, 3.706902214968272*^9}}],

Cell[BoxData[
 RowBox[{"scplt", "=", 
  RowBox[{"Show", "[", 
   RowBox[{"{", 
    RowBox[{"dplt", ",", "prplt1", ",", "prplt2", ",", "prplt3"}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6933237544373417`*^9, 3.693323762063835*^9}, {
  3.693324057248186*^9, 3.693324060480835*^9}, {3.6933242128517303`*^9, 
  3.693324214025663*^9}, {3.6933246167277737`*^9, 3.693324621526331*^9}, {
  3.693325074571287*^9, 3.693325076241613*^9}, {3.693915369438003*^9, 
  3.693915370399345*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</6D.eps\>\""}], ",", "scplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.6949672518398046`*^9, {3.702829555516952*^9, 3.702829555709353*^9}, {
   3.703466578992782*^9, 3.7034666050985823`*^9}, {3.703466650370698*^9, 
   3.703466650465455*^9}, {3.7069023154124413`*^9, 3.706902322533128*^9}}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Figure 7, S7", "Section",
 CellChangeTimes->{{3.68149932621382*^9, 3.681499337002256*^9}, {
  3.6816540170467463`*^9, 3.681654018471031*^9}, {3.694831552769101*^9, 
  3.694831552985012*^9}, {3.704799298884391*^9, 3.704799299961261*^9}}],

Cell[CellGroupData[{

Cell["7", "Subsection",
 CellChangeTimes->{{3.704647383258864*^9, 3.7046473875356207`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"seeds", "=", 
   RowBox[{"Range", "[", "20", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"fig7dir", "<>", "\"\</seed\>\"", "<>", 
      RowBox[{"ToString", "[", "#", "]"}], "<>", "\"\<e7\>\""}], "&"}], "/@", 
    "seeds"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"dirs", "[", 
    RowBox[{"[", "15", "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.694968779019415*^9, 3.6949687912123423`*^9}, {
  3.694968849860215*^9, 3.694968851607218*^9}, {3.69496932020059*^9, 
  3.694969320424342*^9}, {3.694970120197131*^9, 3.6949701205664062`*^9}, {
  3.703514747644479*^9, 3.7035147593968678`*^9}, {3.703514800747855*^9, 
  3.703514810164027*^9}, {3.70464700887144*^9, 3.70464700978437*^9}}],

Cell[CellGroupData[{

Cell["A", "Subsubsection",
 CellChangeTimes->{{3.681499339671895*^9, 3.681499348699482*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"acts", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<actins\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"links", "=", 
   RowBox[{
    RowBox[{"pts", "[", 
     RowBox[{"dir", ",", "\"\<links\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ams", "=", 
   RowBox[{
    RowBox[{"pts", "[", 
     RowBox[{"dir", ",", "\"\<amotors\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pms", "=", 
   RowBox[{
    RowBox[{"pts", "[", 
     RowBox[{"dir", ",", "\"\<pmotors\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.703515278862969*^9, 3.703515279322956*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"idxs", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "51", ",", "151", ",", "399"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"frs", "=", 
   RowBox[{"draw", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"links", "[", 
      RowBox[{"[", "idxs", "]"}], "]"}], ",", 
     RowBox[{"ams", "[", 
      RowBox[{"[", 
       RowBox[{"idxs", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "2"}]}], "]"}], "]"}], ",", 
     RowBox[{"pms", "[", 
      RowBox[{"[", 
       RowBox[{"idxs", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "10"}]}], "]"}], "]"}], ",", "dir", ",", 
     "False", ",", 
     RowBox[{"{", 
      RowBox[{"50", ",", "50"}], "}"}], ",", "1", ",", "False", ",", "16"}], 
    "]"}]}], ";"}], "\n", 
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"frs", ",", 
   RowBox[{"ImageSize", "\[Rule]", "1000"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.677921409609124*^9, 3.677921498825314*^9}, {
   3.680459853219302*^9, 3.6804598912569437`*^9}, 3.680459959569092*^9, {
   3.6814994510445232`*^9, 3.681499455260887*^9}, {3.681499536773926*^9, 
   3.68149955144232*^9}, {3.687702410425185*^9, 3.6877024424636106`*^9}, 
   3.690161065397871*^9, {3.690161135077928*^9, 3.690161135438611*^9}, {
   3.69016119539719*^9, 3.690161195906497*^9}, {3.690161260948567*^9, 
   3.690161265201728*^9}, {3.694830417962392*^9, 3.694830418104662*^9}, {
   3.694969003406057*^9, 3.694969011343129*^9}, {3.694969318858549*^9, 
   3.694969323904351*^9}, {3.703514832859023*^9, 3.7035148349462833`*^9}, {
   3.703514869433666*^9, 3.703514888505662*^9}, {3.703515049391817*^9, 
   3.7035150512770643`*^9}, {3.703515143563466*^9, 3.703515147563147*^9}, {
   3.704117114712756*^9, 3.704117116111685*^9}, {3.704117152990814*^9, 
   3.7041172410844727`*^9}, {3.704646548716209*^9, 3.7046465515458317`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7A_t0_50_150_398.eps\>\""}], ",", 
    RowBox[{"GraphicsRow", "[", 
     RowBox[{"frs", ",", 
      RowBox[{"ImageSize", "\[Rule]", "1000"}]}], "]"}]}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.704117275388027*^9, 3.704117292739057*^9}, 
   3.7046477153331614`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsubsection",
 CellChangeTimes->{{3.681499350145357*^9, 3.6814993548903627`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nm", "=", "11"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nparts", "=", 
   RowBox[{"500", "*", "nm"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rngs", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"1", ",", 
          RowBox[{"nm", " ", "i"}]}], "]"}], ",", 
        RowBox[{"Range", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"nm", 
            RowBox[{"(", 
             RowBox[{"i", "+", "1"}], ")"}]}], "+", "1"}], ",", "nparts"}], 
         "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         RowBox[{"nparts", "/", "nm"}], "-", "1"}]}], "}"}]}], "]"}]}], ";"}],
   "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
   "All", " ", "pairs", " ", "of", " ", "actin", " ", "beads", " ", "that", 
    " ", "are", " ", "not", " ", "on", " ", "the", " ", "same", " ", 
    "filament"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"allPairs", "=", 
    RowBox[{"Union", "[", 
     RowBox[{"Sort", "/@", 
      RowBox[{"Flatten", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"i", ",", "nparts"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"RandomSample", "[", 
             RowBox[{
              RowBox[{"rngs", "[", 
               RowBox[{"[", 
                RowBox[{
                 RowBox[{"Quotient", "[", 
                  RowBox[{
                   RowBox[{"i", "-", "1"}], ",", "nm"}], "]"}], "+", "1"}], 
                "]"}], "]"}], ",", "1000"}], "]"}]}], "}"}]}], "]"}], ",", 
        "1"}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"A", " ", "sample", " ", "of", " ", "them"}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"idxs", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "51", ",", "101", ",", "201", ",", "399"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sampPairs", "=", 
   RowBox[{"RandomSample", "[", 
    RowBox[{"allPairs", ",", 
     RowBox[{"500", "*", 
      RowBox[{"499", "/", "2"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"gr", "=", 
   RowBox[{"rdftraj", "[", 
    RowBox[{
     RowBox[{"acts", "[", 
      RowBox[{"[", "idxs", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"50", ",", "50"}], "}"}], ",", "0.1", ",", "5", ",", 
     "sampPairs"}], "]"}]}], ";"}], "\n"}], "Input",
 CellChangeTimes->{{3.671237295782929*^9, 3.671237310302497*^9}, {
   3.671237380037127*^9, 3.67123738127028*^9}, {3.677925014991693*^9, 
   3.677925099847842*^9}, {3.681641867329805*^9, 3.681641903945031*^9}, 
   3.681641995868966*^9, {3.690200205187645*^9, 3.690200222521652*^9}, 
   3.694969226955017*^9, {3.7035151835624037`*^9, 3.703515195517315*^9}, {
   3.703515250836562*^9, 3.7035153246141977`*^9}, 3.703515354998192*^9, {
   3.704117357137608*^9, 3.704117358561795*^9}, {3.704117390746068*^9, 
   3.704117391154512*^9}, {3.704117504249084*^9, 3.704117514213331*^9}, {
   3.704117550301654*^9, 3.7041175599964952`*^9}, {3.704117596652309*^9, 
   3.704117612811248*^9}, 3.704117647314372*^9, {3.704117702833126*^9, 
   3.704117712306651*^9}, {3.704647665430862*^9, 3.704647666926497*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cf", "=", 
   RowBox[{"ColorData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<BlueGreenYellow\>\"", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Min", "[", "idxs", "]"}], ",", 
        RowBox[{"Max", "[", "idxs", "]"}]}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"ts", "=", 
   RowBox[{"idxs", "-", "1"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.704117419968607*^9, 3.704117431615415*^9}}],

Cell[BoxData[
 RowBox[{"grplot", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"gr", ",", 
    RowBox[{"(*", 
     RowBox[{
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"cf", "[", "t", "]"}], ",", "Thick"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "ts"}], "}"}]}], "]"}]}], ","}], "*)"}], 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<distance (\[Mu]m)\>\"", ",", "\"\<radial distr. func.\>\""}], 
      "}"}]}], ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
    RowBox[{"PlotLegends", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{
       RowBox[{"SwatchLegend", "[", 
        RowBox[{"ts", ",", 
         RowBox[{"LegendLabel", "\[Rule]", "\"\<time (s)\>\""}], ",", 
         RowBox[{"LegendMarkerSize", "\[Rule]", "20"}], ",", 
         RowBox[{"Spacings", "\[Rule]", "0"}], ",", 
         RowBox[{"LegendLayout", "\[Rule]", "\"\<ReversedColumn\>\""}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.8", ",", "0.65"}], "}"}]}], "]"}]}], ",", 
    RowBox[{"DataRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0.025", ",", "4.975"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6779269105534563`*^9, 3.677927058470285*^9}, 
   3.6779270891144533`*^9, {3.677927123281006*^9, 3.6779273932182007`*^9}, {
   3.677927462276518*^9, 3.677927526538611*^9}, {3.67792757592375*^9, 
   3.677927654084668*^9}, {3.6779278525073338`*^9, 3.67792787752826*^9}, {
   3.677928001036643*^9, 3.677928006373837*^9}, {3.677928040471978*^9, 
   3.677928089799273*^9}, 3.679330632662074*^9, {3.679330682499823*^9, 
   3.6793307338168983`*^9}, {3.6901613986548986`*^9, 3.690161410799881*^9}, 
   3.703515301690534*^9, {3.704117468928134*^9, 3.7041174808952312`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7B.eps\>\""}], ",", "grplot"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6901614232275763`*^9, 3.690161437205419*^9}, {
  3.690200344048828*^9, 3.69020034687848*^9}, {3.704117677281345*^9, 
  3.70411767947298*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["C", "Subsubsection",
 CellChangeTimes->{{3.6814993558231297`*^9, 3.681499360456851*^9}}],

Cell["\<\
 C and D, and S4 require you to have run the Python divergence calculating \
code \"div_by_vbin.py\" available in the versatile_framework _paper \
directory; see versatile_framework_paper/README.md for details\
\>", "Text",
 CellChangeTimes->{{3.681642368595343*^9, 3.681642388828834*^9}, {
  3.6816424356473103`*^9, 3.681642601942937*^9}, {3.681642757032165*^9, 
  3.681642853913191*^9}, {3.6816429563302593`*^9, 3.681642957242285*^9}, {
  3.6816430028269*^9, 3.68164306060299*^9}, {3.681643404438854*^9, 
  3.681643412197678*^9}, {3.704133887966421*^9, 3.704133899205974*^9}, {
  3.704647773476915*^9, 3.7046478079300337`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"t", "=", "245"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t", "=", "51"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sd", "=", "15"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"wft", ",", "uft", ",", "vft", ",", "divft"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"wft", "=", 
   RowBox[{"Import", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"dirs", "[", 
       RowBox[{"[", "sd", "]"}], "]"}], "<>", 
      "\"\</analysis/vfield_weights_nbins10_dt10_thresh10/t\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"t", "-", "5"}], "]"}], "<>", "\"\<.txt\>\""}], ",", 
     "\"\<TABLE\>\""}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"uft", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"velx", "[", 
    RowBox[{"i", ",", "j", ",", "wft"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vft", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"vely", "[", 
    RowBox[{"i", ",", "j", ",", "wft"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"divft", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"div", "[", 
    RowBox[{"i", ",", "j", ",", "wft"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dplt", "=", 
   RowBox[{"DensityPlot", "[", 
    RowBox[{
     RowBox[{"divft", "[", 
      RowBox[{"x", ",", "y"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"25", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], ",", 
        RowBox[{"25", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], ",", "All"}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"ColorFunction", "\[Rule]", 
      RowBox[{"ColorData", "[", 
       RowBox[{"{", 
        RowBox[{"\"\<ThermometerColors\>\"", ",", 
         RowBox[{"0.07", 
          RowBox[{"{", 
           RowBox[{"1", ",", 
            RowBox[{"-", "1"}]}], "}"}]}]}], "}"}], "]"}]}], ",", 
     RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}]}], 
    RowBox[{"(*", 
     RowBox[{",", 
      RowBox[{"PlotLegends", "\[Rule]", "Automatic"}]}], "*)"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vplt", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"uft", "[", 
        RowBox[{"x", ",", "y"}], "]"}], ",", 
       RowBox[{"vft", "[", 
        RowBox[{"x", ",", "y"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"25", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}]}]}]}], 
    RowBox[{"(*", 
     RowBox[{",", 
      RowBox[{"Background", "\[Rule]", "None"}]}], "*)"}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6816431558845177`*^9, 3.681643157884012*^9}, {
   3.681643210717698*^9, 3.6816432229736767`*^9}, {3.681643279670093*^9, 
   3.681643328318431*^9}, {3.681643425159951*^9, 3.681643583336296*^9}, 
   3.681643662695825*^9, {3.681643717370569*^9, 3.681643758088554*^9}, {
   3.690161967532316*^9, 3.690161996914051*^9}, {3.690208965442931*^9, 
   3.6902089987542753`*^9}, {3.69020902963363*^9, 3.690209044038104*^9}, {
   3.690209088128563*^9, 3.690209094725081*^9}, {3.690209927835723*^9, 
   3.69020994039678*^9}, {3.6902100222016087`*^9, 3.690210049088463*^9}, {
   3.690651107291234*^9, 3.690651130131122*^9}, {3.6906511977003193`*^9, 
   3.690651223612596*^9}, {3.690651474819569*^9, 3.6906515276446943`*^9}, {
   3.694969405443687*^9, 3.694969659962638*^9}, 3.704117777062724*^9, {
   3.704117815287034*^9, 3.704117842822226*^9}, {3.70411787777472*^9, 
   3.704117880637259*^9}, 3.7041180646341667`*^9, 3.704118106059722*^9, {
   3.704118138143923*^9, 3.704118150866871*^9}, {3.704118190382985*^9, 
   3.70411835608293*^9}, {3.7041184050499907`*^9, 3.7041185013651657`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7C_lowk_t50.png\>\""}], ",", 
    RowBox[{"Overlay", "[", 
     RowBox[{"{", 
      RowBox[{"dplt", ",", "vplt"}], "}"}], "]"}]}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.690162021712748*^9, 3.690162051614121*^9}, {
   3.69020894189098*^9, 3.690208942927668*^9}, {3.6902090642721643`*^9, 
   3.6902090645827703`*^9}, {3.690209996473093*^9, 3.690210009023541*^9}, 
   3.690210066606473*^9, {3.694969838178185*^9, 3.694969856646377*^9}, {
   3.7041185097649508`*^9, 3.704118529724061*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.70352982593134*^9, 3.703529827826755*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["D", "Subsubsection",
 CellChangeTimes->{{3.6905722081757593`*^9, 3.69057223784096*^9}, {
   3.690595743810721*^9, 3.690595746826125*^9}, 3.694281807018759*^9, {
   3.694830723372031*^9, 3.694830749049554*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "calculate", " ", "strain", " ", "for", " ", "the", " ", "first", " ", 
    "time"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"acts", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"pts", "[", 
        RowBox[{"dr", ",", "\"\<actins\>\""}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";", "\n", 
    RowBox[{"strs", "=", 
     RowBox[{"Map", "[", 
      RowBox[{"filstrain", ",", "acts"}], "]"}]}], ";", "\n", 
    RowBox[{"Export", "[", 
     RowBox[{
      RowBox[{"fig7dir", "<>", "\"\</strains_all.dat\>\""}], ",", "strs"}], 
     "]"}], ";"}], "*)"}]}]], "Input",
 CellChangeTimes->{
  3.7035315571802883`*^9, {3.704733473563192*^9, 3.704733489063121*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"densdivs", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Import", "[", 
        RowBox[{
         RowBox[{"dir", "<>", "\"\</analysis/dens_div_dr1_nbins10.dat\>\""}], 
         ",", "\"\<TSV\>\""}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"strs", "=", 
    RowBox[{"importCheck", "[", 
     RowBox[{"fig7dir", "<>", "\"\</strains_all.dat\>\""}], "]"}]}], 
   ";"}]}]], "Input",
 CellChangeTimes->{{3.703529871802741*^9, 3.703529890853487*^9}, {
   3.70353146026274*^9, 3.7035314761648006`*^9}, {3.703848612798297*^9, 
   3.703848612930444*^9}, {3.7047334983278427`*^9, 3.704733524990765*^9}, 
   3.704733560603746*^9, {3.704733609686439*^9, 3.704733611565037*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dt", "=", "10"}], ";"}]], "Input",
 CellChangeTimes->{{3.704733622781706*^9, 3.7047336235641193`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"bcs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ToExpression", "[", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</analysis/bin_counts_dr1.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"bcsdt", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"MovingAverage", "[", 
      RowBox[{"#", ",", "dt"}], "]"}], "&"}], "/@", "bcs"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bcsdt", "=", 
   RowBox[{"0.5", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"bcsdt", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{"2", ";;"}]}], "]"}], "]"}], "+", 
      RowBox[{"bcsdt", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{";;", 
          RowBox[{"-", "2"}]}]}], "]"}], "]"}]}], ")"}]}]}], ";"}]}], "Input",\

 CellChangeTimes->{{3.694284576055678*^9, 3.694284689644253*^9}, 
   3.694285225330699*^9, 3.694286248362844*^9, {3.6942862928649683`*^9, 
   3.694286293057583*^9}, 3.694975071836114*^9, {3.694975104735609*^9, 
   3.694975112662395*^9}, {3.703858940508119*^9, 3.703858950201784*^9}, 
   3.703859022288207*^9, 3.7038649475260067`*^9, {3.70413021176527*^9, 
   3.704130239163005*^9}, 3.704724171055202*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"divs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ToExpression", "@", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dir", "<>", "\"\</analysis/div_dr1_nbins10_dt\>\"", "<>", 
         RowBox[{"ToString", "[", "dt", "]"}], "<>", "\"\<.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"densdivsdr1", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Total", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"bcsdt", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "t"}], "]"}], "]"}], 
         RowBox[{"divs", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "t"}], "]"}], "]"}]}], ",", "2"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "bcsdt", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"bcsdt", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], ";"}], 
  "\n", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"densdivs", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Import", "[", 
        RowBox[{
         RowBox[{"dir", "<>", "\"\</analysis/dens_div_dr1_nbins10.dat\>\""}], 
         ",", "\"\<TSV\>\""}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"meandivdr1", "=", 
   RowBox[{"Mean", "[", "densdivsdr1", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"errdivdr1", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", "densdivsdr1", "]"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"Length", "[", "seeds", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.703860047638208*^9, 3.703860048053684*^9}, {
   3.703863663514934*^9, 3.703863665018585*^9}, 3.703863741313793*^9, {
   3.703863846079256*^9, 3.7038638467962103`*^9}, {3.7038639662409563`*^9, 
   3.703863968546935*^9}, 3.704733675854061*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tf", "=", "399"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"strmus", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Mean", ",", 
     RowBox[{"strs", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{";;", "tf"}]}], "]"}], "]"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"meanstr", "=", 
   RowBox[{"Mean", "[", "strmus", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"errstr", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", "strmus", "]"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"Length", "[", "seeds", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.7038486436373053`*^9, 3.7038487324589357`*^9}, {
  3.70412223479191*^9, 3.704122248502982*^9}, {3.7047337593134212`*^9, 
  3.7047337594657097`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"lp1", "=", 
    RowBox[{"ListPlot", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"meandivdr1", ",", 
         RowBox[{"meandivdr1", "-", "errdivdr1"}], ",", 
         RowBox[{"meandivdr1", "+", "errdivdr1"}]}], "}"}], "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "1"}], ";;", "2"}]}], "]"}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotMarkers", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Graphics", "[", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Disk", "[", "]"}]}], "}"}], "]"}], ",", "0.035"}], 
          "}"}], ",", "\"\<\[CenterDot]\>\"", ",", "\"\<\[CenterDot]\>\""}], 
        "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"Blue", ",", 
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Filling", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2", "\[Rule]", 
         RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"FillingStyle", "\[Rule]", 
       RowBox[{"Opacity", "[", "0.4", "]"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "95"}], ",", 
         RowBox[{"-", "2"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"FrameStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", "Black"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Black", ",", "Black"}], "}"}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"FrameTicks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"LinTicks", "[", 
            RowBox[{"0", ",", "400", ",", "100", ",", "10"}], "]"}], ",", 
           "Automatic"}], "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<time (s)\>\"", ",", 
         "\"\<\[Integral]\!\(\*SubscriptBox[\(\[Rho]\), \(a\)]\)\
\[LeftAngleBracket]\[Del]\[CenterDot]v\[RightAngleBracket] dA  (Hz) (\
\[FilledCircle])\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"ImagePadding", "\[Rule]", "100"}], ",", "\[IndentingNewLine]", 
      
      RowBox[{"ImageSize", "\[Rule]", "500"}], ",", "\[IndentingNewLine]", 
      RowBox[{"DataRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"Length", "[", "meandivdr1", "]"}]}], "}"}]}]}], "]"}]}], 
   ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"lp2", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"meanstr", ",", 
        RowBox[{"meanstr", "-", "errstr"}], ",", 
        RowBox[{"meanstr", "+", "errstr"}]}], "}"}], "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "2"}]}], "]"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "[", 
           RowBox[{"{", 
            RowBox[{"Red", ",", 
             RowBox[{"Circle", "[", "]"}]}], "}"}], "]"}], ",", "0.04"}], 
         "}"}], ",", "\"\<\[CenterDot]\>\"", ",", "\"\<\[CenterDot]\>\""}], 
       "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Filling", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"2", "\[Rule]", 
        RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FillingStyle", "\[Rule]", 
      RowBox[{"Opacity", "[", "0.4", "]"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Red", ",", "Red"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Black"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\>\"", ",", "\"\<filament strain [\[EmptyCircle]]\>\"", ",", 
        "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"ImagePadding", "\[Rule]", "100"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "500"}], ",", "\[IndentingNewLine]", 
     RowBox[{"DataRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"Length", "[", "meandivdr1", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"divstrplt", "=", 
  RowBox[{"Overlay", "[", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Show", "[", 
      RowBox[{"lp1", ",", 
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"True", ",", "False"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"True", ",", "True"}], "}"}]}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Show", "[", 
      RowBox[{"lp2", ",", 
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "True"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"False", ",", "False"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", "All"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
          RowBox[{"Rotate", "[", 
           RowBox[{"\"\<filament strain (\[EmptyCircle])\>\"", ",", "Pi"}], 
           "]"}]}], "}"}]}]}], "]"}]}], "}"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.703532111687304*^9, 3.70353211258888*^9}, {
   3.703532146224433*^9, 3.7035321652783613`*^9}, 3.703532239430108*^9, {
   3.703532341218754*^9, 3.703532343865075*^9}, {3.703550985781344*^9, 
   3.70355099484799*^9}, {3.7038487946283607`*^9, 3.703848807371664*^9}, {
   3.704122292529708*^9, 3.704122313719695*^9}, {3.7047338172410107`*^9, 
   3.704733851727199*^9}, {3.706919732181013*^9, 3.7069197762560368`*^9}, {
   3.7069198320341377`*^9, 3.7069199044185333`*^9}, {3.706920052315834*^9, 
   3.706920078493093*^9}, {3.706920113364108*^9, 3.706920228791698*^9}, {
   3.706920288731328*^9, 3.706920333275509*^9}, {3.706923910224998*^9, 
   3.706923934491094*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7D_paren.png\>\""}], ",", "divstrplt"}], 
   "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.690596575792699*^9, 3.690596592489336*^9}, {
   3.690682199930491*^9, 3.690682226852523*^9}, 3.694970705523456*^9, {
   3.704122326268869*^9, 3.704122326948739*^9}, {3.7069203430318003`*^9, 
   3.706920343246295*^9}, {3.706923989155964*^9, 3.706923992333708*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["S7 ", "Subsection",
 CellChangeTimes->{{3.690635346509413*^9, 3.690635357768628*^9}, {
   3.690646851342855*^9, 3.690646853278019*^9}, {3.6948311703979177`*^9, 
   3.694831171135708*^9}, 3.6948312910310307`*^9, 3.69483158822775*^9, {
   3.704799319876893*^9, 3.704799320711656*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"seeds", "=", 
   RowBox[{"Range", "[", "20", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"figS3dir", "<>", "\"\</seed\>\"", "<>", 
      RowBox[{"ToString", "[", "#", "]"}], "<>", "\"\<e7\>\""}], "&"}], "/@", 
    "seeds"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"dirs", "[", 
    RowBox[{"[", "15", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dt", "=", "10"}], ";"}]}], "Input",
 CellChangeTimes->{{3.694968779019415*^9, 3.6949687912123423`*^9}, {
  3.694968849860215*^9, 3.694968851607218*^9}, {3.69496932020059*^9, 
  3.694969320424342*^9}, {3.694970120197131*^9, 3.6949701205664062`*^9}, {
  3.703514747644479*^9, 3.7035147593968678`*^9}, {3.703514800747855*^9, 
  3.703514810164027*^9}, {3.70464700887144*^9, 3.70464700978437*^9}, {
  3.704647530947036*^9, 3.704647531337785*^9}, {3.707585087012082*^9, 
  3.70758508934799*^9}}],

Cell[CellGroupData[{

Cell["A", "Subsubsection",
 CellChangeTimes->{{3.681499339671895*^9, 3.681499348699482*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"acts", "=", 
   RowBox[{"pts", "[", 
    RowBox[{"dir", ",", "\"\<actins\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"links", "=", 
   RowBox[{
    RowBox[{"pts", "[", 
     RowBox[{"dir", ",", "\"\<links\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ams", "=", 
   RowBox[{
    RowBox[{"pts", "[", 
     RowBox[{"dir", ",", "\"\<amotors\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pms", "=", 
   RowBox[{
    RowBox[{"pts", "[", 
     RowBox[{"dir", ",", "\"\<pmotors\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.703515278862969*^9, 3.703515279322956*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"idxs", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "101", ",", "251", ",", "399"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"frs", "=", 
   RowBox[{"draw", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"links", "[", 
      RowBox[{"[", "idxs", "]"}], "]"}], ",", 
     RowBox[{"ams", "[", 
      RowBox[{"[", 
       RowBox[{"idxs", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "2"}]}], "]"}], "]"}], ",", 
     RowBox[{"pms", "[", 
      RowBox[{"[", 
       RowBox[{"idxs", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "10"}]}], "]"}], "]"}], ",", "dir", ",", 
     "False", ",", 
     RowBox[{"{", 
      RowBox[{"50", ",", "50"}], "}"}], ",", "1", ",", "False", ",", "16"}], 
    "]"}]}], ";"}], "\n", 
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"frs", ",", 
   RowBox[{"ImageSize", "\[Rule]", "1000"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.677921409609124*^9, 3.677921498825314*^9}, {
   3.680459853219302*^9, 3.6804598912569437`*^9}, 3.680459959569092*^9, {
   3.6814994510445232`*^9, 3.681499455260887*^9}, {3.681499536773926*^9, 
   3.68149955144232*^9}, {3.687702410425185*^9, 3.6877024424636106`*^9}, 
   3.690161065397871*^9, {3.690161135077928*^9, 3.690161135438611*^9}, {
   3.69016119539719*^9, 3.690161195906497*^9}, {3.690161260948567*^9, 
   3.690161265201728*^9}, {3.694830417962392*^9, 3.694830418104662*^9}, {
   3.694969003406057*^9, 3.694969011343129*^9}, {3.694969318858549*^9, 
   3.694969323904351*^9}, {3.703514832859023*^9, 3.7035148349462833`*^9}, {
   3.703514869433666*^9, 3.703514888505662*^9}, {3.703515049391817*^9, 
   3.7035150512770643`*^9}, {3.703515143563466*^9, 3.703515147563147*^9}, {
   3.704117114712756*^9, 3.704117116111685*^9}, {3.704117152990814*^9, 
   3.7041172410844727`*^9}, {3.704646548716209*^9, 3.7046465515458317`*^9}, {
   3.704647615728198*^9, 3.704647644703156*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7A_t0_50_150_398_lowk.eps\>\""}], ",", 
    RowBox[{"GraphicsRow", "[", 
     RowBox[{"frs", ",", 
      RowBox[{"ImageSize", "\[Rule]", "1000"}]}], "]"}]}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.704117275388027*^9, 3.704117292739057*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsubsection",
 CellChangeTimes->{{3.681499350145357*^9, 3.6814993548903627`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nm", "=", "11"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nparts", "=", 
   RowBox[{"500", "*", "nm"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rngs", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"1", ",", 
          RowBox[{"nm", " ", "i"}]}], "]"}], ",", 
        RowBox[{"Range", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"nm", 
            RowBox[{"(", 
             RowBox[{"i", "+", "1"}], ")"}]}], "+", "1"}], ",", "nparts"}], 
         "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         RowBox[{"nparts", "/", "nm"}], "-", "1"}]}], "}"}]}], "]"}]}], ";"}],
   "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
   "All", " ", "pairs", " ", "of", " ", "actin", " ", "beads", " ", "that", 
    " ", "are", " ", "not", " ", "on", " ", "the", " ", "same", " ", 
    "filament"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"allPairs", "=", 
    RowBox[{"Union", "[", 
     RowBox[{"Sort", "/@", 
      RowBox[{"Flatten", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"i", ",", "j"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"i", ",", "nparts"}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"RandomSample", "[", 
             RowBox[{
              RowBox[{"rngs", "[", 
               RowBox[{"[", 
                RowBox[{
                 RowBox[{"Quotient", "[", 
                  RowBox[{
                   RowBox[{"i", "-", "1"}], ",", "nm"}], "]"}], "+", "1"}], 
                "]"}], "]"}], ",", "1000"}], "]"}]}], "}"}]}], "]"}], ",", 
        "1"}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"A", " ", "sample", " ", "of", " ", "them"}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"idxs", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "101", ",", "201", ",", "301", ",", "399"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sampPairs", "=", 
   RowBox[{"RandomSample", "[", 
    RowBox[{"allPairs", ",", 
     RowBox[{"500", "*", 
      RowBox[{"499", "/", "2"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"gr", "=", 
   RowBox[{"rdftraj", "[", 
    RowBox[{
     RowBox[{"acts", "[", 
      RowBox[{"[", "idxs", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"50", ",", "50"}], "}"}], ",", "0.1", ",", "5", ",", 
     "sampPairs"}], "]"}]}], ";"}], "\n"}], "Input",
 CellChangeTimes->{{3.671237295782929*^9, 3.671237310302497*^9}, {
   3.671237380037127*^9, 3.67123738127028*^9}, {3.677925014991693*^9, 
   3.677925099847842*^9}, {3.681641867329805*^9, 3.681641903945031*^9}, 
   3.681641995868966*^9, {3.690200205187645*^9, 3.690200222521652*^9}, 
   3.694969226955017*^9, {3.7035151835624037`*^9, 3.703515195517315*^9}, {
   3.703515250836562*^9, 3.7035153246141977`*^9}, 3.703515354998192*^9, {
   3.704117357137608*^9, 3.704117358561795*^9}, {3.704117390746068*^9, 
   3.704117391154512*^9}, {3.704117504249084*^9, 3.704117514213331*^9}, {
   3.704117550301654*^9, 3.7041175599964952`*^9}, {3.704117596652309*^9, 
   3.704117612811248*^9}, 3.704117647314372*^9, {3.704117702833126*^9, 
   3.704117712306651*^9}, {3.704647688214576*^9, 3.7046476889180403`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cf", "=", 
   RowBox[{"ColorData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<BlueGreenYellow\>\"", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Min", "[", "idxs", "]"}], ",", 
        RowBox[{"Max", "[", "idxs", "]"}]}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"ts", "=", 
   RowBox[{"idxs", "-", "1"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.704117419968607*^9, 3.704117431615415*^9}}],

Cell[BoxData[
 RowBox[{"grplot", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"gr", ",", 
    RowBox[{"(*", 
     RowBox[{
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"cf", "[", "t", "]"}], ",", "Thick"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "ts"}], "}"}]}], "]"}]}], ","}], "*)"}], 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<distance (\[Mu]m)\>\"", ",", "\"\<radial distr. func.\>\""}], 
      "}"}]}], ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", 
    RowBox[{"PlotLegends", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{
       RowBox[{"SwatchLegend", "[", 
        RowBox[{"ts", ",", 
         RowBox[{"LegendLabel", "\[Rule]", "\"\<time (s)\>\""}], ",", 
         RowBox[{"LegendMarkerSize", "\[Rule]", "20"}], ",", 
         RowBox[{"Spacings", "\[Rule]", "0"}], ",", 
         RowBox[{"LegendLayout", "\[Rule]", "\"\<ReversedColumn\>\""}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.8", ",", "0.65"}], "}"}]}], "]"}]}], ",", 
    RowBox[{"DataRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0.025", ",", "4.975"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6779269105534563`*^9, 3.677927058470285*^9}, 
   3.6779270891144533`*^9, {3.677927123281006*^9, 3.6779273932182007`*^9}, {
   3.677927462276518*^9, 3.677927526538611*^9}, {3.67792757592375*^9, 
   3.677927654084668*^9}, {3.6779278525073338`*^9, 3.67792787752826*^9}, {
   3.677928001036643*^9, 3.677928006373837*^9}, {3.677928040471978*^9, 
   3.677928089799273*^9}, 3.679330632662074*^9, {3.679330682499823*^9, 
   3.6793307338168983`*^9}, {3.6901613986548986`*^9, 3.690161410799881*^9}, 
   3.703515301690534*^9, {3.704117468928134*^9, 3.7041174808952312`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7B.eps\>\""}], ",", "grplot"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6901614232275763`*^9, 3.690161437205419*^9}, {
  3.690200344048828*^9, 3.69020034687848*^9}, {3.704117677281345*^9, 
  3.70411767947298*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7B_lowk.eps\>\""}], ",", "grplot"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.7041176860246267`*^9, 3.704117687049197*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["C", "Subsubsection",
 CellChangeTimes->{{3.6814993558231297`*^9, 3.681499360456851*^9}}],

Cell["\<\
 C and D, and S4 require you to have run the Python divergence calculating \
code \[OpenCurlyDoubleQuote]div_by_vbin.py\[CloseCurlyDoubleQuote] available \
in the versatile_framework _paper directory; see \
versatile_framework_paper/README.md for details\
\>", "Text",
 CellChangeTimes->{{3.681642368595343*^9, 3.681642388828834*^9}, {
   3.6816424356473103`*^9, 3.681642601942937*^9}, {3.681642757032165*^9, 
   3.681642853913191*^9}, {3.6816429563302593`*^9, 3.681642957242285*^9}, {
   3.6816430028269*^9, 3.68164306060299*^9}, {3.681643404438854*^9, 
   3.681643412197678*^9}, {3.704133887966421*^9, 3.704133899205974*^9}, 
   3.704719718633358*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"t", "=", "245"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t", "=", "51"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sd", "=", "15"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"wft", ",", "uft", ",", "vft", ",", "divft"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"wft", "=", 
   RowBox[{"Import", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"dirs", "[", 
       RowBox[{"[", "sd", "]"}], "]"}], "<>", 
      "\"\</analysis/vfield_weights_nbins10_dt10_thresh10/t\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"t", "-", "5"}], "]"}], "<>", "\"\<.txt\>\""}], ",", 
     "\"\<TABLE\>\""}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"uft", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"velx", "[", 
    RowBox[{"i", ",", "j", ",", "wft"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vft", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"vely", "[", 
    RowBox[{"i", ",", "j", ",", "wft"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"divft", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"div", "[", 
    RowBox[{"i", ",", "j", ",", "wft"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dplt", "=", 
   RowBox[{"DensityPlot", "[", 
    RowBox[{
     RowBox[{"divft", "[", 
      RowBox[{"x", ",", "y"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"25", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], ",", 
        RowBox[{"25", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], ",", "All"}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"ColorFunction", "\[Rule]", 
      RowBox[{"ColorData", "[", 
       RowBox[{"{", 
        RowBox[{"\"\<ThermometerColors\>\"", ",", 
         RowBox[{"0.07", 
          RowBox[{"{", 
           RowBox[{"1", ",", 
            RowBox[{"-", "1"}]}], "}"}]}]}], "}"}], "]"}]}], ",", 
     RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}]}], 
    RowBox[{"(*", 
     RowBox[{",", 
      RowBox[{"PlotLegends", "\[Rule]", "Automatic"}]}], "*)"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vplt", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"uft", "[", 
        RowBox[{"x", ",", "y"}], "]"}], ",", 
       RowBox[{"vft", "[", 
        RowBox[{"x", ",", "y"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", 
       RowBox[{"-", "25"}], ",", "25"}], "}"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"25", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}]}]}]}], 
    RowBox[{"(*", 
     RowBox[{",", 
      RowBox[{"Background", "\[Rule]", "None"}]}], "*)"}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6816431558845177`*^9, 3.681643157884012*^9}, {
   3.681643210717698*^9, 3.6816432229736767`*^9}, {3.681643279670093*^9, 
   3.681643328318431*^9}, {3.681643425159951*^9, 3.681643583336296*^9}, 
   3.681643662695825*^9, {3.681643717370569*^9, 3.681643758088554*^9}, {
   3.690161967532316*^9, 3.690161996914051*^9}, {3.690208965442931*^9, 
   3.6902089987542753`*^9}, {3.69020902963363*^9, 3.690209044038104*^9}, {
   3.690209088128563*^9, 3.690209094725081*^9}, {3.690209927835723*^9, 
   3.69020994039678*^9}, {3.6902100222016087`*^9, 3.690210049088463*^9}, {
   3.690651107291234*^9, 3.690651130131122*^9}, {3.6906511977003193`*^9, 
   3.690651223612596*^9}, {3.690651474819569*^9, 3.6906515276446943`*^9}, {
   3.694969405443687*^9, 3.694969659962638*^9}, 3.704117777062724*^9, {
   3.704117815287034*^9, 3.704117842822226*^9}, {3.70411787777472*^9, 
   3.704117880637259*^9}, 3.7041180646341667`*^9, 3.704118106059722*^9, {
   3.704118138143923*^9, 3.704118150866871*^9}, {3.704118190382985*^9, 
   3.70411835608293*^9}, {3.7041184050499907`*^9, 3.7041185013651657`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</7C_lowk_t50.png\>\""}], ",", 
    RowBox[{"Overlay", "[", 
     RowBox[{"{", 
      RowBox[{"dplt", ",", "vplt"}], "}"}], "]"}]}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.690162021712748*^9, 3.690162051614121*^9}, {
   3.69020894189098*^9, 3.690208942927668*^9}, {3.6902090642721643`*^9, 
   3.6902090645827703`*^9}, {3.690209996473093*^9, 3.690210009023541*^9}, 
   3.690210066606473*^9, {3.694969838178185*^9, 3.694969856646377*^9}, {
   3.7041185097649508`*^9, 3.704118529724061*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["D", "Subsubsection",
 CellChangeTimes->{{3.6905722081757593`*^9, 3.69057223784096*^9}, {
   3.690595743810721*^9, 3.690595746826125*^9}, 3.694281807018759*^9, {
   3.694830723372031*^9, 3.694830749049554*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "to", " ", "calculate", " ", "strains", " ", "for", " ", "the", " ", 
    "first", " ", "time"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"acts", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"pts", "[", 
        RowBox[{"dr", ",", "\"\<actins\>\""}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"strs", "=", 
     RowBox[{"Map", "[", 
      RowBox[{"filstrain", ",", "acts"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Export", "[", 
     RowBox[{
      RowBox[{"figS3dir", "<>", "\"\</strains_all.dat\>\""}], ",", "strs"}], 
     "]"}], ";"}], "*)"}]}]], "Input",
 CellChangeTimes->{
  3.7035319028439074`*^9, {3.704724225908511*^9, 3.704724250341837*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"strs", "=", 
   RowBox[{"importCheck", "[", 
    RowBox[{"figS3dir", "<>", "\"\</strains_all.dat\>\""}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.704719757117035*^9, 3.704719783630703*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"bcs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ToExpression", "[", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</analysis/bin_counts_dr1.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"bcsdt", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"MovingAverage", "[", 
      RowBox[{"#", ",", "dt"}], "]"}], "&"}], "/@", "bcs"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bcsdt", "=", 
   RowBox[{"0.5", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"bcsdt", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{"2", ";;"}]}], "]"}], "]"}], "+", 
      RowBox[{"bcsdt", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{";;", 
          RowBox[{"-", "2"}]}]}], "]"}], "]"}]}], ")"}]}]}], ";"}]}], "Input",\

 CellChangeTimes->{{3.694284576055678*^9, 3.694284689644253*^9}, 
   3.694285225330699*^9, 3.694286248362844*^9, {3.6942862928649683`*^9, 
   3.694286293057583*^9}, 3.694975071836114*^9, {3.694975104735609*^9, 
   3.694975112662395*^9}, {3.703858940508119*^9, 3.703858950201784*^9}, 
   3.703859022288207*^9, 3.7038649475260067`*^9, {3.70413021176527*^9, 
   3.704130239163005*^9}, 3.704724171055202*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"divs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ToExpression", "@", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dir", "<>", "\"\</analysis/div_dr1_nbins10_dt\>\"", "<>", 
         RowBox[{"ToString", "[", "dt", "]"}], "<>", "\"\<.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.703860047638208*^9, 3.703860048053684*^9}, {
   3.703863663514934*^9, 3.703863665018585*^9}, 3.703863741313793*^9, {
   3.703863846079256*^9, 3.7038638467962103`*^9}, {3.7038639662409563`*^9, 
   3.703863968546935*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"densdivsdr1", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"bcsdt", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "t"}], "]"}], "]"}], 
        RowBox[{"divs", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "t"}], "]"}], "]"}]}], ",", "2"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "bcsdt", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"bcsdt", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.7047242047915087`*^9, 3.704724206773136*^9}, {
  3.704724276475267*^9, 3.704724340569866*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"densdivs", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Import", "[", 
        RowBox[{
         RowBox[{"dir", "<>", "\"\</analysis/dens_div_dr1_nbins10.dat\>\""}], 
         ",", "\"\<TSV\>\""}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"meandivdr1", "=", 
     RowBox[{"Mean", "[", "densdivsdr1", "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"errdivdr1", "=", 
     RowBox[{
      RowBox[{"StandardDeviation", "[", "densdivsdr1", "]"}], "/", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"Length", "[", "seeds", "]"}], "]"}]}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.703529871802741*^9, 3.703529890853487*^9}, {
  3.70353146026274*^9, 3.7035314761648006`*^9}, {3.703848612798297*^9, 
  3.703848612930444*^9}, {3.704722890263111*^9, 3.704722890431314*^9}, {
  3.70472435418507*^9, 3.704724398880418*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tf", "=", "379"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"strmus", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Mean", ",", 
     RowBox[{"1", "-", 
      RowBox[{
       RowBox[{"strs", "[", 
        RowBox[{"[", 
         RowBox[{"All", ",", 
          RowBox[{";;", "tf"}]}], "]"}], "]"}], "/", "10"}]}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"meanstr", "=", 
   RowBox[{"Mean", "[", "strmus", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"errstr", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", "strmus", "]"}], "/", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"Length", "[", "seeds", "]"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.7038486436373053`*^9, 3.7038487324589357`*^9}, {
  3.70412223479191*^9, 3.704122248502982*^9}, {3.704719811022743*^9, 
  3.70471983224582*^9}, {3.7047244105850773`*^9, 3.7047244107131653`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"divrng", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "110"}], ",", "8"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.7047208316694317`*^9, 3.704720858921485*^9}, {
  3.704724421225161*^9, 3.704724423064045*^9}, {3.704724567077183*^9, 
  3.7047245756122627`*^9}, {3.704732670942856*^9, 3.7047326717729816`*^9}, {
  3.704732749732588*^9, 3.704732752290608*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"lp1", "=", 
    RowBox[{"ListPlot", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"meandivdr1", ",", 
         RowBox[{"meandivdr1", "-", "errdivdr1"}], ",", 
         RowBox[{"meandivdr1", "+", "errdivdr1"}]}], "}"}], "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "1"}], ";;", "2"}]}], "]"}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"Blue", ",", 
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Filling", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2", "\[Rule]", 
         RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"FillingStyle", "\[Rule]", 
       RowBox[{"Opacity", "[", "0.4", "]"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", "divrng"}], ",", "\[IndentingNewLine]", 
      
      RowBox[{"FrameStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", "Black"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Black", ",", "Black"}], "}"}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"FrameTicks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"LinTicks", "[", 
            RowBox[{"0", ",", "400", ",", "100", ",", "10"}], "]"}], ",", 
           "Automatic"}], "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
        "\"\<time (s)\>\"", ",", 
         "\"\<\[Integral]\!\(\*SubscriptBox[\(\[Rho]\), \(a\)]\)\
\[LeftAngleBracket]\[Del]\[CenterDot]v\[RightAngleBracket] dA  (Hz) (\
\[FilledCircle])\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"ImagePadding", "\[Rule]", "100"}], ",", "\[IndentingNewLine]", 
      
      RowBox[{"ImageSize", "\[Rule]", "500"}]}], "]"}]}], ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"lp2", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"meanstr", ",", 
        RowBox[{"meanstr", "-", "errstr"}], ",", 
        RowBox[{"meanstr", "+", "errstr"}]}], "}"}], "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "1"}], ";;", "2"}]}], "]"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "[", 
           RowBox[{"{", 
            RowBox[{"Red", ",", 
             RowBox[{"Circle", "[", "]"}]}], "}"}], "]"}], ",", "0.04"}], 
         "}"}], ",", "\"\<\[CenterDot]\>\"", ",", "\"\<\[CenterDot]\>\""}], 
       "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Filling", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"2", "\[Rule]", 
        RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FillingStyle", "\[Rule]", 
      RowBox[{"Opacity", "[", "0.4", "]"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Automatic", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"Red", ",", "Red"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Black"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\>\"", ",", "\"\<filament strain\>\"", ",", "\"\<\>\"", ",", 
        "\"\<\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"ImagePadding", "\[Rule]", "100"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ImageSize", "\[Rule]", "500"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"divstrplt", "=", 
  RowBox[{"Overlay", "[", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Show", "[", 
      RowBox[{"lp1", ",", 
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"True", ",", "False"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"True", ",", "True"}], "}"}]}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Show", "[", 
      RowBox[{"lp2", ",", 
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "True"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"False", ",", "False"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", "All"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
          RowBox[{"Rotate", "[", 
           RowBox[{"\"\<filament strain (\[EmptyCircle])\>\"", ",", "Pi"}], 
           "]"}]}], "}"}]}]}], "]"}]}], "}"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.703532111687304*^9, 3.70353211258888*^9}, {
   3.703532146224433*^9, 3.7035321652783613`*^9}, 3.703532239430108*^9, {
   3.703532341218754*^9, 3.703532343865075*^9}, {3.703550985781344*^9, 
   3.70355099484799*^9}, {3.7038487946283607`*^9, 3.703848807371664*^9}, {
   3.704122292529708*^9, 3.704122313719695*^9}, {3.704720868876112*^9, 
   3.704720869953595*^9}, {3.704724551443425*^9, 3.704724559189864*^9}, {
   3.7047330102794952`*^9, 3.704733033804769*^9}, {3.7075852536657133`*^9, 
   3.707585254402142*^9}, {3.707585295097312*^9, 3.707585317286537*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S7D.png\>\""}], ",", "divstrplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.690596575792699*^9, 3.690596592489336*^9}, {
   3.690682199930491*^9, 3.690682226852523*^9}, 3.694970705523456*^9, {
   3.704122326268869*^9, 3.704122326948739*^9}, {3.704724449215077*^9, 
   3.704724461830673*^9}, {3.704732772738679*^9, 3.7047327739958353`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["E (Weighted by density)", "Subsubsection",
 CellChangeTimes->{{3.6906519576223583`*^9, 3.69065199580422*^9}, 
   3.690738243366618*^9, {3.70472094095077*^9, 3.704720943182878*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dr", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dt", "=", "10"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fov", "=", "50"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"t", "=", "251"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"sd", "=", "15"}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"divt", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"divft", "[", 
        RowBox[{"x", ",", "y"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "24.5"}], ",", "24.5", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", 
         RowBox[{"-", "24.5"}], ",", "24.5", ",", "1"}], "}"}]}], "]"}]}], 
    ";"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"divt", "=", 
   RowBox[{
    RowBox[{"Import", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"dirs", "[", 
        RowBox[{"[", "sd", "]"}], "]"}], "<>", "\"\</analysis/div_dr\>\"", "<>", 
       RowBox[{"ToString", "[", "dr", "]"}], "<>", "\"\<_nbins10_dt\>\"", "<>", 
       RowBox[{"ToString", "[", "dt", "]"}], "<>", "\"\<.dat\>\""}], ",", 
      "\"\<Table\>\""}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"t", "-", "5"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"divt", "=", 
   RowBox[{"Partition", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"ToExpression", "[", 
         RowBox[{"StringReplace", "[", 
          RowBox[{"#", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"\"\<,\>\"", "\[Rule]", "\"\<\>\""}], ",", 
             RowBox[{"\"\<{\>\"", "\[Rule]", "\"\<\>\""}], ",", 
             RowBox[{"\"\<}\>\"", "\[Rule]", "\"\<\>\""}]}], "}"}]}], "]"}], 
         "]"}], "&"}], ",", "divt"}], "]"}], ",", "50"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"bct", "=", 
   RowBox[{
    RowBox[{"ToExpression", "[", 
     RowBox[{"Import", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"dirs", "[", 
         RowBox[{"[", "sd", "]"}], "]"}], "<>", 
        "\"\</analysis/bin_counts_dr\>\"", "<>", 
        RowBox[{"ToString", "[", "dr", "]"}], "<>", "\"\<.dat\>\""}], ",", 
       "\"\<TSV\>\""}], "]"}], "]"}], "[", 
    RowBox[{"[", 
     RowBox[{"t", "-", "5"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"xs", "=", 
   RowBox[{"ys", "=", 
    RowBox[{"Range", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "fov"}], "/", "2"}], "+", "0.5"}], ",", 
      RowBox[{
       RowBox[{"fov", "/", "2"}], "-", "0.5"}], ",", "1"}], "]"}]}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"dat", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xs", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{"ys", "[", 
          RowBox[{"[", "j", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"divt", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "j"}], "]"}], "]"}], "*", 
          RowBox[{"bct", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "j"}], "]"}], "]"}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "xs", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"Length", "[", "ys", "]"}]}], "}"}]}], "]"}], ",", "1"}], 
    "]"}]}], ";"}], "\n"}], "Input",
 CellChangeTimes->{{3.69497125931572*^9, 3.694971264151162*^9}, 
   3.6949717481974173`*^9, {3.703851173224712*^9, 3.703851243286207*^9}, {
   3.703851283268503*^9, 3.7038513331544647`*^9}, {3.703851377978434*^9, 
   3.7038514031546793`*^9}, {3.703853452134985*^9, 3.703853586547423*^9}, 
   3.703854901872304*^9, {3.7038550091658792`*^9, 3.7038550096290693`*^9}, 
   3.703855047934*^9, {3.70385515983675*^9, 3.703855210799986*^9}, {
   3.703855546087817*^9, 3.7038555524155684`*^9}, {3.703856477289688*^9, 
   3.703856477336185*^9}, {3.7038565501780376`*^9, 3.703856561154489*^9}, {
   3.703856928574032*^9, 3.703856932100459*^9}, {3.7041295307154503`*^9, 
   3.704129552232456*^9}, {3.7041300519752693`*^9, 3.704130054948456*^9}, {
   3.7047244789033203`*^9, 3.704724479115469*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dplt2", "=", 
   RowBox[{"ListDensityPlot", "[", 
    RowBox[{"dat", ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"25", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], ",", 
        RowBox[{"25", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "1"}], "}"}]}], ",", "All"}], "}"}]}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"ColorFunction", "\[Rule]", 
      RowBox[{"ColorData", "[", 
       RowBox[{"{", 
        RowBox[{"\"\<ThermometerColors\>\"", ",", 
         RowBox[{"0.15", 
          RowBox[{"{", 
           RowBox[{"1", ",", 
            RowBox[{"-", "1"}]}], "}"}]}]}], "}"}], "]"}]}], ",", 
     RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameTicksStyle", "\[Rule]", "White"}], ",", 
     RowBox[{"InterpolationOrder", "\[Rule]", "0"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.703854906192594*^9, 3.7038549063915863`*^9}, {
   3.703854939927355*^9, 3.703854968478486*^9}, {3.703855074164619*^9, 
   3.703855082588262*^9}, {3.7038566019274187`*^9, 3.7038566110764513`*^9}, {
   3.703856874535207*^9, 3.703856883367394*^9}, {3.703856950538316*^9, 
   3.7038569665576487`*^9}, 3.704129602697545*^9, 3.70412967503988*^9, {
   3.704130081773176*^9, 3.704130166761527*^9}}],

Cell[BoxData[
 RowBox[{"Overlay", "[", 
  RowBox[{"{", 
   RowBox[{"dplt2", ",", "vplt"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.703851324564616*^9, 3.7038513250420094`*^9}, {
  3.703857049786484*^9, 3.703857052233221*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.704130157741778*^9, 3.704130159105362*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S7E.png\>\""}], ",", 
    RowBox[{"Overlay", "[", 
     RowBox[{"{", 
      RowBox[{"dplt2", ",", "vplt"}], "}"}], "]"}]}], "]"}], ";"}]], "Input",
 CellChangeTimes->{
  3.69497177933112*^9, {3.694971821972698*^9, 3.694971822619314*^9}, {
   3.7041301873452787`*^9, 3.7041301911845827`*^9}, {3.704720959047618*^9, 
   3.704720959334921*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["F", "Subsubsection",
 CellChangeTimes->{{3.6906468555406313`*^9, 3.690646862657566*^9}, 
   3.704720970365963*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"fov", "=", "50"}], ";", 
  RowBox[{"tf", "=", "379"}], ";", 
  RowBox[{"dt", "=", "10"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"drs", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "5", ",", "10", ",", "25", ",", "50"}], 
    "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.690570318909379*^9, 3.690570326692485*^9}, {
  3.690573803924961*^9, 3.6905738048456163`*^9}, {3.6906354862025747`*^9, 
  3.690635512656657*^9}, {3.690651698004421*^9, 3.690651705803833*^9}, {
  3.694286228210342*^9, 3.694286232198764*^9}, {3.704721078428165*^9, 
  3.704721078671515*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"bcsdr", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Total", "[", 
         RowBox[{"Flatten", "[", "#", "]"}], "]"}], "&"}], ",", 
       RowBox[{"Partition", "[", 
        RowBox[{
         RowBox[{"bcsdt", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"dr", ",", "dr"}], "}"}]}], "]"}], ",", 
       RowBox[{"{", "2", "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "drs"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "bcsdt", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"bcsdt", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"divsdr", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Total", "[", 
         RowBox[{"Flatten", "[", "#", "]"}], "]"}], "&"}], ",", 
       RowBox[{"Partition", "[", 
        RowBox[{
         RowBox[{"divs", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"dr", ",", "dr"}], "}"}]}], "]"}], ",", 
       RowBox[{"{", "2", "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "drs"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "divs", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"divs", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"rhodivs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"bcsdr", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j", ",", "t"}], "]"}], "]"}], "*", 
        RowBox[{
         RowBox[{"divsdr", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j", ",", "t"}], "]"}], "]"}], "/", 
         RowBox[{
          RowBox[{"drs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "^", "2"}]}]}], ",", "2"}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "divsdr", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"divsdr", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"divsdr", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"meandiv", "=", 
   RowBox[{"Mean", "/@", "rhodivs"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6942849297429323`*^9, 3.694284931086034*^9}, {
  3.6942849865433283`*^9, 3.694285019023225*^9}, {3.694285308946691*^9, 
  3.694285342395812*^9}, {3.6942862629606323`*^9, 3.694286280505183*^9}, {
  3.6948315129940968`*^9, 3.69483151768209*^9}, {3.704722562749526*^9, 
  3.704722568706708*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"meandivdrs", "=", "meandiv"}], ";"}]], "Input",
 CellChangeTimes->{{3.704722113325429*^9, 3.704722116145541*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"divrng", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "70"}], ",", "2"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.704721238130816*^9, 3.704721290350347*^9}, {
  3.7047227306826057`*^9, 3.704722742778373*^9}}],

Cell[BoxData[
 RowBox[{"divdrplt", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"meandivdrs", ",", "\[IndentingNewLine]", 
    RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotRange", "\[Rule]", "divrng"}], ",", "\[IndentingNewLine]", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<time (s)\>\"", ",", 
       "\"\<\[Integral]\!\(\*SubscriptBox[\(\[Rho]\), \(a\)]\)\
\[LeftAngleBracket]\[Del]\[CenterDot]v\[RightAngleBracket] dA  (Hz)\>\""}], 
      "}"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotLegends", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{
       RowBox[{"SwatchLegend", "[", 
        RowBox[{"drs", ",", 
         RowBox[{"Spacings", "\[Rule]", "0"}], ",", 
         RowBox[{"LegendLayout", "\[Rule]", "\"\<ReversedColumn\>\""}], ",", 
         RowBox[{"LegendMarkerSize", "\[Rule]", "15"}], ",", 
         RowBox[{"LabelStyle", "\[Rule]", 
          RowBox[{"FontSize", "\[Rule]", "17"}]}], ",", 
         RowBox[{"LegendLabel", "\[Rule]", 
          RowBox[{"Placed", "[", 
           RowBox[{
            RowBox[{"Rotate", "[", 
             RowBox[{
             "\"\<(dA\!\(\*SuperscriptBox[\()\), \(1/2\)]\)(\[Mu]m)\>\"", ",", 
              RowBox[{"Pi", "/", "2"}]}], "]"}], ",", "Left"}], "]"}]}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.16", ",", "0.33"}], "}"}]}], "]"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"FrameTicks", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"LinTicks", "[", 
          RowBox[{"0", ",", "400", ",", "100", ",", "10"}], "]"}], ",", 
         "Automatic"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.690636144964364*^9, 3.690636283589464*^9}, 
   3.690636343462289*^9, {3.690640131775195*^9, 3.690640137377305*^9}, 
   3.690640569355448*^9, {3.6906427828177023`*^9, 3.690642854184494*^9}, {
   3.690642937816193*^9, 3.690643009675737*^9}, {3.690643139547955*^9, 
   3.690643283341263*^9}, {3.69064331471973*^9, 3.6906434400176907`*^9}, {
   3.6906435320807743`*^9, 3.6906436049212523`*^9}, {3.690643731770955*^9, 
   3.690643784272748*^9}, {3.690643818536108*^9, 3.6906438452096767`*^9}, {
   3.690644112179247*^9, 3.690644129519923*^9}, {3.6926702313704348`*^9, 
   3.692670231516656*^9}, {3.692670272129177*^9, 3.6926702807640676`*^9}, {
   3.692670321634878*^9, 3.692670402924306*^9}, {3.69267049916189*^9, 
   3.6926705076439743`*^9}, {3.692672069753892*^9, 3.69267216688148*^9}, {
   3.692672201156064*^9, 3.6926722954670153`*^9}, {3.694286521765635*^9, 
   3.694286529860632*^9}, {3.703864408538518*^9, 3.703864473032233*^9}, {
   3.70386453439217*^9, 3.703864534556683*^9}, {3.7038645861881638`*^9, 
   3.703864591707446*^9}, {3.703864625737878*^9, 3.703864634978149*^9}, {
   3.7038646739798326`*^9, 3.7038647118977127`*^9}, {3.704131756710609*^9, 
   3.704131759762484*^9}, {3.704131793809469*^9, 3.704131817322117*^9}, {
   3.704721000639092*^9, 3.7047210025299683`*^9}, {3.704721203315292*^9, 
   3.704721229399391*^9}, {3.704724506336088*^9, 3.704724506550255*^9}, {
   3.704732868737206*^9, 3.704732870984509*^9}, {3.7047330527250767`*^9, 
   3.70473309751651*^9}, {3.704733148018367*^9, 3.7047331803606853`*^9}, {
   3.704733228951836*^9, 3.704733233374997*^9}, {3.704733275894346*^9, 
   3.704733306693214*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S7F.eps\>\""}], ",", "divdrplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.690644132579137*^9, 3.6906441721322803`*^9}, {
   3.703864728143703*^9, 3.703864729297195*^9}, {3.704131828327149*^9, 
   3.704131829119183*^9}, {3.7047246383788643`*^9, 3.7047246602258787`*^9}, 
   3.704732828641013*^9, 3.704733329276738*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["G", "Subsubsection",
 CellChangeTimes->{{3.690646883905723*^9, 3.690646892980994*^9}, 
   3.704797141635213*^9, {3.704799330617579*^9, 3.7047993307726517`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dts", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "5", ",", "10", ",", "20"}], "}"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"fov", "=", "50"}], ";", 
  RowBox[{"tf", "=", "380"}], ";"}]}], "Input",
 CellChangeTimes->{{3.690646963991263*^9, 3.6906469681807632`*^9}, {
   3.694286789433125*^9, 3.69428683495639*^9}, 3.694287444261509*^9, {
   3.694287751956976*^9, 3.6942877690844927`*^9}, {3.694290153519849*^9, 
   3.694290156279235*^9}, 3.703866232596181*^9, {3.704135941979219*^9, 
   3.7041359428274307`*^9}, {3.704724742952458*^9, 3.70472474687164*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"bcs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ToExpression", "[", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dr", "<>", "\"\</analysis/bin_counts_dr1.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.6942888220448303`*^9, 3.694290177614567*^9, {3.703866014434729*^9, 
   3.703866018217762*^9}, 3.704724681945446*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Dimensions", "[", "bcs", "]"}]], "Input",
 CellChangeTimes->{{3.704724771717691*^9, 3.704724773174817*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"20", ",", "380", ",", "50", ",", "50"}], "}"}]], "Output",
 CellChangeTimes->{3.704724773571588*^9}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"dr", "=", "1"}], ";"}], "*)"}], "\n", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"bcsdr", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Total", "[", 
           RowBox[{"Flatten", "[", "#", "]"}], "]"}], "&"}], ",", 
         RowBox[{"Partition", "[", 
          RowBox[{
           RowBox[{"bcs", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "t"}], "]"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"dr", ",", "dr"}], "}"}]}], "]"}], ",", 
         RowBox[{"{", "2", "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "bcs", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"t", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"bcs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "*)"}], "\n", 
  RowBox[{
   RowBox[{
    RowBox[{"bcsdts", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"MovingAverage", "[", 
        RowBox[{
         RowBox[{"bcs", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", "dt"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"dt", ",", "dts"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "bcs", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"bcsdts", "=", 
     RowBox[{"0.5", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"bcsdts", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "All", ",", 
           RowBox[{"2", ";;"}]}], "]"}], "]"}], "+", 
        RowBox[{"bcsdts", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "All", ",", 
           RowBox[{";;", 
            RowBox[{"-", "2"}]}]}], "]"}], "]"}]}], ")"}]}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.7047247306504908`*^9, 3.70472480243635*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"divsdts", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ToExpression", "@", 
      RowBox[{"Import", "[", 
       RowBox[{
        RowBox[{"dir", "<>", "\"\</analysis/div_dr1_nbins10_dt\>\"", "<>", 
         RowBox[{"ToString", "[", "dt", "]"}], "<>", "\"\<.dat\>\""}], ",", 
        "\"\<TSV\>\""}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"dt", ",", "dts"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"rhodivsdts", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Total", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"bcsdts", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j", ",", "t"}], "]"}], "]"}], "*", 
         RowBox[{"divsdts", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j", ",", "t"}], "]"}], "]"}]}], ",", "2"}], 
       "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "divsdts", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"divsdts", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"t", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"divsdts", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
   ";"}], "\n"}]], "Input",
 CellChangeTimes->{{3.694289843100998*^9, 3.69428985918073*^9}, {
   3.6943041162313833`*^9, 3.694304122255838*^9}, {3.7041359883620243`*^9, 
   3.704136002025095*^9}, 3.7041363637755737`*^9, {3.704725172717968*^9, 
   3.704725217813016*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.7047252635882*^9, 3.704725264618257*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"meandivdts", "=", 
   RowBox[{"Mean", "/@", "rhodivsdts"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.7047252675703373`*^9, 3.70472527088181*^9}}],

Cell[BoxData[
 RowBox[{"divdtplt", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"meandivdts", ",", "\[IndentingNewLine]", 
    RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotRange", "\[Rule]", "divrng"}], ",", "\[IndentingNewLine]", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<time (s)\>\"", ",", 
       "\"\<\[Integral]\!\(\*SubscriptBox[\(\[Rho]\), \(a\)]\)\
\[LeftAngleBracket]\[Del]\[CenterDot]v\[RightAngleBracket] dA  (Hz)\>\""}], 
      "}"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"PlotLegends", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{
       RowBox[{"SwatchLegend", "[", 
        RowBox[{"dts", ",", 
         RowBox[{"Spacings", "\[Rule]", "0"}], ",", 
         RowBox[{"LegendLayout", "\[Rule]", "\"\<ReversedColumn\>\""}], ",", 
         RowBox[{"LegendMarkerSize", "\[Rule]", "15"}], ",", 
         RowBox[{"LabelStyle", "\[Rule]", 
          RowBox[{"FontSize", "\[Rule]", "17"}]}], ",", 
         RowBox[{"LegendLabel", "\[Rule]", 
          RowBox[{"Placed", "[", 
           RowBox[{"\"\<  h (sec)\>\"", ",", "Bottom"}], "]"}]}]}], "]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"0.2", ",", "0.35"}], "}"}]}], 
      RowBox[{"(*", 
       RowBox[{"{", 
        RowBox[{"0.8", ",", "0.25"}], "}"}], "*)"}], "]"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"FrameTicks", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Automatic", ",", "Automatic"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"LinTicks", "[", 
          RowBox[{"0", ",", "400", ",", "100", ",", "10"}], "]"}], ",", 
         "Automatic"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.690636144964364*^9, 3.690636283589464*^9}, 
   3.690636343462289*^9, {3.690640131775195*^9, 3.690640137377305*^9}, 
   3.690640569355448*^9, {3.6906427828177023`*^9, 3.690642854184494*^9}, {
   3.690642937816193*^9, 3.690643009675737*^9}, {3.690643139547955*^9, 
   3.690643283341263*^9}, {3.69064331471973*^9, 3.6906434400176907`*^9}, {
   3.6906435320807743`*^9, 3.6906436049212523`*^9}, {3.690643731770955*^9, 
   3.690643784272748*^9}, {3.690643818536108*^9, 3.6906438452096767`*^9}, {
   3.690644112179247*^9, 3.690644129519923*^9}, {3.690648474422998*^9, 
   3.690648499693902*^9}, {3.690648535213551*^9, 3.69064858414774*^9}, {
   3.692671815158641*^9, 3.692671843910318*^9}, {3.69267190220094*^9, 
   3.692671929816902*^9}, {3.692672314853366*^9, 3.692672410867872*^9}, {
   3.694289874341537*^9, 3.6942898751255198`*^9}, {3.694304089071681*^9, 
   3.694304096935276*^9}, {3.694304136537115*^9, 3.694304141440246*^9}, {
   3.694308586299491*^9, 3.694308587833744*^9}, {3.704136377017702*^9, 
   3.704136460022956*^9}, {3.7047252756023073`*^9, 3.704725283498517*^9}, {
   3.704733377870224*^9, 3.70473341055438*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S7G.eps\>\""}], ",", "divdtplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.690648618993516*^9, 3.690648622322097*^9}, {
  3.694308599825939*^9, 3.694308600145821*^9}, {3.704136472909487*^9, 
  3.704136474805489*^9}, {3.704733429601194*^9, 3.7047334360011187`*^9}}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["S4", "Section",
 CellChangeTimes->{{3.694831609004562*^9, 3.694831620425478*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dx", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nfils", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nlks", "=", 
   RowBox[{"L", "/", "dx"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nacts", "=", 
   RowBox[{"nlks", "+", "1"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"teq", "=", "1000"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lppred", "=", "17"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xs", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"dx", ",", 
     RowBox[{"L", "-", "dx"}], ",", "dx"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"maxl", "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lp", "=", "17"}], ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.694893216377878*^9, 3.694893216570236*^9}, {
   3.694963983318602*^9, 3.694963984405876*^9}, {3.7045660944755507`*^9, 
   3.7045660946931343`*^9}, 3.704814487630651*^9, {3.707580469697925*^9, 
   3.707580471578187*^9}}],

Cell[CellGroupData[{

Cell["A", "Subsection",
 CellChangeTimes->{{3.694890558001624*^9, 3.6948905680200357`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"figS4Adir", "=", 
   RowBox[{"figS4dir", "<>", "\"\</fiber20/seg1.00\>\""}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"acts", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"Partition", "[", 
      RowBox[{"#", ",", "nacts"}], "]"}], "&"}], "/@", 
    RowBox[{"ptsCS", "[", 
     RowBox[{"figS4Adir", ",", "\"\<objects\>\""}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"lks", "=", 
    RowBox[{
     RowBox[{"acts", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "All", ",", 
        RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
     RowBox[{"acts", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "All", ",", 
        RowBox[{";;", 
         RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"angs", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"anglesTCS", ",", "lks", ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"angsbw", "=", 
    RowBox[{"Map", "[", 
     RowBox[{"anglesBwT", ",", "angs", ",", 
      RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"th2s", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"th2", ",", "angsbw", ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"muths", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"th2s", "[", 
     RowBox[{"[", 
      RowBox[{"teq", ";;"}], "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"muthsmu", "=", 
   RowBox[{"Mean", "[", "muths", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"muthserr", "=", 
    RowBox[{
     RowBox[{"StandardDeviation", "[", "muths", "]"}], "/", 
     RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ccfs", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"ccf", ",", "angsbw", ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"muccfs", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"ccfs", "[", 
     RowBox[{"[", 
      RowBox[{"teq", ";;", 
       RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"muccfsmu", "=", 
   RowBox[{"Mean", "[", "muccfs", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muccferr", "=", 
   RowBox[{
    RowBox[{"StandardDeviation", "[", "muccfs", "]"}], "/", 
    RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.689511056002594*^9, 3.689511072859887*^9}, 
   3.694890750600453*^9, {3.694890829470446*^9, 3.694890854037586*^9}, {
   3.6949601534740877`*^9, 3.694960154336829*^9}, {3.704809037511401*^9, 
   3.7048090494463*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"th2plt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListPlot", "[", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", "muthsmu"}], "}"}], "\[Transpose]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muthsmu", "-", "muthserr"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muthsmu", "+", "muthserr"}]}], "}"}], "\[Transpose]"}]}],
         "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
          RowBox[{"Rotate", "[", 
           RowBox[{
            RowBox[{"Style", "[", 
             RowBox[{
             "\"\<\[LeftAngleBracket]\!\(\*SuperscriptBox[\(\[Theta]\), \
\(2\)]\)(\[ScriptL])\[RightAngleBracket] (\[FilledSquare])\>\"", ",", 
              "Blue"}], "]"}], ",", "Pi"}], "]"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameStyle", "\[Rule]", "Blue"}], ",", "\[IndentingNewLine]", 
       
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"False", ",", "True"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"False", ",", "False"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"None", ",", "All"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"None", ",", "None"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Comment", " ", "out", " ", "next", " ", "three", " ", "lines", " ", 
         "if", " ", "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Blue", ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Blue", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "5"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Joined", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"False", ",", "True", ",", "True"}], "}"}]}], ","}], 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Uncomment", " ", "next", " ", "four", " ", "line", " ", "if", " ", 
         "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"FillingStyle", "\[Rule]", 
          RowBox[{"Directive", "[", 
           RowBox[{
            RowBox[{"Opacity", "[", "1", "]"}], ",", 
            RowBox[{"Thickness", "[", "0.003", "]"}]}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "12"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"PlotStyle", "\[Rule]", "Blue"}], ","}], 
        "\[IndentingNewLine]", "\[IndentingNewLine]", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "12"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ","}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Rectangle", "[", "]"}]}], "}"}], "]"}], ",", 
            "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"0.75", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Blue", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"0.75", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", "Blue"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"2", "\[Rule]", 
          RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "20"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", "/", 
         RowBox[{"(", "lp", ")"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "50"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Thick", ",", 
          RowBox[{"Dashing", "[", "Large", "]"}], ",", "Blue"}], "}"}]}]}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.668427445277066*^9, 3.668427486875223*^9}, {
   3.668427521157255*^9, 3.668427527602787*^9}, {3.668427925859844*^9, 
   3.668427928592771*^9}, 3.6690295624217243`*^9, 3.66903615163303*^9, {
   3.669036923248163*^9, 3.669036931030847*^9}, {3.669042061073771*^9, 
   3.669042065880272*^9}, {3.6690422178006773`*^9, 3.669042217929419*^9}, {
   3.669042354772778*^9, 3.669042383106879*^9}, {3.669042416811346*^9, 
   3.669042438086665*^9}, 3.669042562020055*^9, {3.670194484092348*^9, 
   3.6701945327415867`*^9}, {3.670194609249497*^9, 3.670194636131197*^9}, {
   3.670194977018643*^9, 3.670194990837554*^9}, 3.6701950444269876`*^9, {
   3.688140567857744*^9, 3.6881405852347183`*^9}, 3.688219692712883*^9, {
   3.688235084651927*^9, 3.688235086032837*^9}, {3.688843888855977*^9, 
   3.6888440266919317`*^9}, 3.6888441160052233`*^9, {3.688844165555574*^9, 
   3.688844187952065*^9}, {3.6889021987668858`*^9, 3.688902211870509*^9}, {
   3.688902281828388*^9, 3.688902282452259*^9}, {3.688903386244416*^9, 
   3.6889033917797203`*^9}, {3.688907532434621*^9, 3.6889075426574497`*^9}, 
   3.689083287423666*^9, {3.692884131945891*^9, 3.692884159880598*^9}, {
   3.692884213108204*^9, 3.692884253305722*^9}, {3.692884329099326*^9, 
   3.692884338750135*^9}, {3.692884420265139*^9, 3.6928844245459623`*^9}, {
   3.7075804303502007`*^9, 3.707580432350152*^9}, {3.707580503323948*^9, 
   3.70758050610926*^9}, {3.70758551810686*^9, 3.707585521729763*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ccfplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListLogPlot", "[", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", "muccfsmu"}], "}"}], "\[Transpose]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muccfsmu", "-", "muccferr"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"xs", ",", 
            RowBox[{"muccfsmu", "+", "muccferr"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<\[ScriptL] (\[Mu]m)\>\"", ",", 
          RowBox[{"Style", "[", 
           RowBox[{
           "\"\<\[LeftAngleBracket]cos\[Theta](\[ScriptL])\[RightAngleBracket]\
 (\[FilledCircle])\>\"", ",", "Red"}], "]"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"Red", ",", "Automatic"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"Black", ",", "Black"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Frame", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"True", ",", "False"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"True", ",", "True"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameTicks", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"All", ",", "None"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"All", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Comment", " ", "out", " ", "next", " ", "three", " ", "lines", " ", 
         "if", " ", "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Red", ",", 
            RowBox[{"{", 
             RowBox[{"Red", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"Red", ",", 
              RowBox[{"Opacity", "[", "0.01", "]"}]}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "5"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"\[EmptyCircle]", ",", "0.01"}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Joined", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"False", ",", "True", ",", "True"}], "}"}]}], ","}], 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Uncomment", " ", "next", " ", "four", " ", "line", " ", "if", " ", 
         "you", " ", 
         RowBox[{"don", "'"}], "t", " ", "want", " ", "joined"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"FillingStyle", "\[Rule]", 
          RowBox[{"Directive", "[", 
           RowBox[{
            RowBox[{"Opacity", "[", "1", "]"}], ",", 
            RowBox[{"Thickness", "[", "0.003", "]"}]}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "12"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"PlotStyle", "\[Rule]", "Red"}], ",", "\[IndentingNewLine]", 
         
         RowBox[{"Joined", "\[Rule]", "False"}], ","}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Disk", "[", "]"}]}], "}"}], "]"}], ",", "0.045"}], 
           "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Line", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}],
              "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", "Red"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"2", "\[Rule]", 
          RowBox[{"{", "3", "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "20"}], "}"}], ",", "Automatic"}], "}"}]}]}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"LogPlot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "x"}], "/", 
          RowBox[{"(", 
           RowBox[{"2", "lp"}], ")"}]}], "]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "50"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Thick", ",", 
          RowBox[{"Dashing", "[", "Large", "]"}], ",", "Red"}], "}"}]}]}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.668428233508287*^9, 3.668428297399631*^9}, 
   3.668428336232131*^9, {3.668428378493622*^9, 3.668428422490027*^9}, {
   3.6684284567724953`*^9, 3.668428462457385*^9}, {3.668428507787252*^9, 
   3.668428511785948*^9}, {3.66904239761924*^9, 3.6690424074258623`*^9}, {
   3.669042451410597*^9, 3.669042488330946*^9}, 3.6690425701407003`*^9, {
   3.670194694374593*^9, 3.670194749703415*^9}, {3.6701947837596197`*^9, 
   3.670194814228922*^9}, {3.670194852589451*^9, 3.670194871265451*^9}, {
   3.670194921406143*^9, 3.6701949386383743`*^9}, {3.670195062437747*^9, 
   3.670195079870307*^9}, {3.6701955190647383`*^9, 3.6701955262349463`*^9}, {
   3.68814059241637*^9, 3.688140598239818*^9}, {3.688219267388626*^9, 
   3.6882192677852583`*^9}, 3.688219682079318*^9, {3.688233935161942*^9, 
   3.688233968274741*^9}, {3.688235093050988*^9, 3.6882350944496193`*^9}, {
   3.688844355293193*^9, 3.688844436729478*^9}, {3.6889022326942463`*^9, 
   3.6889022449238853`*^9}, {3.688902277036995*^9, 3.6889022772372437`*^9}, {
   3.688903333581965*^9, 3.688903341564417*^9}, {3.688903548376672*^9, 
   3.68890355895041*^9}, {3.688903650645397*^9, 3.6889036550446157`*^9}, {
   3.688903689135756*^9, 3.688903696538434*^9}, {3.6889075843623657`*^9, 
   3.6889075872650967`*^9}, {3.689083309278967*^9, 3.689083309759104*^9}, {
   3.692884460638665*^9, 3.692884480703891*^9}, {3.707580413031761*^9, 
   3.707580415742893*^9}, {3.707580460970964*^9, 3.707580461349615*^9}, {
   3.707585528057969*^9, 3.707585530642959*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"ccfplt", ",", "th2plt"}], "}"}]], "Input",
 CellChangeTimes->{{3.668428347267419*^9, 3.668428362876987*^9}, {
  3.668428545209313*^9, 3.668428547608214*^9}, {3.681487836453225*^9, 
  3.681487881018704*^9}, {3.704809075662418*^9, 3.704809078660715*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S4A_red.eps\>\""}], ",", "ccfplt"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S4A_blue.eps\>\""}], ",", "th2plt"}], "]"}],
   ";"}]}], "Input",
 CellChangeTimes->{{3.6888454322637253`*^9, 3.68884543302357*^9}, {
  3.688902342513777*^9, 3.688902370056594*^9}, {3.694890777367214*^9, 
  3.694890808102559*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["C", "Subsection",
 CellChangeTimes->{{3.694890966885312*^9, 3.694890972741623*^9}, 
   3.7046253401098022`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kbdir", "=", 
   RowBox[{"figS4dir", "<>", "\"\</fiber20_seg1\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kbs", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.005", ",", "0.01", ",", "0.02", ",", "0.05", ",", "0.1", ",", "0.2", 
     ",", "0.5", ",", "1.0", ",", "2.0", ",", "5", ",", "10", ",", "20", ",", 
     "50", ",", "100", ",", "200", ",", "500"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kbdirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"kbdir", "<>", "\"\</kb\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "3"}], "}"}], ",", 
         RowBox[{"ExponentFunction", "\[Rule]", 
          RowBox[{"(", 
           RowBox[{"Null", "&"}], ")"}]}]}], "]"}], "]"}]}], "&"}], "/@", 
    "kbs"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.688909031207704*^9, 3.6889090634199743`*^9}, {
   3.68890921290136*^9, 3.688909214521461*^9}, {3.688909248367621*^9, 
   3.688909250845167*^9}, 3.6889094257456017`*^9, 3.688909713826535*^9, {
   3.688909876310413*^9, 3.6889098800451813`*^9}, {3.689091979828493*^9, 
   3.6890919894572906`*^9}, 3.689092023741181*^9, {3.689092088754753*^9, 
   3.689092092435129*^9}, {3.689096314822639*^9, 3.6890963167415333`*^9}, {
   3.6894243567101593`*^9, 3.6894243640773373`*^9}, {3.689446591649548*^9, 
   3.689446591730237*^9}, {3.6948924892609863`*^9, 3.694892492259925*^9}, {
   3.694892530259523*^9, 3.694892531004545*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"acts", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Partition", "[", 
        RowBox[{"#", ",", "nacts"}], "]"}], "&"}], "/@", 
      RowBox[{"ptsCS", "[", 
       RowBox[{"dir", ",", "\"\<objects\>\""}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"dir", ",", "kbdirs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{
    RowBox[{"acts", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", "All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"acts", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", "All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lps", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"plMeanThsq", "[", 
      RowBox[{
       RowBox[{"lks", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "All", ",", "j"}], "]"}], "]"}], ",", "dx", ",", 
       "L", ",", "maxl"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "lks", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"lks", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"lpmeans", "=", 
   RowBox[{"Mean", "/@", "lps"}]}], ";"}]}], "Input",
 CellChangeTimes->{3.694962782104269*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lplpplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListLogLogPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"kbs", "/", "0.004"}], ",", "lpmeans"}], "}"}], 
         "\[Transpose]"}], 
        RowBox[{"(*", 
         RowBox[{",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"kbs", "[", 
               RowBox[{"[", 
                RowBox[{"3", ";;"}], "]"}], "]"}], "/", "0.004"}], ",", 
             RowBox[{"1", "/", 
              RowBox[{"msmidkb", "[", 
               RowBox[{"[", 
                RowBox[{"3", ";;"}], "]"}], "]"}]}]}], "}"}], 
           "\[Transpose]"}]}], "*)"}], "\[IndentingNewLine]", "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\[FilledCircle]", ",", "15"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Red", ",", 
          RowBox[{"{", 
           RowBox[{"Blue", ",", 
            RowBox[{"Opacity", "[", "0.75", "]"}]}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}]}], "\[IndentingNewLine]", "]"}],
      ",", "\[IndentingNewLine]", 
     RowBox[{"LogLogPlot", "[", 
      RowBox[{"x", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0.1", ",", "500000"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", 
          RowBox[{"Dashing", "[", "0.02", "]"}]}], 
         RowBox[{"(*", 
          RowBox[{",", "Thick"}], "*)"}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(\[Kappa]\), \(B\)]\) (\!\(\*SubscriptBox[\(k\
\), \(B\)]\)T\[Times]\[Mu]m)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(L\), \(\(p\)\(\\\ \)\)]\)(\[Mu]m)\>\""}], 
       "}"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.673699939102161*^9, 3.673700011867363*^9}, {
   3.673700352501178*^9, 3.673700376970888*^9}, {3.673700443211453*^9, 
   3.673700492096118*^9}, {3.6737005832553673`*^9, 3.673700798192556*^9}, {
   3.673700972293968*^9, 3.6737010583513193`*^9}, {3.673701109946662*^9, 
   3.6737011497922583`*^9}, {3.673701198712723*^9, 3.673701207038722*^9}, {
   3.6793386008167973`*^9, 3.6793386673352823`*^9}, {3.6793933330419283`*^9, 
   3.6793933339930687`*^9}, {3.688909926757944*^9, 3.688910115144164*^9}, {
   3.688910146425112*^9, 3.688910204333703*^9}, {3.6889103440035343`*^9, 
   3.688910391825008*^9}, {3.6890969403796463`*^9, 3.6890969409787207`*^9}, {
   3.689446793558056*^9, 3.68944683974835*^9}, {3.689446945289797*^9, 
   3.6894470732458572`*^9}, {3.6895119752448378`*^9, 3.689511999786426*^9}, 
   3.694893018433034*^9, 3.69496276016713*^9}],

Cell[BoxData["lplpplt"], "Input",
 CellChangeTimes->{{3.679339354022794*^9, 3.679339355484564*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S4B.eps\>\""}], ",", "lplpplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.688910400993116*^9, 3.688910440719552*^9}, {
  3.689447089005581*^9, 3.689447093318288*^9}, {3.694964086631386*^9, 
  3.6949640935516853`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsection",
 CellChangeTimes->{{3.694893038607576*^9, 3.6948930472028418`*^9}, 
   3.704625342733726*^9}],

Cell[CellGroupData[{

Cell["Red", "Subsubsection",
 CellChangeTimes->{{3.70462531018888*^9, 3.704625333130218*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"dxdir", "=", 
   RowBox[{"csoutdir", "<>", "\"\</vf_r2/pl/fiber20_dt1e-5/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dxs", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.08", ",", "0.1", ",", "0.2", ",", "0.25", ",", "0.4", ",", "0.5", ",", 
     "0.8", ",", "1", ",", "2", ",", "2.5", ",", "4"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dxdirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dxdir", "<>", "\"\</seg\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "2"}], "}"}], ",", 
         RowBox[{"ExponentFunction", "\[Rule]", 
          RowBox[{"(", 
           RowBox[{"Null", "&"}], ")"}]}]}], "]"}], "]"}]}], "&"}], "/@", 
    "dxs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nacts", "=", 
   RowBox[{
    RowBox[{"L", "/", "dxs"}], "+", "1"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"acts", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"ptsCS", "[", 
      RowBox[{"#", ",", "\"\<objects\>\""}], "]"}], "&"}], "/@", "dxdirs"}]}],
   ";"}], "\n", 
 RowBox[{
  RowBox[{"actsp", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Partition", "[", 
        RowBox[{"#", ",", 
         RowBox[{"Round", "[", 
          RowBox[{"nacts", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}], "&"}], "/@", 
      RowBox[{"acts", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "nacts", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{
    RowBox[{"actsp", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", "All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"actsp", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", "All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lps", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"plMeanThsq", "[", 
      RowBox[{
       RowBox[{"lks", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "All", ",", "j"}], "]"}], "]"}], ",", 
       RowBox[{"dxs", "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", "L", ",", "maxl"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "lks", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"lks", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"lpmeans", "=", 
   RowBox[{"Mean", "/@", "lps"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "lps"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.689097555503458*^9, 3.689097571208023*^9}, {
   3.689097618055848*^9, 3.689097618807582*^9}, {3.689205181675218*^9, 
   3.689205183107914*^9}, 3.689205627574235*^9, {3.689205795767713*^9, 
   3.68920579968746*^9}, {3.6892064996138897`*^9, 3.689206501602137*^9}, {
   3.689424386886306*^9, 3.689424389772966*^9}, {3.6894244435165854`*^9, 
   3.689424446251532*^9}, {3.689427349963771*^9, 3.689427350065324*^9}, {
   3.689512259267599*^9, 3.689512263403207*^9}, {3.68951231369773*^9, 
   3.689512313761372*^9}, {3.694893095237274*^9, 3.694893097180801*^9}, {
   3.694893134796274*^9, 3.694893189970757*^9}, {3.694893241801985*^9, 
   3.694893242625469*^9}, {3.694893360204031*^9, 3.694893365230847*^9}, {
   3.694960787166011*^9, 3.694960895837327*^9}, {3.69496279630486*^9, 
   3.694962833510364*^9}, {3.694963072794495*^9, 3.694963077423566*^9}, {
   3.704200704042799*^9, 3.704200706569662*^9}, {3.704566111869125*^9, 
   3.7045661142672567`*^9}, {3.7045661645248833`*^9, 3.7045661791061773`*^9}}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.707580574875889*^9}],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.7075805751642027`*^9}]
}, Closed]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lpmmeanscs", "=", "lpmeans"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrscs", "=", "lperrs"}], ";"}]}], "Input",
 CellChangeTimes->{{3.704625278097076*^9, 3.7046252903517437`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lpmeanscs", "=", 
   RowBox[{"lpmmeanscs", "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrscs", "=", 
   RowBox[{"lperrscs", "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", 
      RowBox[{"-", "2"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.704812804387863*^9, 3.7048128203552713`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{
    "csoutdir", "<>", "\"\</vf_r2/pl/fiber20_dt1e-5/pls_mu_err.dat\>\""}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"lpmmeanscs", ",", "lperrscs"}], "}"}]}], "]"}], ";"}]], "Input",\

 CellChangeTimes->{{3.7046254023429623`*^9, 3.704625455316152*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ldat", "=", 
   RowBox[{"importCheck", "[", 
    RowBox[{
    "csoutdir", "<>", "\"\</vf_r2/pl/fiber20_dt1e-5/pls_mu_err.dat\>\""}], 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.704625773548175*^9, 3.704625817702944*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"lpmeanscs", ",", "lperrscs"}], "}"}], "=", 
   RowBox[{"ldat", "[", 
    RowBox[{"[", 
     RowBox[{"All", ",", 
      RowBox[{"1", ";;", 
       RowBox[{"-", "2"}]}]}], "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.7046258281659107`*^9, 3.704625843809739*^9}, {
  3.704625925640585*^9, 3.704625927895886*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Blue", "Subsubsection",
 CellChangeTimes->{{3.704625357404627*^9, 3.704625362926179*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"teq", "=", "100"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dxdir", "=", 
   RowBox[{"vfdir", "<>", "\"\</pl/dx_arr_kldx1/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dxs", "=", 
   RowBox[{"{", 
    RowBox[{
    "0.08", ",", "0.1", ",", "0.2", ",", "0.25", ",", "0.4", ",", "0.5", ",", 
     "0.8", ",", "1", ",", "2", ",", "2.5"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Ls", "=", 
   RowBox[{"nlks", "*", "dxs"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dxdirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dxdir", "<>", "\"\</dx\>\"", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "2"}], "}"}], ",", 
         RowBox[{"ExponentFunction", "\[Rule]", 
          RowBox[{"(", 
           RowBox[{"Null", "&"}], ")"}]}]}], "]"}], "]"}]}], "&"}], "/@", 
    "dxs"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"lks", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Partition", "[", 
        RowBox[{"#", ",", "nlks"}], "]"}], "&"}], "/@", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"pts", "[", 
         RowBox[{
          RowBox[{"dxdirs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "\"\<links\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"teq", ";;", 
          RowBox[{"-", "2"}], ";;", "2"}], "]"}], "]"}], ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "dxs", "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"lps", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"plMeanThsq", "[", 
         RowBox[{
          RowBox[{"lks", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "All", ",", "j", ",", "All", ",", 
             RowBox[{"3", ";;", "4"}]}], "]"}], "]"}], ",", 
          RowBox[{"dxs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", 
          RowBox[{"Ls", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "maxl"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", 
          RowBox[{"Length", "[", 
           RowBox[{"lks", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "lks", "]"}]}], "}"}]}], "]"}]}], ";"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"lpmeans", "=", 
   RowBox[{"Mean", "/@", "lps"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lperrs", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"StandardDeviation", "/@", "lps"}], ")"}], "/", 
    RowBox[{"Sqrt", "[", "nfils", "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.688912702925228*^9, 3.688912777908717*^9, {3.688912836186125*^9, 
   3.688912872664366*^9}, {3.6889141888718233`*^9, 3.688914192078882*^9}, {
   3.688914232999997*^9, 3.68891423819844*^9}, {3.6889156079958963`*^9, 
   3.688915628410925*^9}, {3.688915712241868*^9, 3.6889157910153093`*^9}, {
   3.688916481431432*^9, 3.688916481702526*^9}, {3.688917041432994*^9, 
   3.688917045319569*^9}, {3.688920465913409*^9, 3.6889204823842688`*^9}, 
   3.688920688443486*^9, 3.690047959754725*^9, {3.690048864204405*^9, 
   3.690048927851812*^9}, {3.6907135094297943`*^9, 3.6907135096447783`*^9}, {
   3.690715087404726*^9, 3.690715087611973*^9}, {3.690715944335157*^9, 
   3.690715973365397*^9}, {3.690716200968707*^9, 3.690716256390687*^9}, {
   3.690716574055274*^9, 3.690716581275573*^9}, {3.690718523429895*^9, 
   3.690718526157374*^9}, {3.690718648683062*^9, 3.690718654809513*^9}, 
   3.690718748600213*^9, {3.694827643074876*^9, 3.6948276656675997`*^9}, 
   3.694827952485569*^9, 3.6949594067003107`*^9, 3.7023217103286333`*^9, 
   3.70462525349681*^9, {3.70462547754821*^9, 3.704625480539412*^9}, 
   3.704625531830041*^9, {3.704625596737173*^9, 3.704625600568787*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Lp", "=", "17"}], ";"}]], "Input",
 CellChangeTimes->{{3.707580794670904*^9, 3.7075808016650953`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"dxlpplt", "=", 
   RowBox[{"Show", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ListLogLinearPlot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", "lpmeans"}], "}"}], "\[Transpose]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", 
            RowBox[{"lpmeans", "-", "lperrs"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", 
            RowBox[{"lpmeans", "+", "lperrs"}]}], "}"}], "\[Transpose]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", "lpmeanscs"}], "}"}], "\[Transpose]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", 
            RowBox[{"lpmeanscs", "-", "lperrscs"}]}], "}"}], "\[Transpose]"}],
          ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"dxs", ",", 
            RowBox[{"lpmeanscs", "+", "lperrscs"}]}], "}"}], 
          "\[Transpose]"}]}], "\[IndentingNewLine]", "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Disk", "[", "]"}], "}"}], "]"}], ",", "0.045"}], "}"}],
           ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Graphics", "[", 
             RowBox[{"{", 
              RowBox[{"Line", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "0.75"}], ",", "0"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0.75", ",", "0"}], "}"}]}], "}"}], "]"}], "}"}], 
             "]"}], ",", "0.045"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"PlotMarkers", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\"", ",", 
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", "\"\<+\>\"", 
            ",", "\"\<+\>\""}], "}"}]}], ","}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "Blue", ",", "Blue", ",", "Blue", ",", "Red", ",", "Red", ",", 
          "Red"}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0.07", ",", "1.2"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"14", ",", "18.1"}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Joined", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Filling", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"2", "\[Rule]", 
            RowBox[{"{", "3", "}"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"5", "\[Rule]", 
            RowBox[{"{", "6", "}"}]}], "}"}]}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FillingStyle", "\[Rule]", 
        RowBox[{"Directive", "[", 
         RowBox[{
          RowBox[{"Opacity", "[", "1", "]"}], ",", 
          RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"Placed", "[", 
         RowBox[{
          RowBox[{"SwatchLegend", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
               "\"\<AFINES\>\"", ",", "None", ",", "None", ",", 
                "\"\<Cytosim\>\""}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"Blue", ",", "Red"}], "}"}]}], "}"}], ",", 
            RowBox[{"LegendMarkers", "\[Rule]", 
             RowBox[{"{", 
              RowBox[{"\[FilledCircle]", ",", "15"}], "}"}]}], ",", 
            RowBox[{"LegendFunction", "\[Rule]", "Framed"}], ",", 
            RowBox[{"LabelStyle", "->", 
             RowBox[{"FontSize", "\[Rule]", "20"}]}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"0.3", ",", "0.25"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"LogLinearPlot", "[", 
      RowBox[{"Lp", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0.001", ",", "10000"}], "}"}], ",", 
       RowBox[{"PlotStyle", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"Black", ",", 
          RowBox[{"Dashing", "[", "0.02", "]"}]}], "}"}]}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(l\), \(a\)]\) (\[Mu]m)\>\"", ",", 
        "\"\<\!\(\*SubscriptBox[\(L\), \(\(p\)\(\\\ \)\)]\)(\[Mu]m)\>\""}], 
       "}"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.688915903573367*^9, 3.688915957026971*^9}, {
   3.68891659836347*^9, 3.68891663991546*^9}, {3.690123543371933*^9, 
   3.6901235504030657`*^9}, {3.690715379838122*^9, 3.690715439348922*^9}, {
   3.69071651445778*^9, 3.690716528856823*^9}, {3.694827960550668*^9, 
   3.694827982199066*^9}, {3.6949597004147263`*^9, 3.69495970074914*^9}, {
   3.6949597643754797`*^9, 3.694959780165606*^9}, {3.704625886474324*^9, 
   3.704625964751443*^9}, {3.704626042815238*^9, 3.704626068183024*^9}, {
   3.704626115701227*^9, 3.704626138826796*^9}, {3.70462617581185*^9, 
   3.704626235809564*^9}, {3.70462637977365*^9, 3.704626406941181*^9}, {
   3.7046264755798807`*^9, 3.704626509242672*^9}, {3.704812694891088*^9, 
   3.704812697615814*^9}, {3.704812770583865*^9, 3.704812796437799*^9}, {
   3.7048128348452883`*^9, 3.704812836804398*^9}, {3.707580606223784*^9, 
   3.707580607425292*^9}, {3.7075806725582037`*^9, 3.707580672782707*^9}, {
   3.707580733780943*^9, 3.7075807535077343`*^9}, 3.707580806523423*^9}],

Cell[BoxData["dxlpplt"], "Input",
 CellChangeTimes->{{3.704625978566663*^9, 3.704625979759696*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</dx_cytosim_afines.eps\>\""}], ",", 
    "dxlpplt"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.7046262471866827`*^9, 3.704626280463006*^9}, {
  3.704626339365552*^9, 3.7046263672289248`*^9}, {3.704626498289544*^9, 
  3.7046265265775957`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S4C.eps\>\""}], ",", "lplpplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{3.6949641095348577`*^9}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["S6", "Section",
 CellChangeTimes->{{3.694831622670178*^9, 3.694831627795074*^9}, {
  3.704799576264473*^9, 3.7047995782355757`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rd0", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]0", "=", "4"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L0", "=", "15"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Lm", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"koff0", "=", "1"}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell["A", "Subsection",
 CellChangeTimes->{{3.6814983603361387`*^9, 3.6814983709557962`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"figS5dir", "<>", "\"\</kon_arr/sd1000000/kon\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ks", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Flatten", "[", 
      RowBox[{"Outer", "[", 
       RowBox[{"Times", ",", 
        RowBox[{"{", 
         RowBox[{"0.001", ",", "0.01", ",", "0.1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "2", ",", "5"}], "}"}]}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"10", ",", "20"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dir", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}]}], "&"}], "/@", 
    "ks"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"konarr", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"ptsCS", "[", 
      RowBox[{"#", ",", "\"\<objects\>\""}], "]"}], "&"}], "/@", "dirs"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"konarr", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"rijP", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", 
         RowBox[{"50", ",", "50"}], "}"}]}], "]"}], "&"}], ",", "konarr", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"konarr", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", "0.5", "}"}]}], "]"}], "&"}], ",", "konarr", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"maxf", "=", "401"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cf", "=", 
   RowBox[{"ColorData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<DeepSeaColors\>\"", ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "maxf"}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"cf2", "[", "i_", "]"}], ":=", 
   RowBox[{"Blend", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Pink", ",", 
       RowBox[{"Darker", "[", "Red", "]"}]}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"i", "-", "1"}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"maxf", "-", "1"}], ")"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"start", "=", "5"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"fs", "=", 
   RowBox[{"Table", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"draw", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"konarr", "[", 
        RowBox[{"[", 
         RowBox[{"j", ",", 
          RowBox[{"{", 
           RowBox[{"i", "+", "start"}], "}"}]}], "]"}], "]"}], ",", 
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "}"}], ",", 
       RowBox[{"{", "}"}], ",", 
       RowBox[{"dirs", "[", 
        RowBox[{"[", 
         RowBox[{"-", "1"}], "]"}], "]"}], ",", "False", ",", 
       RowBox[{"25", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}]}], ",", "True", 
       ",", "16", ",", 
       RowBox[{"cf2", "[", "i", "]"}]}], "\[IndentingNewLine]", "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", "konarr", "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "maxf", ",", "40"}], "}"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"pics", "=", 
   RowBox[{"Show", "/@", "fs"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"grow", "=", 
   RowBox[{"GraphicsRow", "[", 
    RowBox[{
     RowBox[{"pics", "[", 
      RowBox[{"[", 
       RowBox[{"{", 
        RowBox[{"1", ",", "6", ",", "9", ",", "11"}], "}"}], "]"}], "]"}], 
     ",", 
     RowBox[{"Spacings", "\[Rule]", "20"}]}], "]"}]}], 
  ";"}], "\n", "grow"}], "Input",
 CellChangeTimes->{{3.689507526781604*^9, 3.68950753728224*^9}, {
   3.6895076059949417`*^9, 3.6895076061063833`*^9}, 3.689507888547427*^9, {
   3.689509249265059*^9, 3.689509255552005*^9}, {3.694894315942539*^9, 
   3.694894318925766*^9}, 3.69496736938485*^9, {3.70290857344318*^9, 
   3.702908584286982*^9}, {3.702918477150181*^9, 3.702918479269155*^9}, 
   3.704814367080491*^9, {3.704814692307581*^9, 3.704814692696868*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5A.eps\>\""}], ",", "grow"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.687531250143922*^9, 3.687531275730743*^9}, {
  3.688124588139453*^9, 3.688124598499362*^9}, {3.694894455778997*^9, 
  3.6948944614585667`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["B", "Subsection",
 CellChangeTimes->{{3.68149922441538*^9, 3.681499236235417*^9}}],

Cell[CellGroupData[{

Cell["Green", "Subsubsection",
 CellChangeTimes->{{3.6814996892082767`*^9, 3.681499705793477*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir1", "=", 
   RowBox[{"figS5dir", "<>", "\"\</dens_arr/sd1000000/nmyo\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]s", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"25", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "5"}], "}"}]}], ",", 
     RowBox[{"250", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "5"}], "}"}]}], ",", 
     RowBox[{"2500", 
      RowBox[{"Range", "[", "6", "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dir1", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"Round", "[", "#", "]"}], "]"}]}], "&"}], "/@", "\[Rho]s"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"acts1", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ptsCS", "[", 
      RowBox[{"dr", ",", "\"\<objects\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n"}], "Input",
 CellChangeTimes->{{3.689516245848775*^9, 3.6895162517964687`*^9}, {
   3.68955483308033*^9, 3.689554913996091*^9}, 3.689554993001536*^9, 
   3.689555069953165*^9, 3.6895558623678627`*^9, {3.694790709498667*^9, 
   3.694790714418105*^9}, {3.694894400892042*^9, 3.694894406275996*^9}, {
   3.702908658468729*^9, 3.702908667508315*^9}, {3.702908761154642*^9, 
   3.702908785929667*^9}, {3.702908838138907*^9, 3.702908839576836*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lks1", "=", 
   RowBox[{
    RowBox[{"acts1", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"acts1", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"cms1", "=", 
   RowBox[{"0.5", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"acts1", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "All", ",", 
         RowBox[{";;", 
          RowBox[{"-", "2"}]}]}], "]"}], "]"}], "+", 
      RowBox[{"acts1", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "All", ",", 
         RowBox[{"2", ";;"}]}], "]"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"drps1", "=", 
    RowBox[{
     RowBox[{"cms1", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
     RowBox[{"cms1", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{";;", 
         RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"drps1", "=", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"rijP", "[", 
         RowBox[{"#", ",", 
          RowBox[{"{", 
           RowBox[{"50", ",", "50"}], "}"}]}], "]"}], "&"}], ",", "drs", ",", 
       
       RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "*)"}]}], "\n", 
 RowBox[{
  RowBox[{"vs", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Norm", "[", "#", "]"}], "&"}], ",", "drps1", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"norms", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Norm", ",", 
     RowBox[{"lks1", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "2"}]}]}], "]"}], "]"}], ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"rspar", "=", 
   RowBox[{
    RowBox[{"lks1", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}], "/", "norms"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rsperp", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j", ",", "k", ",", "2"}], "]"}], "]"}]}], ",", 
        RowBox[{"rspar", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j", ",", "k", ",", "1"}], "]"}], "]"}]}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "rspar", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
   ";"}], "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vspar", "=", 
   RowBox[{"MapThread", "[", 
    RowBox[{"Dot", ",", 
     RowBox[{"{", 
      RowBox[{"drps1", ",", "rspar"}], "}"}], ",", "3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vsperp", "=", 
    RowBox[{"MapThread", "[", 
     RowBox[{"Dot", ",", 
      RowBox[{"{", 
       RowBox[{"drps1", ",", "rsperp"}], "}"}], ",", "3"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dt", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muvs", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vs"}]}], "/", "dt"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"par1", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vspar"}]}], "/", "dt"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"perp1", "=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"par1", "^", "2"}]}], "+", 
     RowBox[{"muvs", "^", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6895552839300547`*^9, 3.689555305843281*^9}, {
  3.689555882527713*^9, 3.689555895216404*^9}, {3.689556266520852*^9, 
  3.6895562766272383`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Red Curve", "Subsubsection",
 CellChangeTimes->{{3.681500080359632*^9, 3.681500099144232*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir2", "=", 
   RowBox[{"figS5dir", "<>", "\"\</len_arr/sd1000000/len\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nms", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"1", ",", "25", ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dir2", "<>", 
      RowBox[{"ToString", "[", "#", "]"}]}], "&"}], "/@", "nms"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"acts2", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ptsCS", "[", 
      RowBox[{"dr", ",", "\"\<objects\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n"}], "Input",
 CellChangeTimes->{{3.689556861551242*^9, 3.689556886894628*^9}, {
  3.694894420836541*^9, 3.6948944217076902`*^9}, {3.702908870991481*^9, 
  3.702908903317629*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"lks2", "=", 
   RowBox[{
    RowBox[{"acts2", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"acts2", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"cms2", "=", 
   RowBox[{"0.5", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"acts2", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "All", ",", 
         RowBox[{";;", 
          RowBox[{"-", "2"}]}]}], "]"}], "]"}], "+", 
      RowBox[{"acts2", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "All", ",", 
         RowBox[{"2", ";;"}]}], "]"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"drps2", "=", 
   RowBox[{
    RowBox[{"cms2", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"cms2", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"vs", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Norm", "[", "#", "]"}], "&"}], ",", "drps2", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"norms", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Norm", ",", 
     RowBox[{"lks2", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "2"}]}]}], "]"}], "]"}], ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"rspar", "=", 
   RowBox[{
    RowBox[{"lks2", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}], "/", "norms"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rsperp", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"rspar", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j", ",", "k", ",", "2"}], "]"}], "]"}]}], ",", 
       RowBox[{"rspar", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "j", ",", "k", ",", "1"}], "]"}], "]"}]}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "rspar", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"rspar", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"rspar", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"vspar", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"drps2", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}], ".", 
      RowBox[{"rspar", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "drps2", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"drps2", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"drps2", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vsperp", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"drps2", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}], ".", 
       RowBox[{"rsperp", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "drps2", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"drps2", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"drps2", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
   ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"dt", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muvs", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vs"}]}], "/", "dt"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"par2", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vspar"}]}], "/", "dt"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"perp2", "=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"par2", "^", "2"}]}], "+", 
     RowBox[{"muvs", "^", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{3.6895576554508677`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Blue Curve", "Subsubsection",
 CellChangeTimes->{{3.681500091319038*^9, 3.681500102463667*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dir", "=", 
   RowBox[{"figS5dir", "<>", "\"\</kon_arr/sd1000000/kon\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ks", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Flatten", "[", 
      RowBox[{"Outer", "[", 
       RowBox[{"Times", ",", 
        RowBox[{"{", 
         RowBox[{"0.001", ",", "0.01", ",", "0.1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "2", ",", "5"}], "}"}]}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"10", ",", "20"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"dir", "<>", 
      RowBox[{"ToString", "[", 
       RowBox[{"NumberForm", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "3"}], "}"}]}], "]"}], "]"}]}], "&"}], "/@", 
    "ks"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"acts3", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"ptsCS", "[", 
       RowBox[{"dr", ",", "\"\<objects\>\""}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"dr", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"lks3", "=", 
   RowBox[{
    RowBox[{"acts3", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"acts3", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", "All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"cms3", "=", 
   RowBox[{"0.5", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"acts3", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "All", ",", 
         RowBox[{";;", 
          RowBox[{"-", "2"}]}]}], "]"}], "]"}], "+", 
      RowBox[{"acts3", "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "All", ",", 
         RowBox[{"2", ";;"}]}], "]"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"drps3", "=", 
   RowBox[{
    RowBox[{"cms3", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"cms3", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"vs", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Norm", "[", "#", "]"}], "&"}], ",", "drps3", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"norms", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Norm", ",", 
     RowBox[{"lks3", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", 
        RowBox[{"1", ";;", 
         RowBox[{"-", "2"}]}]}], "]"}], "]"}], ",", 
     RowBox[{"{", "3", "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"rspar", "=", 
   RowBox[{
    RowBox[{"lks3", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"1", ";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}], "/", "norms"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rsperp", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j", ",", "k", ",", "2"}], "]"}], "]"}]}], ",", 
        RowBox[{"rspar", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j", ",", "k", ",", "1"}], "]"}], "]"}]}], "}"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "rspar", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"rspar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
   ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"vspar", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"drps3", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}], ".", 
      RowBox[{"rspar", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "drps3", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"drps3", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", 
        RowBox[{"drps3", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"vsperp", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"drps3", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}], ".", 
       RowBox[{"rsperp", "[", 
        RowBox[{"[", 
         RowBox[{"i", ",", "j", ",", "k"}], "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "drps3", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"drps3", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", 
         RowBox[{"drps3", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
   ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{"dt", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"muvs", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vs"}]}], "/", "dt"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"par3", "=", 
   RowBox[{
    RowBox[{"Mean", "/@", 
     RowBox[{"Flatten", "/@", "vspar"}]}], "/", "dt"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"perp3", "=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"par3", "^", "2"}]}], "+", 
     RowBox[{"muvs", "^", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{
  3.68955764021532*^9, 3.689557933252294*^9, {3.694790322080064*^9, 
   3.694790325143435*^9}, {3.694894432764503*^9, 3.694894433575286*^9}, {
   3.7029089781259747`*^9, 3.7029089813165627`*^9}, {3.70291442653226*^9, 
   3.702914426771502*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Whole Plot", "Subsubsection",
 CellChangeTimes->{{3.6815002234258423`*^9, 3.681500237705695*^9}, {
  3.7029195498567743`*^9, 3.702919551513562*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rd0", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]0", "=", "4"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"L0", "=", "15"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Lm", "=", "0.5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"koff0", "=", "1"}], ";"}]}], "Input",
 CellChangeTimes->{{3.675612558758986*^9, 3.675612572014722*^9}, {
   3.6756126893944807`*^9, 3.675612697112899*^9}, {3.675618744809306*^9, 
   3.675618754631495*^9}, {3.675618828599145*^9, 3.6756188299991293`*^9}, {
   3.675618875646193*^9, 3.6756188772686253`*^9}, {3.675618964052663*^9, 
   3.675618964314345*^9}, {3.675679649250039*^9, 3.675679652985635*^9}, {
   3.680368402222104*^9, 3.680368404077057*^9}, {3.680368434574329*^9, 
   3.680368435853072*^9}, {3.680368511038554*^9, 3.6803685174311953`*^9}, 
   3.681638770383143*^9, {3.681641035619134*^9, 3.681641050897395*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xvals1", "=", 
   RowBox[{
    RowBox[{"\[Rho]s", "/", "2500"}], " ", "Lm", " ", "L0", " ", "rd0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]0", " ", "Lm", 
      RowBox[{"(", 
       RowBox[{"nms", "-", "1"}], ")"}]}], ")"}], " ", "rd0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals3", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]0", " ", "Lm", " ", "L0"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"ks", "/", 
      RowBox[{"(", 
       RowBox[{"ks", "+", "koff0"}], ")"}]}], ")"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.669547394333199*^9, 3.669547395335994*^9}, {
   3.669547457910119*^9, 3.6695474619181547`*^9}, {3.669547504293866*^9, 
   3.669547577075108*^9}, {3.66954763002936*^9, 3.669547698552413*^9}, 
   3.669547794719522*^9, {3.669547851732976*^9, 3.669547865916278*^9}, {
   3.669548944417613*^9, 3.669548953800785*^9}, {3.675608241306411*^9, 
   3.675608251281824*^9}, {3.6756082899849358`*^9, 3.6756082947206793`*^9}, {
   3.6756084883561153`*^9, 3.675608497819375*^9}, {3.6756085869548483`*^9, 
   3.675608592088807*^9}, {3.675608721719131*^9, 3.675608723133561*^9}, {
   3.6756087736373672`*^9, 3.67560877929205*^9}, {3.675610506089459*^9, 
   3.675610511831223*^9}, {3.675610548184681*^9, 3.675610550966566*^9}, {
   3.675619149744605*^9, 3.6756192344597*^9}, {3.6756193047316227`*^9, 
   3.675619326497716*^9}, {3.675639750430951*^9, 3.675639804982925*^9}, {
   3.675639835251245*^9, 3.675639856722138*^9}, {3.675639908767729*^9, 
   3.675639920041058*^9}, {3.675639970987195*^9, 3.675639971997177*^9}, {
   3.675640041424684*^9, 3.675640042433385*^9}, {3.675679659211442*^9, 
   3.675679665609633*^9}, 3.675681237937204*^9, {3.67568136630999*^9, 
   3.675681366845221*^9}, 3.675681473683605*^9, {3.6803685073752193`*^9, 
   3.6803685078059797`*^9}, {3.681638741048915*^9, 3.68163877443894*^9}, {
   3.6895583837077312`*^9, 3.689558385203802*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tab", "=", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(||\)]\)\>\"", ",", 
       "\"\<\!\(\*SubscriptBox[\(v\), \(\[Perpendicular]\)]\)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Green", ",", 
           RowBox[{"Rectangle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"Directive", "[", 
             RowBox[{"Thick", ",", "Green"}], "]"}], "]"}], ",", "White", ",", 
           RowBox[{"Rectangle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<L\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Red", ",", 
           RowBox[{"Rotate", "[", 
            RowBox[{
             RowBox[{"Rectangle", "[", "]"}], ",", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{"Red", ",", "Thick"}], "}"}], "]"}], ",", "White", ",", 
           RowBox[{"Rotate", "[", 
            RowBox[{
             RowBox[{"Rectangle", "[", "]"}], ",", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(r\), \(D\)]\)\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Disk", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Circle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}]}], "}"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"leg", "=", 
   RowBox[{"Grid", "[", 
    RowBox[{"tab", ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"FontSize", "\[Rule]", "19"}], ",", 
        RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}]}], "}"}]}], ",", 
     RowBox[{"Frame", "\[Rule]", "All"}], ",", 
     RowBox[{"Spacings", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0.2", ",", "0.2"}], "}"}]}], ",", 
     RowBox[{"Background", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"LightGray", ",", "None"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"LightGray", ",", "None"}], "}"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"vplt1", "=", 
   RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals1", ",", 
          RowBox[{"-", "par1"}]}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals2", ",", 
          RowBox[{"-", "par2"}]}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals3", ",", 
           RowBox[{"-", "par3"}]}], "}"}], "\[Transpose]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "4"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals1", ",", "perp1"}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals2", ",", "perp2"}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals3", ",", "perp3"}], "}"}], "\[Transpose]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "4"}]}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "->", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Green", ",", 
             RowBox[{"Rectangle", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}],
         ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Red", ",", 
             RowBox[{"Rotate", "[", 
              RowBox[{
               RowBox[{"Rectangle", "[", "]"}], ",", 
               RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}]}], ",", "0.06"}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Blue", ",", 
             RowBox[{"Disk", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}], ",",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{"Green", ",", "Thick"}], "}"}], "]"}], ",", "White", 
             ",", 
             RowBox[{"Rectangle", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}],
         ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{"Red", ",", "Thick"}], "}"}], "]"}], ",", "White", ",", 
             RowBox[{"Rotate", "[", 
              RowBox[{
               RowBox[{"Rectangle", "[", "]"}], ",", 
               RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}]}], ",", "0.06"}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Blue", ",", 
             RowBox[{"Circle", "[", "]"}]}], "}"}]}], ",", "0.05"}], "}"}]}], 
       "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[EmptySquare]", ",", "15"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[EmptySquare]", ",", "15"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"\[EmptySquare]", ",", "15"}], "}"}]}], "}"}]}], ","}], 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "Green", ",", "Red", ",", "Blue", ",", "Green", ",", "Red", ",", 
        "Blue"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "27"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1.1"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\!\(\*SubscriptBox[\(l\), \
\(m\)]\)\!\(\*SubscriptBox[\(Lr\), \(D\)]\)\>\"", ",", 
        "\"\<v(\[Mu]m/s)\>\""}], "}"}]}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.707584244682942*^9, 3.707584248169607*^9}, {
  3.707584285593655*^9, 3.707584307752326*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.70758431377153*^9, 3.7075843141677217`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sz", "=", "10"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"tab", "=", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(||\)]\)\>\"", ",", 
       "\"\<\!\(\*SubscriptBox[\(v\), \(\[Perpendicular]\)]\)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Green", ",", 
           RowBox[{"Disk", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"Directive", "[", 
             RowBox[{"Thick", ",", "Green"}], "]"}], "]"}], ",", "White", ",", 
           RowBox[{"Rectangle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<L\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Red", ",", 
           RowBox[{"Disk", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "0"}], "}"}], ",", "0.1"}], "]"}]}], "}"}], 
         ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"Directive", "[", 
             RowBox[{"Thick", ",", "Red"}], "]"}], "]"}], ",", "White", ",", 
           RowBox[{"Rectangle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(r\), \(D\)]\)\>\"", ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", 
           RowBox[{"Disk", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}], ",", 
       RowBox[{"Graphics", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"Directive", "[", 
             RowBox[{"Thick", ",", "Blue"}], "]"}], "]"}], ",", "White", ",", 
           
           RowBox[{"Rectangle", "[", "]"}]}], "}"}], ",", 
         RowBox[{"ImageSize", "\[Rule]", "sz"}]}], "]"}]}], "}"}]}], "}"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"leg", "=", 
   RowBox[{"Grid", "[", 
    RowBox[{"tab", ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"FontSize", "\[Rule]", "19"}], ",", 
        RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}]}], "}"}]}], ",", 
     RowBox[{"Frame", "\[Rule]", "All"}], ",", 
     RowBox[{"Spacings", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"0.2", ",", "0.2"}], "}"}]}], ",", 
     RowBox[{"Background", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"LightGray", ",", "None"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"LightGray", ",", "None"}], "}"}]}], "}"}]}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.669550669236497*^9, 3.669550718308292*^9}, {
   3.6702481101005774`*^9, 3.670248151224307*^9}, 3.6702482898038197`*^9, {
   3.6702483235832443`*^9, 3.670248324118381*^9}, {3.6702483661021557`*^9, 
   3.670248372812759*^9}, {3.670248539305471*^9, 3.670248606608013*^9}, 
   3.670249477834488*^9, {3.6702505794042053`*^9, 3.670250647145918*^9}, 
   3.670250762990954*^9, {3.670250812150035*^9, 3.670250813685503*^9}, {
   3.675639990359192*^9, 3.675640004480277*^9}, {3.675679900957788*^9, 
   3.67567992323146*^9}, {3.675679959733045*^9, 3.675679970146429*^9}, 
   3.680368922322282*^9, {3.6803689585564137`*^9, 3.6803689589900227`*^9}, {
   3.70290903680494*^9, 3.7029090382121143`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vplt1", "=", 
   RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals1", ",", 
          RowBox[{"-", "par1"}]}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals2", ",", 
          RowBox[{"-", "par2"}]}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals3", ",", 
           RowBox[{"-", "par3"}]}], "}"}], "\[Transpose]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "4"}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals1", ",", "perp1"}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xvals2", ",", "perp2"}], "}"}], "\[Transpose]"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals3", ",", "perp3"}], "}"}], "\[Transpose]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ";;", 
          RowBox[{"-", "4"}]}], "]"}], "]"}]}], "}"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[FilledCircle]", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[EmptySquare]", ",", "15"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[EmptySquare]", ",", "15"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[EmptySquare]", ",", "15"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "Green", ",", "Red", ",", "Blue", ",", "Green", ",", "Red", ",", 
        "Blue"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "27"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1.1"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\!\(\*SubscriptBox[\(l\), \
\(m\)]\)\!\(\*SubscriptBox[\(Lr\), \(D\)]\)\>\"", ",", 
        "\"\<v(\[Mu]m/s)\>\""}], "}"}]}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6695478771184464`*^9, 3.6695479204055977`*^9}, {
   3.6695480448400707`*^9, 3.669548139164968*^9}, 3.6695482177952433`*^9, {
   3.669548249618132*^9, 3.669548354679434*^9}, {3.669548409894514*^9, 
   3.669548517995739*^9}, {3.669548556476121*^9, 3.669548618031929*^9}, {
   3.669548656393095*^9, 3.6695486830398912`*^9}, 3.669548742439077*^9, {
   3.669548778229817*^9, 3.66954885996277*^9}, 3.669549137355266*^9, {
   3.6695496338723803`*^9, 3.669549704423223*^9}, {3.66954974119702*^9, 
   3.66954996570363*^9}, {3.669550130340262*^9, 3.669550182634329*^9}, {
   3.669550340751532*^9, 3.6695503547655087`*^9}, {3.669550515138198*^9, 
   3.669550546089595*^9}, {3.669550629135091*^9, 3.669550629791273*^9}, {
   3.669550732364532*^9, 3.669550748219718*^9}, {3.669550852114626*^9, 
   3.669550857568844*^9}, {3.670248069030323*^9, 3.670248127252166*^9}, {
   3.6702481591879168`*^9, 3.670248160954851*^9}, {3.670248204851452*^9, 
   3.670248278273314*^9}, {3.6702486551276073`*^9, 3.670248675181221*^9}, 
   3.6702487053681507`*^9, {3.6702522793993053`*^9, 3.670252281373768*^9}, {
   3.670253545387753*^9, 3.670253546298993*^9}, {3.675608621530798*^9, 
   3.6756086411119204`*^9}, {3.675608797705686*^9, 3.675608798122505*^9}, {
   3.6756105655618*^9, 3.675610590582549*^9}, {3.675612778642228*^9, 
   3.67561277988785*^9}, {3.6756128187147913`*^9, 3.675612819660055*^9}, 
   3.675612861409473*^9, {3.675619034971669*^9, 3.675619044489603*^9}, {
   3.675619074643805*^9, 3.675619103271556*^9}, {3.675619266493928*^9, 
   3.67561928804602*^9}, {3.675619346275859*^9, 3.675619346986946*^9}, {
   3.6756795150122213`*^9, 3.675679619268136*^9}, {3.675679683532072*^9, 
   3.6756796892965717`*^9}, {3.680368570712068*^9, 3.680368591815707*^9}, {
   3.68036884369182*^9, 3.680368846323228*^9}, {3.6803793355202503`*^9, 
   3.680379341896099*^9}, {3.680452821586677*^9, 3.6804528224587793`*^9}, {
   3.681500298299859*^9, 3.681500300899229*^9}, {3.688125127631274*^9, 
   3.688125145965948*^9}, {3.688125236940612*^9, 3.688125257778968*^9}, 
   3.6881271181996737`*^9, {3.688127181702064*^9, 3.688127182140829*^9}, {
   3.688127663257448*^9, 3.688127730374956*^9}, {3.688128501845261*^9, 
   3.6881285036353292`*^9}, 3.689558088890608*^9, {3.689558181890617*^9, 
   3.689558231203001*^9}, {3.689558308904142*^9, 3.689558316699326*^9}, 
   3.689558410105941*^9, {3.689558449869165*^9, 3.689558450348634*^9}, {
   3.689558489798113*^9, 3.689558489957212*^9}, {3.702909061659985*^9, 
   3.702909096522335*^9}, {3.7029196114499702`*^9, 3.7029196115842743`*^9}, {
   3.702919662707583*^9, 3.70291966314327*^9}, 3.707584155789554*^9}],

Cell[BoxData[
 RowBox[{"vplt", "=", 
  RowBox[{"Overlay", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"vplt1", ",", "leg"}], "}"}], ",", 
    RowBox[{"Alignment", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0.1", ",", 
       RowBox[{"-", "0.25"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.670250770748762*^9, 3.67025084882266*^9}, {
   3.6702512080113153`*^9, 3.670251208180633*^9}, 3.6702512839694767`*^9, {
   3.670251402266375*^9, 3.6702514153004093`*^9}, {3.675679766135846*^9, 
   3.675679767104607*^9}, {3.675679820910652*^9, 3.6756798668974237`*^9}, {
   3.675680023785091*^9, 3.675680067033328*^9}, {3.680368632409597*^9, 
   3.680368649022732*^9}, {3.680368879915257*^9, 3.6803689516579447`*^9}, {
   3.68036898207446*^9, 3.680368990882536*^9}, {3.68752838021579*^9, 
   3.68752839217188*^9}, 3.68812521777288*^9, {3.6881277389434023`*^9, 
   3.688127758621648*^9}, {3.689558457581278*^9, 3.689558476605596*^9}, 
   3.707584183071693*^9}],

Cell[BoxData["vplt1"], "Input",
 CellChangeTimes->{{3.707584839559132*^9, 3.70758484040149*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5B_no_leg.eps\>\""}], ",", "vplt1"}], 
   "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.687531320957047*^9, 3.687531356680743*^9}, {
  3.6881277837009983`*^9, 3.688127789396841*^9}, {3.694894508233757*^9, 
  3.694894516049189*^9}, {3.707584833218727*^9, 3.707584837306198*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["C (Continued from B)", "Subsection",
 CellChangeTimes->{{3.6814992413611526`*^9, 3.681499246798682*^9}, {
  3.6815000286587553`*^9, 3.681500032385903*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ms1", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"msdc", "[", 
      RowBox[{"drps1", "[", 
       RowBox[{"[", 
        RowBox[{"j", ",", "All", ",", "i"}], "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", "drps1", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "L0"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ms2", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"msdc", "[", 
      RowBox[{"drps2", "[", 
       RowBox[{"[", 
        RowBox[{"j", ",", "All", ",", "i"}], "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", "drps2", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"nms", "[", 
        RowBox[{"[", "j", "]"}], "]"}]}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ms3", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"msdc", "[", 
      RowBox[{"drps3", "[", 
       RowBox[{"[", 
        RowBox[{"j", ",", "All", ",", "i"}], "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", 
       RowBox[{"Length", "[", "drps3", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "L0"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6695566422605658`*^9, 3.669556654835635*^9}, 
   3.669557017014677*^9, 3.675619539026721*^9, {3.6756195813634367`*^9, 
   3.67561958250961*^9}, {3.6803700454276457`*^9, 3.680370047195122*^9}, {
   3.681500012964095*^9, 3.681500014011656*^9}, {3.6816388579447803`*^9, 
   3.681638860527729*^9}, {3.6816389021198807`*^9, 3.681638914800406*^9}, {
   3.690282182502824*^9, 3.690282194773806*^9}, 3.690282603876214*^9, {
   3.694894571029087*^9, 3.6948945744877996`*^9}, {3.7029099760923*^9, 
   3.702909981723014*^9}, 3.702914226023719*^9, {3.7048132097025747`*^9, 
   3.7048132123636007`*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{3.702914225247395*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figS5dir", "<>", "\"\</msds_dens.dat\>\""}], ",", "ms1"}], "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figS5dir", "<>", "\"\</msds_len.dat\>\""}], ",", "ms2"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figS5dir", "<>", "\"\</msds_kon.dat\>\""}], ",", "ms3"}], "]"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.702910174443162*^9, 3.702910260890098*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"ms1", ",", "ms2", ",", "ms3"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.704813361743883*^9, 3.70481336693606*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ms1", "=", 
   RowBox[{"importCheck", "[", 
    RowBox[{"figS5dir", "<>", "\"\</msds_dens.dat\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ms2", "=", 
   RowBox[{"importCheck", "[", 
    RowBox[{"figS5dir", "<>", "\"\</msds_len.dat\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ms3", "=", 
   RowBox[{"importCheck", "[", 
    RowBox[{"figS5dir", "<>", "\"\</msds_kon.dat\>\""}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.704813217256144*^9, 3.704813255308812*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
   "Sort", " ", "MSD", " ", "data", " ", "sets", " ", "by", " ", "their", " ",
     "xval"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"msddats1", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xvals1", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"Range", "[", 
              RowBox[{"Length", "[", 
               RowBox[{"ms1", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}], "]"}], "*", 
             "dt"}], ",", 
            RowBox[{"Mean", "[", 
             RowBox[{"ms1", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "ms1", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"msddats2", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xvals2", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"Range", "[", 
              RowBox[{"Length", "[", 
               RowBox[{"ms2", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}], "]"}], "*", 
             "dt"}], ",", 
            RowBox[{"Mean", "[", 
             RowBox[{"ms2", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "ms2", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"msddats3", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"xvals3", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"Range", "[", 
              RowBox[{"Length", "[", 
               RowBox[{"ms3", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "1"}], "]"}], "]"}], "]"}], "]"}], "*", 
             "dt"}], ",", 
            RowBox[{"Mean", "[", 
             RowBox[{"ms3", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], 
          "\[Transpose]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{
          RowBox[{"Length", "[", "ms3", "]"}], "-", "3"}]}], "}"}]}], "]"}]}],
     ";"}], "\n", 
   RowBox[{
    RowBox[{"allmsddats", "=", 
     RowBox[{"Sort", "[", 
      RowBox[{"Join", "[", 
       RowBox[{"msddats1", ",", "msddats2", ",", "msddats3"}], "]"}], "]"}]}],
     ";"}], "\n", 
   RowBox[{
    RowBox[{"allxvals", "=", 
     RowBox[{"allmsddats", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "1"}], "]"}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.6695592830546913`*^9, 3.66955938004792*^9}, {
   3.669559747788515*^9, 3.669559780644438*^9}, {3.670252094595261*^9, 
   3.67025212160218*^9}, 3.6756809210013113`*^9, {3.680375396747889*^9, 
   3.6803754921939783`*^9}, {3.6804485571215143`*^9, 3.680448560960495*^9}, {
   3.681640021537374*^9, 3.6816400511485643`*^9}, 3.68164008378819*^9, {
   3.681640123046792*^9, 3.681640143387857*^9}, {3.681641060923131*^9, 
   3.681641067155027*^9}, {3.690282992468348*^9, 3.690283000498102*^9}, {
   3.702914277704496*^9, 3.702914278166627*^9}, 3.7029148161059437`*^9, {
   3.704813303043613*^9, 3.70481334332507*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xidxs", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"2", ",", 
     RowBox[{
      RowBox[{"Length", "[", "allxvals", "]"}], "-", "4"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tidxs", "=", 
   RowBox[{"Union", "[", 
    RowBox[{"Round", "[", 
     RowBox[{"10", "^", 
      RowBox[{"Range", "[", 
       RowBox[{"0", ",", 
        RowBox[{"Log10", "[", "200", "]"}], ",", "0.15"}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"logxvals", "=", 
   RowBox[{"Log", "[", 
    RowBox[{"allxvals", "[", 
     RowBox[{"[", "xidxs", "]"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cf", "=", 
   RowBox[{"ColorData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<BlueGreenYellow\>\"", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Min", "[", "logxvals", "]"}], ",", 
        RowBox[{"Max", "[", "logxvals", "]"}]}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"msdplt", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"ListLogLogPlot", "[", 
     RowBox[{
      RowBox[{"allmsddats", "[", 
       RowBox[{"[", 
        RowBox[{"xidxs", ",", "2", ",", "tidxs"}], "]"}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", "\[IndentingNewLine]", 
      RowBox[{"PlotStyle", "\[Rule]", "\"\<BlueGreenYellow\>\""}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"PlotStyle", "\[Rule]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"ColorData", "[", 
              RowBox[{"{", 
               RowBox[{"\"\<BlueGreenYellow\>\"", ",", 
                RowBox[{"Log", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"allxvals", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                   RowBox[{"allxvals", "[", 
                    RowBox[{"[", "maxidx", "]"}], "]"}]}], "}"}], "]"}]}], 
               "}"}], "]"}], "[", "#", "]"}], "&"}], "/@", 
           RowBox[{"Log", "[", 
            RowBox[{"allxvals", "[", 
             RowBox[{"[", 
              RowBox[{"1", ";;", "maxidx", ";;", "didx"}], "]"}], "]"}], 
            "]"}]}], ")"}]}], ","}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"FrameLabel", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<\[Delta]t (s)\>\"", ",", 
         RowBox[{"Style", "[", 
          RowBox[{
          "\"\<\[LeftAngleBracket]|r(t+\[Delta]t)-r(t)\!\(\*SuperscriptBox[\(|\
\), \(2\)]\)\[RightAngleBracket] (\!\(\*SuperscriptBox[\(\[Mu]m\), \
\(2\)]\))\>\"", ",", 
           RowBox[{"FontSize", "\[Rule]", "20"}]}], "]"}]}], "}"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"PlotMarkers", "\[Rule]", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\[FilledCircle]", ",", "15"}], "}"}], ",", "6"}], "]"}]}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"Joined", "\[Rule]", "False"}]}], "\[IndentingNewLine]", "]"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"Plot", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{
         RowBox[{"2", "x"}], "+", "0.5"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"-", "1"}], ",", "200"}], "}"}], ",", 
      RowBox[{"PlotStyle", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Blue", ",", "Thick", ",", 
           RowBox[{"Dashing", "[", "Large", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Orange", ",", "Thick", ",", 
           RowBox[{"Dashing", "[", "Large", "]"}]}], "}"}]}], "}"}]}]}], 
     "]"}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.669559791479904*^9, 3.669559809914703*^9}, {
   3.66955987425764*^9, 3.669559937992004*^9}, {3.66955999335984*^9, 
   3.66956002067796*^9}, {3.6695600555183077`*^9, 3.669560072028809*^9}, {
   3.669560186379463*^9, 3.669560193657913*^9}, {3.669560287464293*^9, 
   3.669560743277512*^9}, {3.669560777591743*^9, 3.669560790388888*^9}, {
   3.6695667581365213`*^9, 3.6695669311940536`*^9}, {3.669566966907544*^9, 
   3.6695669689146*^9}, {3.670252181307631*^9, 3.670252227776977*^9}, {
   3.670252344973756*^9, 3.670252412946159*^9}, 3.670252575976788*^9, {
   3.670252726010165*^9, 3.670252766156967*^9}, 3.670252796423959*^9, {
   3.670252851228488*^9, 3.67025286546266*^9}, 3.670252900118739*^9, 
   3.6702532205690327`*^9, {3.675681046927827*^9, 3.675681049174952*^9}, {
   3.675681293538048*^9, 3.675681295554626*^9}, {3.680373237458084*^9, 
   3.6803732548916407`*^9}, {3.680375716710082*^9, 3.68037573479631*^9}, {
   3.680375799477467*^9, 3.680375959725904*^9}, {3.680376067952094*^9, 
   3.680376082576666*^9}, {3.680376297371682*^9, 3.6803764443712387`*^9}, {
   3.680376652395946*^9, 3.6803767420662403`*^9}, {3.6804485830600986`*^9, 
   3.6804487852280607`*^9}, {3.6804488226675863`*^9, 3.680448886881852*^9}, {
   3.680448937986287*^9, 3.680448987521428*^9}, {3.680449019280162*^9, 
   3.680449079645751*^9}, {3.6816400693473587`*^9, 3.681640072203498*^9}, {
   3.687531089946885*^9, 3.687531095720957*^9}, {3.687531152056962*^9, 
   3.687531155551291*^9}, {3.687531207463706*^9, 3.687531230142337*^9}, {
   3.688128418888012*^9, 3.6881284314295053`*^9}, {3.688131271082498*^9, 
   3.688131301124642*^9}, {3.688131346284482*^9, 3.6881313511796227`*^9}, 
   3.688131395032092*^9, {3.68813147989225*^9, 3.688131589884988*^9}, {
   3.690283026123262*^9, 3.690283117256633*^9}, {3.690283218622579*^9, 
   3.690283218972711*^9}, {3.702912310834754*^9, 3.702912322996676*^9}, 
   3.7029136736292343`*^9, {3.70291370959838*^9, 3.702913712461323*^9}, {
   3.702913755453979*^9, 3.702913755760394*^9}, {3.702914303696886*^9, 
   3.7029143094865723`*^9}, {3.704814074203931*^9, 3.704814078722127*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5C_0.03-20.eps\>\""}], ",", "msdplt"}], 
   "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.687531381519882*^9, 3.687531389863661*^9}, {
  3.688131657754567*^9, 3.688131672897607*^9}, {3.690283241727618*^9, 
  3.690283241882914*^9}, {3.694894718044407*^9, 3.69489472826772*^9}, {
  3.7029147785216503`*^9, 3.702914787577024*^9}}],

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{"logbarleg", "[", 
   RowBox[{"0.03", ",", "20", ",", "4", ",", "\"\<BlueGreenYellow\>\"", ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"LegendLabel", "\[Rule]", 
       RowBox[{"Placed", "[", 
        RowBox[{
        "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\!\(\*SubscriptBox[\(l\), \
\(m\)]\)\!\(\*SubscriptBox[\(Lr\), \(D\)]\)\>\"", ",", "Bottom"}], "]"}]}], 
      ",", 
      RowBox[{"LegendMarkerSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"40", ",", "250"}], "}"}]}], ",", 
      RowBox[{"LabelStyle", "\[Rule]", 
       RowBox[{"Directive", "[", 
        RowBox[{"Black", ",", 
         RowBox[{"FontSize", "\[Rule]", "30"}]}], "]"}]}], ",", 
      RowBox[{"TicksStyle", "\[Rule]", "Black"}]}], "}"}]}], "]"}]}]], "Input",\

 CellChangeTimes->{{3.702854569948374*^9, 3.702854597405547*^9}, {
   3.702854689118669*^9, 3.7028547072863417`*^9}, {3.70285473751163*^9, 
   3.702854737623871*^9}, {3.7028548052407084`*^9, 3.7028549981434717`*^9}, {
   3.702855075035797*^9, 3.7028550757790327`*^9}, {3.702855118947908*^9, 
   3.702855127283193*^9}, {3.702855169315529*^9, 3.702855171888589*^9}, 
   3.7029199487998333`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5C_leg.pdf\>\""}], ",", "x"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.70285500185756*^9, 3.702855016081697*^9}, {
  3.7029199579516373`*^9, 3.7029199583432407`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["D", "Subsection",
 CellChangeTimes->{{3.6948947432995443`*^9, 3.694894751953668*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ddir", "=", 
   RowBox[{"figS5dir", "<>", "\"\</dens_arr/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mdenss", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"25", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "5"}], "}"}]}], ",", 
     RowBox[{"250", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "5"}], "}"}]}], ",", 
     RowBox[{"2500", 
      RowBox[{"Range", "[", "6", "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ddirs", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"ddir", "<>", "\"\<sd\>\"", "<>", 
       RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\<000000/nmyo\>\"", "<>", 
       RowBox[{"ToString", "[", "md", "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"md", ",", "mdenss"}], "}"}]}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"ldir", "=", 
   RowBox[{"figS5dir", "<>", "\"\</len_arr/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ls", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"1", ",", "25", ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ldirs", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"ldir", "<>", "\"\<sd\>\"", "<>", 
       RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\<000000/len\>\"", "<>", 
       
       RowBox[{"ToString", "[", "l", "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"l", ",", "ls"}], "}"}]}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"kdir", "=", 
   RowBox[{"figS5dir", "<>", "\"\</kon_arr/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ks", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"Outer", "[", 
       RowBox[{"Times", ",", 
        RowBox[{"10", "^", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "3"}], ",", 
           RowBox[{"-", "2"}], ",", 
           RowBox[{"-", "1"}]}], "}"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"1.", ",", "2.", ",", "5."}], "}"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kdirs", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"kdir", "<>", "\"\<sd\>\"", "<>", 
       RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\<000000/kon\>\"", "<>", 
       
       RowBox[{"ToString", "[", 
        RowBox[{"NumberForm", "[", 
         RowBox[{"k", ",", 
          RowBox[{"{", 
           RowBox[{"2", ",", "3"}], "}"}]}], "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", "ks"}], "}"}]}], "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dirs", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"ddirs", ",", "ldirs", ",", "kdirs"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"maxX", "=", "51"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6930103193253202`*^9, 3.693010384685337*^9}, {
   3.6930146981175203`*^9, 3.693014699058861*^9}, 3.69301477121972*^9, {
   3.6930589597508173`*^9, 3.6930590275928917`*^9}, {3.6930678061765633`*^9, 
   3.693067811382667*^9}, {3.694806253292941*^9, 3.694806253621492*^9}, {
   3.6948062863942413`*^9, 3.694806299797227*^9}, {3.6948063324455023`*^9, 
   3.69480633496634*^9}, {3.694807993011964*^9, 3.69480799403473*^9}, {
   3.694808024770486*^9, 3.694808045123011*^9}, 3.694808103628652*^9, 
   3.694894797414928*^9, {3.6949683704213877`*^9, 3.694968373008115*^9}, {
   3.6949684440778837`*^9, 3.694968462057651*^9}, {3.702914889375848*^9, 
   3.702914963501318*^9}, {3.7029151106820087`*^9, 3.7029151116335497`*^9}, {
   3.702915143538015*^9, 3.702915145392947*^9}, 3.702915404187134*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"acts", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"ptsCS", "[", 
      RowBox[{"dir", ",", "\"\<objects\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"dir", ",", "dirs"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"cms", "=", 
   RowBox[{"Map", "[", 
    RowBox[{"Mean", ",", 
     RowBox[{"acts", "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "All", ",", "All", ",", 
        RowBox[{"1", ";;", "2"}]}], "]"}], "]"}], ",", 
     RowBox[{"{", "2", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cms", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"MovingAverage", "[", 
      RowBox[{"#", ",", "20"}], "]"}], "&"}], "/@", "cms"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lkext", "=", 
   RowBox[{
    RowBox[{"cms", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{"2", ";;"}]}], "]"}], "]"}], "-", 
    RowBox[{"cms", "[", 
     RowBox[{"[", 
      RowBox[{"All", ",", 
       RowBox[{";;", 
        RowBox[{"-", "2"}]}]}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lps", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"plThsq", "[", 
      RowBox[{
       RowBox[{"lkext", "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", "maxX"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "lkext", "]"}]}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{
  3.694894803208844*^9, {3.694968343385366*^9, 3.694968364192203*^9}, {
   3.704814124397855*^9, 3.704814127745659*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xvals1", "=", 
   RowBox[{
    RowBox[{"mdenss", "/", "2500"}], " ", "Lm", " ", "L0", " ", "rd0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]0", " ", "Lm", " ", "ls"}], ")"}], " ", "rd0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xvals3", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[Rho]0", " ", "Lm", " ", "L0"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"ks", "/", 
      RowBox[{"(", 
       RowBox[{"ks", "+", "koff0"}], ")"}]}], ")"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"bnd1", "=", 
   RowBox[{"(", 
    RowBox[{"5", 
     RowBox[{"Length", "[", "mdenss", "]"}]}], ")"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bnd2", "=", 
   RowBox[{"bnd1", "+", 
    RowBox[{"5", 
     RowBox[{"Length", "[", "ls", "]"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lps1", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"Partition", "[", 
     RowBox[{
      RowBox[{"lps", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", "bnd1"}], "]"}], "]"}], ",", 
      RowBox[{"Length", "[", "mdenss", "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lps2", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"Partition", "[", 
     RowBox[{
      RowBox[{"lps", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"bnd1", "+", "1"}], ")"}], ";;", "bnd2"}], "]"}], "]"}], 
      ",", 
      RowBox[{"Length", "[", "ls", "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"lps3", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"Partition", "[", 
     RowBox[{
      RowBox[{"lps", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"bnd2", "+", "1"}], ")"}], ";;"}], "]"}], "]"}], ",", 
      RowBox[{"Length", "[", "ks", "]"}]}], "]"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.675612558758986*^9, 3.675612572014722*^9}, {
   3.6756126893944807`*^9, 3.675612697112899*^9}, {3.675618744809306*^9, 
   3.675618754631495*^9}, {3.675618828599145*^9, 3.6756188299991293`*^9}, {
   3.675618875646193*^9, 3.6756188772686253`*^9}, {3.675618964052663*^9, 
   3.675618964314345*^9}, {3.675679649250039*^9, 3.675679652985635*^9}, {
   3.680368402222104*^9, 3.680368404077057*^9}, {3.680368434574329*^9, 
   3.680368435853072*^9}, {3.680368511038554*^9, 3.6803685174311953`*^9}, 
   3.681638770383143*^9, {3.681641035619134*^9, 3.681641050897395*^9}, 
   3.694894928231501*^9, 3.694968543306313*^9, {3.702915360211594*^9, 
   3.7029153611384363`*^9}, 3.704815285022537*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xtix", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", 
     RowBox[{"-", "3"}], ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ytix", "=", 
   RowBox[{"LogTicks", "[", 
    RowBox[{"10", ",", 
     RowBox[{"-", "3"}], ",", "2"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"pr", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.5"}], ",", "1.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.6"}], ",", "1.4"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dplt", "=", 
   RowBox[{"ListPlot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Log10", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals1", ",", "lps1"}], "}"}], "\[Transpose]"}], ",", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"xvals2", ",", "lps2"}], "}"}], "\[Transpose]"}], ",", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"xvals3", ",", "lps3"}], "}"}], "\[Transpose]"}], ")"}], 
         "[", 
         RowBox[{"[", 
          RowBox[{"1", ";;", 
           RowBox[{"-", "2"}]}], "]"}], "]"}]}], "\[IndentingNewLine]", "}"}],
       "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Joined", "\[Rule]", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"ConstantArray", "[", 
         RowBox[{"True", ",", "3"}], "]"}], ",", 
        RowBox[{"ConstantArray", "[", 
         RowBox[{"True", ",", "6"}], "]"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"PlotMarkers", "\[Rule]", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"ConstantArray", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\[FilledCircle]", ",", "15"}], "}"}], ",", "3"}], "]"}],
           ",", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"\"\<\[CenterDot]\>\"", ",", "6"}], "]"}]}], "]"}]}], 
       ","}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"PlotMarkers", "->", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Green", ",", 
             RowBox[{"Rectangle", "[", "]"}]}], "}"}]}], ",", "0.06"}], "}"}],
         ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Red", ",", 
             RowBox[{"Rotate", "[", 
              RowBox[{
               RowBox[{"Rectangle", "[", "]"}], ",", 
               RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}]}], ",", "0.07"}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Graphics", "@", 
           RowBox[{"{", 
            RowBox[{"Blue", ",", 
             RowBox[{"Disk", "[", "]"}]}], "}"}]}], ",", "0.06"}], "}"}]}], 
       "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Green", ",", "Red", ",", "Blue"}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<\!\(\*SubscriptBox[\(\[Rho]\), \(m\)]\)\!\(\*SubscriptBox[\(l\), \
\(m\)]\)\!\(\*SubscriptBox[\(Lr\), \(D\)]\)\>\"", ",", 
        "\"\<Path Persistence\\nLength (\[Mu]m)\>\""}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Filling", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"4", "\[Rule]", 
         RowBox[{"{", "5", "}"}]}], ",", 
        RowBox[{"6", "\[Rule]", 
         RowBox[{"{", "7", "}"}]}], ",", 
        RowBox[{"8", "\[Rule]", 
         RowBox[{"{", "9", "}"}]}]}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameTicks", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"ytix", ",", "Automatic"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"xtix", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"FillingStyle", "\[Rule]", 
      RowBox[{"Directive", "[", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.25", "]"}], ",", 
        RowBox[{"Thickness", "[", "0.01", "]"}]}], "]"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Axes", "\[Rule]", "None"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6933224599533167`*^9, 3.6933224826614323`*^9}, {
   3.693322562846937*^9, 3.693322564282528*^9}, {3.693322718294518*^9, 
   3.693322722567976*^9}, 3.694894936940605*^9, {3.7029179633958273`*^9, 
   3.7029179922993507`*^9}, {3.707584642344914*^9, 3.7075846469789*^9}}],

Cell[CellGroupData[{

Cell["theory: duke holy 1995", "Subsubsection",
 CellChangeTimes->{{3.6948292762794847`*^9, 3.6948292865699053`*^9}, {
  3.6948293686956863`*^9, 3.694829369711838*^9}, {3.694968593778799*^9, 
  3.694968595291177*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Lp", "=", "17"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kbT", "=", "0.004"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"visc", "=", "0.001"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"maxBindDist", "=", "0.063"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ka", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v0", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Gamma]", "=", 
   RowBox[{"kbT", "/", "visc"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ls", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"2", ",", "25"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Sigma]0", "=", "\[Rho]0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Sigma]x", "=", 
   RowBox[{
    RowBox[{"maxBindDist", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "5"}], "/", "3"}], ")"}]}], 
    RowBox[{"Lp", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "/", "3"}], ")"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Sigma]xx", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "/", "v0"}], " ", ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "5"}], "/", "6"}], ")"}]}], 
    RowBox[{"Lp", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "/", "3"}], ")"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"davg0", "=", 
    RowBox[{"\[Sigma]0", " ", 
     RowBox[{"Lp", "^", 
      RowBox[{"(", 
       RowBox[{"1", "/", "5"}], ")"}]}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ClearAll", "[", 
   RowBox[{"davg", ",", "savg", ",", "pcur", ",", "dth", ",", "persist"}], 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"davg", "[", 
    RowBox[{"\[Sigma]_", ",", 
     RowBox[{"\[Sigma]p_:", "\[Sigma]x"}], ",", " ", 
     RowBox[{"\[Sigma]pp_:", "\[Sigma]xx"}]}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"\[Sigma]", "<", "\[Sigma]pp"}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"\[Sigma]", " ", 
        RowBox[{"Sqrt", "[", 
         RowBox[{"\[Gamma]", "/", "v0"}], "]"}]}], ")"}]}], ",", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"\[Sigma]", "<", "\[Sigma]p"}], ",", 
       RowBox[{
        RowBox[{"\[Sigma]", "^", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "2."}], "/", "5"}], ")"}]}], 
        RowBox[{"Lp", "^", 
         RowBox[{"(", 
          RowBox[{"1", "/", "5"}], ")"}]}]}], ",", 
       RowBox[{"1", "/", 
        RowBox[{"(", 
         RowBox[{"\[Sigma]", " ", "maxBindDist"}], ")"}]}]}], "]"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"savg", "[", 
    RowBox[{"l_", ",", "d_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"l", "+", 
       RowBox[{"2", "d"}]}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"l", "+", 
       RowBox[{"3", "d"}]}], ")"}]}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"d", "^", "2"}], "/", "l"}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Exp", "[", 
       RowBox[{"l", "/", "d"}], "]"}], "-", "1", "-", 
      RowBox[{"l", "/", "d"}]}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"pcur", "[", 
    RowBox[{"\[Sigma]_", ",", 
     RowBox[{"\[Sigma]pp_:", "\[Sigma]xx"}]}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"\[Sigma]", ">", "\[Sigma]pp"}], ",", "Lp", ",", 
     RowBox[{"Lp", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"\[Sigma]", "/", "\[Sigma]pp"}], ")"}], "^", "2"}]}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"dth", "[", 
    RowBox[{"\[Rho]_", ",", "L_"}], "]"}], ":=", 
   RowBox[{"3", "/", 
    RowBox[{"(", 
     RowBox[{"\[Rho]", " ", 
      RowBox[{"L", "^", "2"}]}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"persist", "[", 
     RowBox[{"\[Rho]_", ",", "L_", ",", 
      RowBox[{"\[Sigma]p_:", "\[Sigma]x"}], ",", 
      RowBox[{"\[Sigma]pp_:", "\[Sigma]xx"}]}], "]"}], ":=", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"1", "/", 
        RowBox[{"pcur", "[", 
         RowBox[{"\[Rho]", ",", "\[Sigma]pp"}], "]"}]}], "+", 
       RowBox[{
        RowBox[{
         RowBox[{"dth", "[", 
          RowBox[{"\[Rho]", ",", "L"}], "]"}], "^", "2"}], "/", 
        RowBox[{"savg", "[", 
         RowBox[{"L", ",", 
          RowBox[{"davg", "[", 
           RowBox[{"\[Rho]", ",", "\[Sigma]p", ",", "\[Sigma]pp"}], "]"}]}], 
         "]"}]}]}], ")"}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pr", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.5"}], ",", "1.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1.5"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prplt1", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Rho]0", " ", "Lm", " ", "L", " ", "rd0"}], ",", 
          RowBox[{"persist", "[", 
           RowBox[{"\[Rho]0", ",", "L"}], "]"}]}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"L", ",", "0.01", ",", "26", ",", "0.01"}], "}"}]}], "]"}], 
     ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", "Thick", ",", "Dashed"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prplt2", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Rho]0", " ", "Lm", " ", "L0", " ", "rD"}], ",", 
          RowBox[{"persist", "[", 
           RowBox[{
            RowBox[{"\[Rho]0", "*", "rD"}], ",", "L0", ",", "\[Sigma]x", ",", 
            "\[Sigma]xx"}], " ", "]"}]}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"rD", ",", 
         RowBox[{".001", "/", "1.01"}], ",", 
         RowBox[{"3", "/", "4"}], ",", "0.01"}], "}"}]}], "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", "Thick", ",", "Dotted"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prplt3", "=", 
   RowBox[{"ListPlot", "[", "  ", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"Log10", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Rho]", " ", "Lm", " ", "L0", " ", "rd0"}], ",", 
          RowBox[{"persist", "[", 
           RowBox[{"\[Rho]", ",", "L0"}], "]"}]}], "}"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Rho]", ",", "0.001", ",", "6", ",", "0.01"}], "}"}]}], 
      "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Green", ",", "Thick"}], "}"}]}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "pr"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotMarkers", "\[Rule]", "None"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6908289518540154`*^9, 3.690829183326593*^9}, {
   3.690829215686035*^9, 3.690829256532992*^9}, {3.6908292891965857`*^9, 
   3.690829339562786*^9}, 3.690829482351533*^9, {3.690829653475766*^9, 
   3.690829832973675*^9}, {3.690831152381*^9, 3.690831165446589*^9}, {
   3.6908312622101183`*^9, 3.690831288865958*^9}, {3.693914563085428*^9, 
   3.693914566692687*^9}, {3.6939147316493063`*^9, 3.693914731806114*^9}, {
   3.693914790903183*^9, 3.693914798238636*^9}, {3.693915167932311*^9, 
   3.693915328228868*^9}, {3.693915508799697*^9, 3.6939155385120564`*^9}, {
   3.693915596954473*^9, 3.693915622854863*^9}, {3.693915657071535*^9, 
   3.6939156932023487`*^9}, {3.6939158269263363`*^9, 3.693915830471766*^9}, {
   3.693915887257145*^9, 3.693915887776473*^9}, 3.693923096387932*^9, {
   3.693923138968843*^9, 3.693923143241933*^9}, {3.693923176283086*^9, 
   3.693923267229465*^9}, {3.693925791938081*^9, 3.69392581235466*^9}, {
   3.693926327318202*^9, 3.693926371493339*^9}, {3.693926412268839*^9, 
   3.6939264530975657`*^9}, {3.6939264838427267`*^9, 3.693926523081642*^9}, 
   3.694829339946741*^9, {3.694968659795073*^9, 3.6949686719262543`*^9}, 
   3.702662762772193*^9, {3.7028195459793262`*^9, 3.702819585728086*^9}, {
   3.7028196693284597`*^9, 3.702819695288974*^9}, {3.702819746269766*^9, 
   3.702819792299273*^9}, {3.702819845490922*^9, 3.702819913417247*^9}, {
   3.702819990414629*^9, 3.7028200182212963`*^9}, {3.7028287249726152`*^9, 
   3.702828742684601*^9}, {3.702828799237197*^9, 3.70282880035021*^9}, {
   3.70282886152954*^9, 3.702828862462503*^9}, 3.704815338187283*^9, {
   3.7048156527067833`*^9, 3.704815688198893*^9}, {3.707584699392398*^9, 
   3.70758470662276*^9}}],

Cell[BoxData[
 RowBox[{"scplt", "=", 
  RowBox[{"Show", "[", 
   RowBox[{"{", 
    RowBox[{"dplt", ",", "prplt1", ",", "prplt2", ",", "prplt3"}], "}"}], 
   "]"}], " ", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
    "The", " ", "\"\<prplt\>\"", "s", " ", "should", " ", "have", " ", "been",
      " ", "computed", " ", "above"}], ",", " ", 
    RowBox[{"in", " ", "Fig6"}]}], "*)"}]}]], "Input",
 CellChangeTimes->{{3.6933237544373417`*^9, 3.693323762063835*^9}, {
  3.693324057248186*^9, 3.693324060480835*^9}, {3.6933242128517303`*^9, 
  3.693324214025663*^9}, {3.6933246167277737`*^9, 3.693324621526331*^9}, {
  3.693325074571287*^9, 3.693325076241613*^9}, {3.693915369438003*^9, 
  3.693915370399345*^9}, {3.6948948727764*^9, 3.6948948999598103`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Export", "[", 
   RowBox[{
    RowBox[{"figdir", "<>", "\"\</S5D.eps\>\""}], ",", "scplt"}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.693928121831173*^9, 3.693928187700358*^9}, {
  3.694176217351467*^9, 3.694176219511282*^9}, {3.6948083792368803`*^9, 
  3.694808412069407*^9}, {3.6948949669500113`*^9, 3.6948949697019157`*^9}}]
}, Closed]]
}, Closed]]
}, Closed]]
},
WindowSize->{1335, 1158},
WindowMargins->{{Automatic, 583}, {11, Automatic}},
FrontEndVersion->"10.2 for Linux x86 (64-bit) (July 29, 2015)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 102, 1, 67, "Section"],
Cell[685, 25, 1913, 37, 391, "Text"],
Cell[2601, 64, 379, 8, 32, "Input"],
Cell[CellGroupData[{
Cell[3005, 76, 156, 2, 47, "Subsection"],
Cell[3164, 80, 3974, 88, 407, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[7187, 174, 93, 1, 53, "Section"],
Cell[CellGroupData[{
Cell[7305, 179, 89, 1, 47, "Subsection"],
Cell[7397, 182, 302, 7, 77, "Text"],
Cell[7702, 191, 1636, 39, 121, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[9375, 235, 91, 1, 47, "Subsection"],
Cell[9469, 238, 396, 9, 99, "Text"],
Cell[9868, 249, 1748, 44, 143, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[11665, 299, 144, 2, 53, "Section"],
Cell[11812, 303, 796, 17, 187, "Text"],
Cell[CellGroupData[{
Cell[12633, 324, 91, 1, 47, "Subsection"],
Cell[12727, 327, 383, 10, 99, "Input"],
Cell[CellGroupData[{
Cell[13135, 341, 100, 1, 37, "Subsubsection"],
Cell[13238, 344, 2690, 69, 143, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[15965, 418, 101, 1, 37, "Subsubsection"],
Cell[16069, 421, 2442, 66, 165, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[18548, 492, 102, 1, 37, "Subsubsection"],
Cell[18653, 495, 2070, 61, 143, "Input"],
Cell[20726, 558, 1118, 27, 99, "Input"],
Cell[21847, 587, 6918, 155, 253, "Input"],
Cell[28768, 744, 224, 6, 32, "Input"],
Cell[28995, 752, 69, 1, 32, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[29113, 759, 88, 1, 47, "Subsection"],
Cell[29204, 762, 1123, 27, 103, "Input"],
Cell[CellGroupData[{
Cell[30352, 793, 103, 1, 37, "Subsubsection"],
Cell[30458, 796, 1678, 46, 143, "Input"],
Cell[32139, 844, 2633, 65, 231, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[34809, 914, 104, 1, 37, "Subsubsection"],
Cell[34916, 917, 2215, 60, 143, "Input"],
Cell[37134, 979, 2972, 73, 209, "Input"],
Cell[40109, 1054, 770, 22, 32, "Input"],
Cell[40882, 1078, 441, 12, 32, "Input"],
Cell[41326, 1092, 322, 8, 32, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[41709, 1107, 198, 3, 53, "Section"],
Cell[CellGroupData[{
Cell[41932, 1114, 91, 1, 47, "Subsection"],
Cell[42026, 1117, 1013, 21, 143, "Input"],
Cell[43042, 1140, 1781, 43, 143, "Input"],
Cell[44826, 1185, 1010, 22, 77, "Input"],
Cell[45839, 1209, 928, 21, 77, "Input"],
Cell[46770, 1232, 282, 7, 32, "Input"],
Cell[47055, 1241, 15449, 376, 1265, "Input"],
Cell[62507, 1619, 333, 6, 32, "Input"],
Cell[62843, 1627, 725, 16, 55, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[63605, 1648, 191, 3, 47, "Subsection"],
Cell[CellGroupData[{
Cell[63821, 1655, 2217, 64, 209, "Input"],
Cell[66041, 1721, 79, 1, 32, "Output"]
}, Closed]],
Cell[CellGroupData[{
Cell[66157, 1727, 94, 1, 37, "Subsubsection"],
Cell[66254, 1730, 601, 16, 32, "Input"],
Cell[66858, 1748, 1618, 38, 77, "Input"],
Cell[68479, 1788, 270, 7, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[68786, 1800, 94, 1, 37, "Subsubsection"],
Cell[68883, 1803, 1570, 33, 55, "Input"],
Cell[70456, 1838, 219, 6, 32, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[70724, 1850, 119, 2, 47, "Subsection"],
Cell[70846, 1854, 891, 23, 99, "Input"],
Cell[CellGroupData[{
Cell[71762, 1881, 112, 1, 37, "Subsubsection"],
Cell[71877, 1884, 2322, 56, 231, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[74236, 1945, 134, 1, 37, "Subsubsection"],
Cell[74373, 1948, 407, 10, 32, "Input"],
Cell[74783, 1960, 1991, 50, 165, "Input"],
Cell[76777, 2012, 499, 13, 99, "Input"],
Cell[77279, 2027, 345, 9, 55, "Input"],
Cell[77627, 2038, 514, 13, 32, "Input"],
Cell[78144, 2053, 9984, 257, 649, "Input"],
Cell[88131, 2312, 100, 1, 32, "Input"],
Cell[88234, 2315, 469, 10, 32, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[88764, 2332, 93, 1, 53, "Section"],
Cell[88860, 2335, 1116, 31, 275, "Input"],
Cell[CellGroupData[{
Cell[90001, 2370, 117, 2, 47, "Subsection"],
Cell[90121, 2374, 1501, 34, 77, "Input"],
Cell[91625, 2410, 2056, 57, 99, "Input"],
Cell[93684, 2469, 5000, 123, 364, "Input"],
Cell[98687, 2594, 99, 1, 32, "Input"],
Cell[98789, 2597, 276, 7, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[99102, 2609, 163, 2, 47, "Subsection"],
Cell[99268, 2613, 1675, 39, 77, "Input"],
Cell[100946, 2654, 2120, 59, 99, "Input"],
Cell[103069, 2715, 4938, 126, 430, "Input"],
Cell[108010, 2843, 99, 1, 32, "Input"],
Cell[108112, 2846, 448, 9, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[108597, 2860, 217, 3, 47, "Subsection"],
Cell[108817, 2865, 3884, 101, 187, "Input"],
Cell[112704, 2968, 4762, 119, 430, "Input"],
Cell[117469, 3089, 101, 1, 32, "Input"],
Cell[117573, 3092, 350, 9, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[117960, 3106, 188, 3, 47, "Subsection"],
Cell[118151, 3111, 1940, 44, 77, "Input"],
Cell[120094, 3157, 2252, 61, 99, "Input"],
Cell[122349, 3220, 5418, 137, 452, "Input"],
Cell[127770, 3359, 419, 9, 32, "Input"],
Cell[128192, 3370, 101, 1, 32, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[128342, 3377, 246, 3, 53, "Section"],
Cell[128591, 3382, 897, 30, 143, "Input"],
Cell[CellGroupData[{
Cell[129513, 3416, 88, 1, 47, "Subsection"],
Cell[129604, 3419, 1360, 32, 121, "Input"],
Cell[130967, 3453, 2679, 60, 99, "Input"],
Cell[133649, 3515, 726, 20, 77, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[134412, 3540, 91, 1, 47, "Subsection"],
Cell[134506, 3543, 11280, 301, 1067, "Input"],
Cell[145789, 3846, 2427, 64, 275, "Input"],
Cell[148219, 3912, 2404, 55, 209, "Input"],
Cell[150626, 3969, 415, 9, 32, "Input"],
Cell[151044, 3980, 272, 7, 32, "Input"],
Cell[151319, 3989, 688, 21, 99, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[152044, 4015, 89, 1, 47, "Subsection"],
Cell[152136, 4018, 2817, 84, 231, "Input"],
Cell[154956, 4104, 1209, 26, 77, "Input"],
Cell[156168, 4132, 3197, 79, 319, "Input"],
Cell[159368, 4213, 95, 1, 32, "Input"],
Cell[159466, 4216, 361, 8, 32, "Input"],
Cell[159830, 4226, 222, 6, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[160089, 4237, 163, 2, 47, "Subsection"],
Cell[160255, 4241, 2746, 69, 231, "Input"],
Cell[163004, 4312, 316, 8, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[163357, 4325, 251, 3, 47, "Subsection"],
Cell[163611, 4330, 1061, 36, 77, "Input"],
Cell[164675, 4368, 3305, 88, 163, "Input"],
Cell[167983, 4458, 264, 6, 32, "Input"],
Cell[168250, 4466, 714, 19, 77, "Input"],
Cell[168967, 4487, 3623, 95, 206, "Input"],
Cell[172593, 4584, 305, 7, 32, "Input"],
Cell[172901, 4593, 759, 20, 77, "Input"],
Cell[173663, 4615, 405, 9, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[174105, 4629, 191, 3, 47, "Subsection"],
Cell[174299, 4634, 6077, 179, 341, "Input"],
Cell[180379, 4815, 1635, 42, 231, "Input"],
Cell[182017, 4859, 323, 8, 32, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[182389, 4873, 140, 2, 53, "Section"],
Cell[182532, 4877, 2376, 51, 253, "Input"],
Cell[CellGroupData[{
Cell[184933, 4932, 93, 1, 47, "Subsection"],
Cell[185029, 4935, 5527, 141, 429, "Input"],
Cell[190559, 5078, 98, 1, 32, "Input"],
Cell[190660, 5081, 631, 14, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[191328, 5100, 88, 1, 47, "Subsection"],
Cell[CellGroupData[{
Cell[191441, 5105, 98, 1, 37, "Subsubsection"],
Cell[191542, 5108, 9121, 256, 583, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[200700, 5369, 100, 1, 37, "Subsubsection"],
Cell[200803, 5372, 5195, 140, 385, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[206035, 5517, 101, 1, 37, "Subsubsection"],
Cell[206139, 5520, 6454, 170, 451, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[212630, 5695, 103, 1, 37, "Subsubsection"],
Cell[212736, 5698, 823, 27, 121, "Input"],
Cell[213562, 5727, 10006, 246, 503, "Input"],
Cell[223571, 5975, 148, 2, 32, "Input"],
Cell[223722, 5979, 1292, 23, 32, "Input"],
Cell[225017, 6004, 1055, 17, 32, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[226121, 6027, 161, 2, 47, "Subsection"],
Cell[226285, 6031, 622, 8, 32, "Input"],
Cell[226910, 6041, 2483, 70, 165, "Input"],
Cell[229396, 6113, 966, 27, 99, "Input"],
Cell[230365, 6142, 3280, 91, 143, "Input"],
Cell[233648, 6235, 4949, 107, 313, "Input"],
Cell[238600, 6344, 499, 10, 32, "Input"],
Cell[239102, 6356, 1222, 27, 55, "Input"],
Cell[240327, 6385, 218, 6, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[240582, 6396, 89, 1, 47, "Subsection"],
Cell[240674, 6399, 300, 7, 32, "Input"],
Cell[CellGroupData[{
Cell[240999, 6410, 101, 1, 37, "Subsubsection"],
Cell[241103, 6413, 3670, 90, 253, "Input"],
Cell[244776, 6505, 1240, 39, 121, "Input"],
Cell[246019, 6546, 399, 13, 32, "Input"],
Cell[246421, 6561, 3192, 89, 231, "Input"],
Cell[249616, 6652, 1467, 42, 99, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[251120, 6699, 217, 3, 37, "Subsubsection"],
Cell[251340, 6704, 734, 19, 77, "Input"],
Cell[252077, 6725, 10348, 281, 715, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[262462, 7011, 115, 1, 37, "Subsubsection"],
Cell[262580, 7014, 5708, 135, 429, "Input"],
Cell[268291, 7151, 489, 10, 32, "Input"],
Cell[268783, 7163, 396, 9, 32, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[269240, 7179, 242, 3, 53, "Section"],
Cell[CellGroupData[{
Cell[269507, 7186, 91, 1, 47, "Subsection"],
Cell[269601, 7189, 832, 19, 77, "Input"],
Cell[CellGroupData[{
Cell[270458, 7212, 92, 1, 37, "Subsubsection"],
Cell[270553, 7215, 953, 30, 99, "Input"],
Cell[271509, 7247, 1900, 42, 77, "Input"],
Cell[273412, 7291, 369, 10, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[273818, 7306, 94, 1, 37, "Subsubsection"],
Cell[273915, 7309, 3520, 91, 297, "Input"],
Cell[277438, 7402, 461, 14, 55, "Input"],
Cell[277902, 7418, 1906, 43, 99, "Input"],
Cell[279811, 7463, 318, 8, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[280166, 7476, 94, 1, 37, "Subsubsection"],
Cell[280263, 7479, 639, 10, 55, "Text"],
Cell[280905, 7491, 4552, 123, 253, "Input"],
Cell[285460, 7616, 589, 12, 32, "Input"],
Cell[286052, 7630, 91, 1, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[286180, 7636, 215, 3, 37, "Subsubsection"],
Cell[286398, 7641, 813, 23, 99, "Input"],
Cell[287214, 7666, 863, 22, 55, "Input"],
Cell[288080, 7690, 137, 3, 32, "Input"],
Cell[288220, 7695, 1373, 39, 77, "Input"],
Cell[289596, 7736, 2144, 61, 121, "Input"],
Cell[291743, 7799, 860, 23, 99, "Input"],
Cell[292606, 7824, 7674, 193, 707, "Input"],
Cell[300283, 8019, 450, 9, 32, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[300782, 8034, 288, 4, 47, "Subsection"],
Cell[301073, 8040, 1000, 23, 99, "Input"],
Cell[CellGroupData[{
Cell[302098, 8067, 92, 1, 37, "Subsubsection"],
Cell[302193, 8070, 953, 30, 99, "Input"],
Cell[303149, 8102, 1951, 43, 77, "Input"],
Cell[305103, 8147, 346, 9, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[305486, 8161, 94, 1, 37, "Subsubsection"],
Cell[305583, 8164, 3523, 91, 297, "Input"],
Cell[309109, 8257, 461, 14, 55, "Input"],
Cell[309573, 8273, 1906, 43, 99, "Input"],
Cell[311482, 8318, 318, 8, 32, "Input"],
Cell[311803, 8328, 227, 6, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[312067, 8339, 94, 1, 37, "Subsubsection"],
Cell[312164, 8342, 663, 11, 55, "Text"],
Cell[312830, 8355, 4552, 123, 253, "Input"],
Cell[317385, 8480, 589, 12, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[318011, 8497, 215, 3, 37, "Subsubsection"],
Cell[318229, 8502, 888, 24, 121, "Input"],
Cell[319120, 8528, 234, 6, 32, "Input"],
Cell[319357, 8536, 1373, 39, 77, "Input"],
Cell[320733, 8577, 692, 16, 32, "Input"],
Cell[321428, 8595, 801, 24, 32, "Input"],
Cell[322232, 8621, 1045, 28, 77, "Input"],
Cell[323280, 8651, 967, 25, 99, "Input"],
Cell[324250, 8678, 409, 9, 32, "Input"],
Cell[324662, 8689, 6676, 167, 641, "Input"],
Cell[331341, 8858, 444, 9, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[331822, 8872, 186, 2, 37, "Subsubsection"],
Cell[332011, 8876, 4279, 117, 275, "Input"],
Cell[336293, 8995, 1397, 34, 77, "Input"],
Cell[337693, 9031, 231, 5, 32, "Input"],
Cell[337927, 9038, 92, 1, 32, "Input"],
Cell[338022, 9041, 437, 11, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[338496, 9057, 120, 2, 37, "Subsubsection"],
Cell[338619, 9061, 622, 14, 55, "Input"],
Cell[339244, 9077, 3216, 97, 121, "Input"],
Cell[342463, 9176, 148, 3, 32, "Input"],
Cell[342614, 9181, 260, 7, 32, "Input"],
Cell[342877, 9190, 3559, 68, 213, "Input"],
Cell[346439, 9260, 423, 9, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[346899, 9274, 166, 2, 37, "Subsubsection"],
Cell[347068, 9278, 605, 13, 55, "Input"],
Cell[347676, 9293, 517, 14, 32, "Input"],
Cell[CellGroupData[{
Cell[348218, 9311, 131, 2, 32, "Input"],
Cell[348352, 9315, 140, 3, 32, "Output"]
}, Closed]],
Cell[348507, 9321, 1984, 62, 99, "Input"],
Cell[350494, 9385, 519, 14, 32, "Input"],
Cell[351016, 9401, 1215, 34, 77, "Input"],
Cell[352234, 9437, 90, 1, 32, "Input"],
Cell[352327, 9440, 180, 4, 32, "Input"],
Cell[352510, 9446, 2985, 60, 201, "Input"],
Cell[355498, 9508, 369, 8, 32, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[355928, 9523, 87, 1, 53, "Section"],
Cell[356018, 9526, 1129, 30, 253, "Input"],
Cell[CellGroupData[{
Cell[357172, 9560, 91, 1, 47, "Subsection"],
Cell[357266, 9563, 2693, 82, 363, "Input"],
Cell[359962, 9647, 9078, 213, 759, "Input"],
Cell[369043, 9862, 9026, 214, 715, "Input"],
Cell[378072, 10078, 292, 5, 32, "Input"],
Cell[378367, 10085, 487, 13, 55, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[378891, 10103, 117, 2, 47, "Subsection"],
Cell[379011, 10107, 1548, 35, 77, "Input"],
Cell[380562, 10144, 1505, 48, 99, "Input"],
Cell[382070, 10194, 3043, 70, 254, "Input"],
Cell[385116, 10266, 99, 1, 32, "Input"],
Cell[385218, 10269, 322, 8, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[385577, 10282, 117, 2, 47, "Subsection"],
Cell[CellGroupData[{
Cell[385719, 10288, 93, 1, 37, "Subsubsection"],
Cell[CellGroupData[{
Cell[385837, 10293, 4011, 108, 231, "Input"],
Cell[389851, 10403, 77, 1, 32, "Output"],
Cell[389931, 10406, 79, 1, 32, "Output"]
}, Closed]],
Cell[390025, 10410, 232, 5, 55, "Input"],
Cell[390260, 10417, 442, 13, 55, "Input"],
Cell[390705, 10432, 327, 10, 32, "Input"],
Cell[391035, 10444, 264, 7, 32, "Input"],
Cell[391302, 10453, 384, 11, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[391723, 10469, 95, 1, 37, "Subsubsection"],
Cell[391821, 10472, 4014, 103, 209, "Input"],
Cell[395838, 10577, 137, 3, 32, "Input"],
Cell[395978, 10582, 6794, 165, 496, "Input"],
Cell[402775, 10749, 99, 1, 32, "Input"],
Cell[402877, 10752, 342, 8, 32, "Input"],
Cell[403222, 10762, 200, 6, 32, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[403483, 10775, 138, 2, 53, "Section"],
Cell[403624, 10779, 360, 10, 121, "Input"],
Cell[CellGroupData[{
Cell[404009, 10793, 93, 1, 47, "Subsection"],
Cell[404105, 10796, 4509, 136, 451, "Input"],
Cell[408617, 10934, 319, 8, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[408973, 10947, 88, 1, 47, "Subsection"],
Cell[CellGroupData[{
Cell[409086, 10952, 98, 1, 37, "Subsubsection"],
Cell[409187, 10955, 1432, 39, 121, "Input"],
Cell[410622, 10996, 4439, 147, 385, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[415098, 11148, 100, 1, 37, "Subsubsection"],
Cell[415201, 11151, 901, 26, 121, "Input"],
Cell[416105, 11179, 5104, 169, 341, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[421246, 11353, 101, 1, 37, "Subsubsection"],
Cell[421350, 11356, 6557, 212, 473, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[427944, 11573, 154, 2, 37, "Subsubsection"],
Cell[428101, 11577, 944, 18, 121, "Input"],
Cell[429048, 11597, 2022, 41, 77, "Input"],
Cell[431073, 11640, 8084, 220, 503, "Input"],
Cell[439160, 11862, 93, 1, 32, "Input"],
Cell[439256, 11865, 4044, 101, 217, "Input"],
Cell[443303, 11968, 5221, 107, 143, "Input"],
Cell[448527, 12077, 968, 19, 32, "Input"],
Cell[449498, 12098, 96, 1, 32, "Input"],
Cell[449597, 12101, 374, 8, 32, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[450020, 12115, 161, 2, 47, "Subsection"],
Cell[450184, 12119, 2001, 51, 77, "Input"],
Cell[452188, 12172, 68, 1, 32, "Input"],
Cell[452259, 12175, 548, 16, 77, "Input"],
Cell[452810, 12193, 184, 4, 32, "Input"],
Cell[452997, 12199, 563, 16, 77, "Input"],
Cell[453563, 12217, 3788, 105, 143, "Input"],
Cell[457354, 12324, 5937, 133, 335, "Input"],
Cell[463294, 12459, 424, 9, 32, "Input"],
Cell[463721, 12470, 1254, 28, 55, "Input"],
Cell[464978, 12500, 272, 7, 32, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[465287, 12512, 91, 1, 47, "Subsection"],
Cell[465381, 12515, 3955, 107, 253, "Input"],
Cell[469339, 12624, 1594, 52, 121, "Input"],
Cell[470936, 12678, 2642, 75, 187, "Input"],
Cell[473581, 12755, 4831, 132, 429, "Input"],
Cell[CellGroupData[{
Cell[478437, 12891, 217, 3, 37, "Subsubsection"],
Cell[478657, 12896, 9316, 255, 627, "Input"],
Cell[487976, 13153, 755, 16, 32, "Input"],
Cell[488734, 13171, 370, 8, 32, "Input"]
}, Closed]]
}, Closed]]
}, Closed]]
}
]
*)

(* End of internal cache information *)
